@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -40,7 +35,7 @@
  * Interface to directory handling routines
  */
 
-/* $Id: adir.h,v 1.11 1999/06/20 03:53:07 assar Exp $ */
+/* $Id: adir.h,v 1.14 2000/10/02 22:30:01 lha Exp $ */
 
 #ifndef _ADIR_H_
 #define _ADIR_H_
@@ -50,28 +45,28 @@
 #include "fcache.h"
 #include "fdir.h"
 
-int adir_lookup (VenusFid dir, const char *name, VenusFid *file,
-		 CredCacheEntry *ce);
+int adir_lookup (VenusFid *dir, const char *name, VenusFid *file,
+		 FCacheEntry **dentry, CredCacheEntry **ce);
 
 int
-adir_lookup_fcacheentry (FCacheEntry *centry,  VenusFid dir,
+adir_lookup_fcacheentry (FCacheEntry *centry, VenusFid dir,
 			 const char *name,
 			 VenusFid *file, CredCacheEntry *ce);
 
 int
-adir_changefid (VenusFid dir,
+adir_changefid (VenusFid *dir,
 		const char *name,
 		VenusFid *file,
-		CredCacheEntry *ce);
+		CredCacheEntry **ce);
 
 int
-adir_emptyp (VenusFid dir,
-	     CredCacheEntry *ce);
+adir_emptyp (VenusFid *dir,
+	     CredCacheEntry **ce);
 
-int adir_readdir (VenusFid dir,
+int adir_readdir (VenusFid *dir,
 		  void (*)(VenusFid *, const char *name, void *), 
 		  void *arg,
-		  CredCacheEntry *ce);
+		  CredCacheEntry **ce);
 
 int
 adir_creat (FCacheEntry *dir,
