@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -36,52 +31,11 @@
  * SUCH DAMAGE.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-#include <assert.h>
-#include <stdarg.h>
-#include <stdlib.h>
-#include <parse_units.h>
-#include <roken.h>
-#include "ko.h"
-#include "log.h"
+#include <arla_local.h>
+RCSID("$Id: arladeb.c,v 1.24 2000/10/02 22:30:28 lha Exp $");
 
-#include "arladeb.h"
-
-RCSID("$Id: arladeb.c,v 1.17 1999/08/03 23:23:23 assar Exp $");
-
-static Log_method* arla_log_method = NULL;
-
-#define all (ADEBERROR | ADEBWARN | ADEBDISCONN | ADEBFBUF |		\
-	     ADEBMSG | ADEBKERNEL | ADEBCLEANER | ADEBCALLBACK |	\
-	     ADEBCM | ADEBVOLCACHE | ADEBFCACHE | ADEBINIT |		\
-	     ADEBCONN | ADEBMISC | ADEBVLOG)
-
-#define DEFAULT_LOG (ADEBWARN | ADEBERROR)
-
-struct units arla_deb_units[] = {
-    { "all",		all},
-    { "almost-all",	all & ~ADEBCLEANER},
-    { "errors",		ADEBERROR },
-    { "warnings",	ADEBWARN },
-    { "disconn",	ADEBDISCONN },
-    { "fbuf",		ADEBFBUF },
-    { "messages",	ADEBMSG },
-    { "kernel",		ADEBKERNEL },
-    { "cleaner",	ADEBCLEANER },
-    { "callbacks",	ADEBCALLBACK },
-    { "cache-manager",	ADEBCM },
-    { "volume-cache",	ADEBVOLCACHE },
-    { "file-cache",	ADEBFCACHE },
-    { "initialization",	ADEBINIT },
-    { "connection",	ADEBCONN },
-    { "miscellaneous",	ADEBMISC },
-    { "venuslog",	ADEBVLOG },
-    { "default",	DEFAULT_LOG },
-    { "none",		0 },
-    { NULL }
-};
+Log_method* arla_log_method = NULL;
+Log_unit* arla_log_unit = NULL;
 
 void
 arla_log(unsigned level, char *fmt, ...)
@@ -91,56 +45,48 @@ arla_log(unsigned level, char *fmt, ...)
     assert (arla_log_method);
     
     va_start(args, fmt);
-    log_vlog(arla_log_method, level, fmt, args);
+    log_vlog(arla_log_unit, level, fmt, args);
     va_end(args);
 }
 
+
 void
 arla_loginit(char *log)
 {
     assert (log);
     
     arla_log_method = log_open("arla", log);
-    log_set_mask(arla_log_method, DEFAULT_LOG);
+    if (arla_log_method == NULL)
+	errx (1, "arla_loginit: log_opened failed with log `%s'", log);
+    arla_log_unit = log_unit_init (arla_log_method, "arla", arla_deb_units,
+				   ARLA_DEFAULT_LOG);
+    if (arla_log_unit == NULL)
+	errx (1, "arla_loginit: log_unit_init failed");
 }
 
 int
 arla_log_set_level (const char *s)
 {
-    int ret;
-
-    ret = parse_flags (s, arla_deb_units, log_get_mask(arla_log_method));
-    if (ret < 0)
-	return ret;
-    else {
-	log_set_mask (arla_log_method, ret);
-	return 0;
-    }
+    log_set_mask_str (arla_log_method, NULL, s);
+    return 0;
 }
 
 void
 arla_log_set_level_num (unsigned level)
 {
-    log_set_mask (arla_log_method, level);
+    log_set_mask (arla_log_unit, level);
 }
 
 void
 arla_log_get_level (char *s, size_t len)
 {
-    unparse_flags (log_get_mask (arla_log_method),
-		   arla_deb_units, s, len);
+    log_mask2str (arla_log_method, NULL, s, len);
 }
 
 unsigned
 arla_log_get_level_num (void)
 {
-    return log_get_mask (arla_log_method);
-}
-
-void
-arla_log_print_levels (FILE *f)
-{
-    print_flags_table (arla_deb_units, f);
+    return log_get_mask (arla_log_unit);
 }
 
 /*
@@ -164,11 +110,11 @@ arla_verr (int eval, unsigned level, int error, const char *fmt, va_list args)
 
     vasprintf (&s, fmt, args);
     if (s == NULL) {
-	log_log (arla_log_method, level,
+	log_log (arla_log_unit, level,
 		 "Sorry, no memory to print `%s'...", fmt);
 	exit (eval);
     }
-    log_log (arla_log_method, level, "%s: %s", s, koerr_gettext (error));
+    log_log (arla_log_unit, level, "%s: %s", s, koerr_gettext (error));
     free (s);
     exit (eval);
 }
@@ -186,7 +132,7 @@ arla_errx (int eval, unsigned level, const char *fmt, ...)
 void
 arla_verrx (int eval, unsigned level, const char *fmt, va_list args)
 {
-    log_vlog (arla_log_method, level, fmt, args);
+    log_vlog (arla_log_unit, level, fmt, args);
     exit (eval);
 }
 
@@ -207,11 +153,11 @@ arla_vwarn (unsigned level, int error, const char *fmt, va_list args)
 
     vasprintf (&s, fmt, args);
     if (s == NULL) {
-	log_log (arla_log_method, level,
+	log_log (arla_log_unit, level,
 		 "Sorry, no memory to print `%s'...", fmt);
 	return;
     }
-    log_log (arla_log_method, level, "%s: %s", s, koerr_gettext (error));
+    log_log (arla_log_unit, level, "%s: %s", s, koerr_gettext (error));
     free (s);
 }
 
@@ -228,5 +174,43 @@ arla_warnx (unsigned level, const char *fmt, ...)
 void
 arla_vwarnx (unsigned level, const char *fmt, va_list args)
 {
-    log_vlog (arla_log_method, level, fmt, args);
+    log_vlog (arla_log_unit, level, fmt, args);
+}
+
+void
+arla_warnx_with_fid (unsigned level, const VenusFid *fid, const char *fmt, ...)
+{
+    va_list args;
+
+    va_start (args, fmt);
+    arla_vwarnx_with_fid (level, fid, fmt, args);
+    va_end(args);
+}
+
+void
+arla_vwarnx_with_fid (unsigned level, const VenusFid *fid, const char *fmt,
+		      va_list args)
+{
+    char *s;
+    const char *cellname;
+    const char *volname;
+
+    cellname = cell_num2name (fid->Cell);
+    if (cellname == NULL)
+	cellname = "<unknown>";
+    volname = volcache_getname (fid->fid.Volume, fid->Cell);
+    if (volname == NULL)
+	volname = "<unknown>";
+
+    vasprintf (&s, fmt, args);
+    if (s == NULL) {
+	log_log (arla_log_unit, level,
+		 "Sorry, no memory to print `%s'...", fmt);
+	return;
+    }
+    log_log (arla_log_unit, level,
+	     "volume %s (%ld) in cell %s (%ld): %s",
+	     volname, (unsigned long)fid->fid.Volume, cellname,
+	     (long)fid->Cell, s);
+    free (s);
 }
