@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995-2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -40,7 +35,7 @@
  * The interface for the file-cache.
  */
 
-/* $Id: fcache.h,v 1.55 1999/07/25 00:09:53 lha Exp $ */
+/* $Id: fcache.h,v 1.72 2000/11/23 08:58:08 lha Exp $ */
 
 #ifndef _FCACHE_H_
 #define _FCACHE_H_
@@ -60,7 +55,7 @@
 #define NACCESS MAXRIGHTS
 
 typedef struct {
-     pag_t cred;
+     xfs_pag_t cred;
      u_long access;
 } AccessEntry;
 
@@ -73,30 +68,24 @@ enum Access { ANONE   = 0x0,
 	      ALOCK   = 0x20,
 	      AADMIN  = 0x40 };
 
-/*
- * Keep the necessary state with traversing mount points.
- * Might want to make this per-user/per-process in some distant future.
- */
-
-struct mp_traversal {
-    VenusFid parent;		/* the point where the traversal started */
-    unsigned refcount;
-};
-
-typedef struct mp_traversal mp_traversal;
+typedef struct {
+    Bool valid;
+    xfs_cache_handle xfs_handle;
+} fcache_cache_handle;
 
 typedef struct {
     struct Lock lock;		/* locking information for this entry */
     VenusFid fid;		/* The fid of the file for this entry */
     unsigned refcount;		/* reference count */
     u_int32_t host;		/* the source of this entry */
+    size_t length;		/* length of our cached copy */
     AFSFetchStatus status;	/* Removed unused stuff later */
     AFSCallBack callback;	/* Callback to the AFS-server */
     AFSVolSync volsync;		/* Sync info for ro-volumes */
     AccessEntry acccache[NACCESS]; /* cache for the access rights */
     u_int32_t anonaccess;	/* the access mask for system:anyuser */
     unsigned index;		/* this is V%u */
-    xfs_cache_handle handle;	/* handle */
+    fcache_cache_handle handle;	/* handle */
     struct {
 	unsigned usedp : 1;	/* Is this entry used? */
 	unsigned attrp : 1;	/* Are the attributes in status valid? */
@@ -108,17 +97,17 @@ typedef struct {
 	unsigned kernelp : 1;	/* Does this entry exist in the kernel? */
 	unsigned sentenced : 1;	/* This entry should die */
 	unsigned silly : 1;	/* Instead of silly-rename */
+	unsigned fake_mp : 1;	/* a `fake' mount point */
+	unsigned vol_root : 1;	/* root of a volume */
     } flags;
     u_int tokens;		/* read/write tokens for the kernel */
-#if 0
-    mp_traversal *mp_traversal;	/* valid iff flags.mountp */
-#endif
     VenusFid parent;
-    VenusFid realfid;		/* Real fid (mountpoints) */
     Listitem *lru_le;		/* lru */
     heap_ptr invalid_ptr;	/* pointer into the heap */
     VolCacheEntry *volume;	/* pointer to the volume entry */
-    unsigned priority;		/* the priority of keeping the file 0-100 */
+    Bool priority;		/* is the file worth keeping */
+    int hits;			/* number of lookups */
+    int cleanergen;		/* generation cleaner */
 } FCacheEntry;
 
 /*
@@ -133,11 +122,25 @@ struct fs_server_context {
 
 typedef struct fs_server_context fs_server_context;
 
+/*
+ * This is magic cookie for the dump of the fcache.
+ * It's supposed not to be able to be confused with an old-style
+ * dump (with no header)
+ */
+
+#define FCACHE_MAGIC_COOKIE	0xff1201ff
+
+/*
+ * current version number of the dump file
+ */
+
+#define FCACHE_VERSION		0x2
+
 /*
  * How far the cleaner will go went cleaning things up.
  */
 
-extern unsigned fprioritylevel;
+extern Bool fprioritylevel;
 
 void
 fcache_init (u_long alowvnodes,
@@ -159,7 +162,7 @@ void
 fcache_purge_host (u_long host);
 
 void
-fcache_purge_cred (pag_t cred, int32_t cell);
+fcache_purge_cred (xfs_pag_t cred, int32_t cell);
 
 void
 fcache_stale_entry (VenusFid fid, AFSCallBack callback);
@@ -183,7 +186,7 @@ int
 fcache_open_extra_dir (FCacheEntry *entry, int flag, mode_t mode);
 
 int
-fcache_fhget (char *filename, xfs_cache_handle *handle);
+fcache_fhget (char *filename, fcache_cache_handle *handle);
 
 int
 write_data (FCacheEntry *entry, AFSStoreStatus *status, CredCacheEntry *ce);
@@ -246,13 +249,18 @@ int
 fcache_find (FCacheEntry **res, VenusFid fid);
 
 int
-fcache_get_attr (FCacheEntry *e, CredCacheEntry *ce);
+fcache_get_data (FCacheEntry **res, VenusFid *fid, CredCacheEntry **ce);
 
 int
-fcache_get_data (FCacheEntry *e, CredCacheEntry *ce);
+fcache_verify_attr (FCacheEntry *entry, FCacheEntry *parent_entry,
+		    const char *prefered_name, CredCacheEntry* ce);
 
 int
-followmountpoint (VenusFid *fid, VenusFid *parent, CredCacheEntry **ce);
+fcache_verify_data (FCacheEntry *e, CredCacheEntry *ce);
+
+int
+followmountpoint (VenusFid *fid, const VenusFid *parent, FCacheEntry *parent_e,
+		  CredCacheEntry **ce);
 
 void
 fcache_status (void);
@@ -266,7 +274,7 @@ getacl(VenusFid fid, CredCacheEntry *ce,
 
 int
 setacl(VenusFid fid, CredCacheEntry *ce,
-       AFSOpaque *opaque);
+       AFSOpaque *opaque, FCacheEntry **ret);
 
 int
 getvolstat(VenusFid fid, CredCacheEntry *ce,
@@ -294,6 +302,12 @@ fcache_highvnodes(void);
 u_long
 fcache_usedvnodes(void);
 
+int
+fcache_need_bytes(u_long needed);
+
+Bool
+fcache_need_nodes (void);
+
 int
 fcache_giveup_all_callbacks (void);
 
@@ -305,7 +319,7 @@ int
 read_attr (FCacheEntry *, CredCacheEntry *);
 
 Bool
-findaccess (pag_t cred, AccessEntry *ae, AccessEntry **pos);
+findaccess (xfs_pag_t cred, AccessEntry *ae, AccessEntry **pos);
 
 void
 fcache_unused(FCacheEntry *entry);
@@ -319,7 +333,9 @@ find_first_fs (FCacheEntry *e,
 	       fs_server_context *context);
 
 ConnCacheEntry *
-find_next_fs (fs_server_context *context, ConnCacheEntry *prev_conn);
+find_next_fs (fs_server_context *context,
+	      ConnCacheEntry *prev_conn,
+	      int mark_as_dead);
 
 void
 free_fs_server_context (fs_server_context *context);
@@ -331,10 +347,16 @@ void
 recon_hashtabdel(FCacheEntry *entry);
 
 int
-fcache_get_fbuf (FCacheEntry *centry, fbuf *fbuf,
+fcache_get_fbuf (FCacheEntry *centry, int *fd, fbuf *fbuf,
 		 int open_flags, int fbuf_flags);
 
 u_long
 fcache_calculate_usage (void);
 
+const VenusFid *
+fcache_realfid (const FCacheEntry *entry);
+
+void
+fcache_mark_as_mountpoint (FCacheEntry *entry);
+
 #endif /* _FCACHE_H_ */
