@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -55,12 +50,13 @@
 
 #include "resolve.h"
 
-RCSID("$Id: kocell.c,v 1.24 1999/07/06 17:27:21 lha Exp $");
+RCSID("$Id: kocell.c,v 1.47 2000/12/21 22:07:43 lha Exp $");
 
 #define TRANSARCSYSCONFDIR "/usr/vice/etc"
 #define CELLFILENAME "CellServDB"
 #define THISCELLFILENAME "ThisCell"
 #define SUIDCELLSFILENAME "SuidCells"
+#define THESECELLFILENAME "TheseCells"
 #define DEFCELLCACHESIZE 499
 
 /*
@@ -74,6 +70,34 @@ static Hashtab *cellnamehtab, *cellnumhtab;
  */
 
 static char *thiscell = NULL;
+static char **thesecells = NULL;
+static int numthesecells = 1;
+
+enum { CELL_INVALID_HOST = 1 };
+
+/*
+ * Logging
+ */
+
+#define CDEBERR		0x800
+#define CDEBWARN	0x400
+#define CDEBDNS		0x200
+
+#define CDEBDEFAULT	(CDEBWARN|CDEBERR)
+#define cdeball		(CDEBWARN|CDEBERR)
+
+static struct units celldebug_units[]  = {
+    { "all", 		cdeball },
+    { "almost-all",	cdeball },
+    { "warn",		CDEBWARN },
+    { "error", 		CDEBERR },
+    { "dns",		CDEBDNS },
+    { NULL, 0 },
+};
+
+#undef cdeball
+
+static Log_unit *cell_log = NULL;
 
 /*
  *
@@ -93,7 +117,7 @@ cellnamecmp (void *a, void *b)
      cell_entry *c1 = (cell_entry *)a;
      cell_entry *c2 = (cell_entry *)b;
 
-     return strcmp (c1->name, c2->name);
+     return strcasecmp (c1->name, c2->name);
 }
 
 static unsigned
@@ -101,7 +125,7 @@ cellnamehash (void *a)
 {
      cell_entry *c = (cell_entry *)a;
 
-     return hashadd (c->name);
+     return hashcaseadd (c->name);
 }
 
 static int
@@ -149,86 +173,179 @@ newcell (char *line)
 }
 
 /*
- * Record this cell in the hashtable
+ *
  */
 
 static void
-recordcell (cell_entry *c, int dbnum, cell_db_entry *dbservers)
+fetch_host (cell_entry *c, cell_db_entry *host)
 {
-     if(dbnum == 0)
-	  KODEB (KODEBMISC, ("No db-servers for cell %s\n", c->name));
+    struct dns_reply *r;
+    struct resource_record *rr;
+    struct timeval tv;
 
-     c->ndbservers  = dbnum;
-     c->dbservers = (cell_db_entry *)malloc (dbnum * sizeof(cell_db_entry));
-     if (c->dbservers == NULL)
-	 err (1, "malloc %lu", (unsigned long)dbnum * sizeof(cell_db_entry));
-     memcpy (c->dbservers, dbservers, dbnum * sizeof (cell_db_entry));
+    gettimeofday (&tv, NULL);
+    
+    r = dns_lookup(host->name, "A");
+    if (r == NULL) {
+	log_log (cell_log, CDEBDNS, 
+		 "fetch_host: failed to resolve host %s in cell %s",
+		 host->name, c->name);
+	host->timeout = CELL_INVALID_HOST;
+	return;
+    }
+    for(rr = r->head; rr;rr=rr->next){
+	if (rr->type == T_A) {
+	    if (strcmp(host->name,rr->domain) == 0) {
+		host->addr = *(rr->u.a);
+		host->timeout = tv.tv_sec + rr->ttl;
+		break;
+	    }
+	}
+    }
+    dns_free_data(r);
+}
+
+
+/*
+ * Help function for updatehosts
+ */
+
+static int 
+host_sort (const void *p1, const void *p2)
+{
+    const cell_db_entry *a = (const cell_db_entry *)p1;
+    const cell_db_entry *b = (const cell_db_entry *)p2;
+    if (a->timeout == CELL_INVALID_HOST)
+	return -1;
+    if (b->timeout == CELL_INVALID_HOST)
+	return 1;
+    return strcmp(a->name, b->name);
+}
+
+/*
+ * Update the hosts for this cell
+ */
+
+static void
+updatehosts (cell_entry *c, int dbnum, cell_db_entry *dbservers)
+{
+    cell_db_entry *old_servers = c->dbservers;
+    struct timeval tv;
+    int i;
+
+    gettimeofday(&tv, NULL);
+
+    for (i = 0; i < dbnum; i++) {
+	if (dbservers[i].timeout == CELL_INVALID_HOST
+	    || (dbservers[i].timeout && dbservers[i].timeout < tv.tv_sec)) {
+	    fetch_host (c, &dbservers[i]);
+	}
+    }
+
+    c->ndbservers = dbnum;
+    c->dbservers  = malloc (dbnum * sizeof(cell_db_entry));
+    if (c->dbservers == NULL)
+	err (1, "malloc %lu", (unsigned long)dbnum * sizeof(cell_db_entry));
+    memcpy (c->dbservers, dbservers, dbnum * sizeof (cell_db_entry));
+    free (old_servers);
+
+    qsort (c->dbservers, c->ndbservers, sizeof(c->dbservers[0]),
+	   host_sort);
+
+    c->active_hosts = 0;
+    for (i = 0; i < c->ndbservers; i++) {
+	if (c->dbservers[i].timeout != CELL_INVALID_HOST)
+	    c->active_hosts++;
+    }
 }
 
 /*
  * try to lookup `cell' in DNS
+ * if c == NULL, a new cell will be allocated
  */
 
-static void
-try_to_find_cell(const char *cell)
+static int
+dns_lookup_cell (const char *cell, cell_entry *c)
 {
     struct dns_reply *r;
     struct resource_record *rr;
-    cell_entry *c = NULL;
     int dbnum = 0;
     cell_db_entry dbservers[256];
+    int lowest_ttl;
+    int i;
+    struct timeval tv;
+
+    memset (dbservers, 0, sizeof(dbservers));
+    gettimeofday(&tv, NULL);
 
     r = dns_lookup(cell, "AFSDB");
-    if (r == NULL)
-	return;
+    if (r == NULL) {
+	log_log (cell_log, CDEBDNS, 
+		 "dns_lookup_cell: failed to resolve cell %s", cell);
+	return 1;
+    }
+    if (c == NULL)
+	c = cell_new_dynamic (cell);
+
     for(rr = r->head; rr;rr=rr->next){
-	switch(rr->type){
-	case T_AFSDB: {
+	if(rr->type == T_AFSDB) {
 	    struct mx_record *mx = (struct mx_record*)rr->u.data;
 
 	    if (mx->preference != 1) {
-		KODEB (KODEBMISC,
-		       ("Ignoring host with cell type %d in cell %s",
-			mx->preference, c->name));
 		break;
 	    }
-	    if (c == NULL)
-		c = cell_new (cell);
 	    if (dbnum >= sizeof (dbservers) / sizeof(*dbservers)) {
-		KODEB (KODEBMISC, ("Too many database servers for "
-				   "cell %s. Ignoring\n", c->name));
 	        break;
 	    }
+	    if (lowest_ttl > rr->ttl || lowest_ttl == 0)
+		lowest_ttl = rr->ttl;
 	    dbservers[dbnum].name = strdup (mx->domain);
 	    if (dbservers[dbnum].name == NULL)
 		err (1, "strdup");
-	    dbservers[dbnum].addr.s_addr = inet_addr ("0.0.0.0");
-	    ++dbnum;
-	    break;
-	}
+	    dbservers[dbnum].timeout = CELL_INVALID_HOST;
+	    dbnum++;
 	}
     }
     for(rr = r->head; rr;rr=rr->next){
-	int i;
-
-	switch(rr->type){
-	case T_A: {
+	if (rr->type == T_A) {
 	    for (i = 0; i < dbnum; i++) {
 		if (strcmp(dbservers[i].name,rr->domain) == 0) {
 		    dbservers[i].addr = *(rr->u.a);
+		    dbservers[i].timeout = tv.tv_sec + rr->ttl;
 		    break;
 		}
 	    }
 	}
-	}
     }
-    if (c)
-	recordcell (c, dbnum, dbservers);
     dns_free_data(r);
+
+    /* catch the hosts that didn't fit in additional rr */
+    c->timeout = lowest_ttl + tv.tv_sec;
+    updatehosts (c, dbnum, dbservers);
+    return 0;
 }
 
 /*
- *
+ * If the cell-information comes from a source that have a time-limit,
+ * make sure the data is uptodate.
+ */
+
+static void
+update_cell (cell_entry *c)
+{
+    struct timeval tv;
+
+    gettimeofday(&tv, NULL);
+
+    if ((c->timeout && c->timeout < tv.tv_sec)
+	|| c->ndbservers == 0) {
+	dns_lookup_cell (c->name, c);
+    }
+    updatehosts (c, c->ndbservers, c->dbservers);
+}
+
+/*
+ * cell name -> cell_entry *
  */
 
 cell_entry *
@@ -239,23 +356,28 @@ cell_get_by_name (const char *cellname)
     key.name = cellname;
     data = (cell_entry *)hashtabsearch (cellnamehtab, &key);
     if (data == NULL) {
-	try_to_find_cell (cellname);
+	dns_lookup_cell (cellname, NULL);
 	data = (cell_entry *)hashtabsearch (cellnamehtab, &key);
     }
+    if (data)
+	update_cell (data);
     return data;
 }
 
 /*
- *
+ * cell id -> cell_entry *
  */
 
 cell_entry *
 cell_get_by_id (int32_t cell)
 {
-    cell_entry key;
+    cell_entry key, *data;
 
     key.id = cell;
-    return (cell_entry *)hashtabsearch (cellnumhtab, &key);
+    data = (cell_entry *)hashtabsearch (cellnumhtab, &key);
+    if (data)
+	update_cell (data);
+    return data;
 }
 
 /*
@@ -283,17 +405,45 @@ cell_new (const char *name)
 	return NULL;
     }
     assert (cellno != 0);
-    c->id         = cellno++;
-    c->expl       = NULL;
-    c->ndbservers = 0;
-    c->dbservers  = NULL;
-    c->suid_cell  = NOSUID_CELL;
+    c->id           = cellno++;
+    c->expl         = NULL;
+    c->ndbservers   = 0;
+    c->active_hosts = 0;
+    c->dbservers    = NULL;
+    c->suid_cell    = NOSUID_CELL;
     hashtabadd (cellnamehtab, c);
     hashtabadd (cellnumhtab, c);
+    c->timeout      = 0;
     celldb_version++;
     return c;
 }
 
+/*
+ * add a new `dynamic' cell
+ */
+
+cell_entry *
+cell_new_dynamic (const char *name)
+{
+    cell_entry *c;
+    FILE *f;
+
+    c = cell_new (name);
+    if (c == NULL)
+	return NULL;
+    c->expl = "dynamically added cell";
+    f = fopen (SYSCONFDIR "/" CELLFILENAME, "a");
+    if (f == NULL)
+	f = fopen (TRANSARCSYSCONFDIR "/" CELLFILENAME, "a");
+    if (f == NULL) {
+	log_log (cell_log, CDEBWARN,"Cannot open CellServDB for writing");
+	return c;
+    }
+    fprintf (f, ">%s	#dynamically added cell\n", name);
+    fclose (f);
+    return c;
+}
+
 /*
  * Read one line of database information.
  */
@@ -303,26 +453,43 @@ readdb (char *line, cell_entry* c, int *dbnum, int maxdbs,
 	cell_db_entry *dbservs, int lineno)
 {
      struct in_addr numaddr;
-     char *hostname;
+     char *hostname, *eh;
 
      if (*dbnum >= maxdbs) {
-	  KODEB (KODEBMISC, ("Too many database servers for "
-		   "cell %s. Ignoring\n", c->name));
 	  return -1;
      }
-     if ( (hostname = strchr (line, '#')) == NULL
-	 || (numaddr.s_addr = inet_addr (line)) == -1 ) {
-	  KODEB (KODEBMISC, ("Syntax error at line %d in %s\n",
-		   lineno, CELLFILENAME));
+     
+     while (*line && isspace((unsigned char)*line))
+	    ++line;
+
+     if (inet_aton (line, &numaddr) == 0) {
 	  return -1;
-     } else {
-	  ++hostname;
-	  dbservs[*dbnum].name = strdup (hostname);
-	  if (dbservs[*dbnum].name == NULL)
-	      err (1, "strdup");
-	  dbservs[*dbnum].addr = numaddr;
-	  ++(*dbnum);
      }
+
+     while (*line && !isspace((unsigned char)*line) && *line != '#')
+	 ++line;
+
+     hostname = line;
+
+     while (isspace ((unsigned char)*hostname) || *hostname == '#')
+	 ++hostname;
+
+     eh = hostname;
+
+     while (*eh && !isspace((unsigned char)*eh) && *eh != '#')
+	 ++eh;
+
+     *eh = '\0';
+
+     if (*hostname == '\0')
+	 hostname = inet_ntoa (numaddr);
+     
+     dbservs[*dbnum].name = strdup (hostname);
+     if (dbservs[*dbnum].name == NULL)
+	 err (1, "strdup");
+     dbservs[*dbnum].addr = numaddr;
+     dbservs[*dbnum].timeout = 0;
+     ++(*dbnum);
      return 0;
 }
 
@@ -339,41 +506,83 @@ readcellservdb (const char *filename)
      int lineno = 0;
      int dbnum;
      cell_db_entry dbservs[256];
+     int i;
 
      f = fopen (filename, "r");
      if (f == NULL) {
-	  KODEB (KODEBMISC, ("Cannot read cell information from %s\n", 
-		   filename));
 	  return 1;
      }
 
      while (fgets (line, sizeof (line), f)) {
 	  ++lineno;
-	  if (line[strlen(line) - 1 ] != '\n')
-	       KODEB (KODEBMISC, ("Too long line at line %d in %s\n",
-			lineno, filename));
-	  else
-	       line[strlen(line) - 1] = '\0';
-	  if (*line == '#' || *line == '\0')
+	  i = 0;
+	  line[strlen(line) - 1] = '\0';
+	  while (line[0] && isspace((unsigned char)line[i]))
+		 i++;
+	  if (line[i] == '#' || line[i] == '\0')
 	       continue;
-	  if (*line == '>') {
+	  if (line[i] == '>') {
 	       if (c != NULL)
-		    recordcell (c, dbnum, dbservs);
-	       c = newcell (line + 1);
+		    updatehosts (c, dbnum, dbservs);
+	       c = newcell (&line[i] + 1);
+	       memset (dbservs, 0, sizeof(dbservs));
 	       dbnum = 0;
 	  } else {
-	       if (readdb(line, c, &dbnum, sizeof (dbservs) /
-			  sizeof(*dbservs),
-			  dbservs, lineno))
-		    continue;
+	      if (readdb(&line[i], c, &dbnum, sizeof (dbservs) /
+			 sizeof(*dbservs),
+			 dbservs, lineno))
+		  continue;
 	  }
      }
      if (c != NULL)
-	  recordcell (c, dbnum, dbservs);
+	  updatehosts (c, dbnum, dbservs);
      fclose (f);
      return 0;
 }
 
+/*
+ * Add expanded `cellname' to Thesecells list, filter out duplicates
+ */
+
+static void
+addthesecell (const char *cellname)
+{
+    int i;
+    cellname = cell_expand_cell (cellname);
+    for (i = 0; thesecells && thesecells[i]; i++)
+	if (strcasecmp (thesecells[i], cellname) == 0)
+	    return;
+    thesecells = erealloc (thesecells, 
+			   (numthesecells + 1) * sizeof (char *));
+    thesecells[numthesecells - 1] = estrdup (cellname);
+    thesecells[numthesecells] = NULL;
+    ++numthesecells;
+}
+
+/*
+ * Read cells in these cell's
+ */
+
+static int
+readthesecells (const char *filename)
+{
+     FILE *f;
+     char cell[256];
+
+     f = fopen (filename, "r");
+     if (f == NULL)
+	 return 1;
+
+     while ((fgets (cell, sizeof cell, f) != NULL)) {
+	 cell[strlen(cell) - 1] = '\0';
+	 addthesecell (cell);
+     }
+     fclose (f);
+     if (numthesecells == 1)
+	 return 1;
+     return 0;
+}
+
 /*
  * Read name of this cell.
  */
@@ -385,17 +594,14 @@ readthiscell (const char *filename)
      char cell[256];
 
      f = fopen (filename, "r");
-     if (f == NULL) {
-	 fprintf(stderr, "Can't open file %s.\n", filename);
+     if (f == NULL)
 	 return 1;
-     }
 
      if (fgets (cell, sizeof cell, f) == NULL) {
-	  fprintf(stderr, "Cannot read cellname from %s\n",
+	 log_log (cell_log, CDEBERR, "Cannot read cellname from %s\n",
 		  filename);
-	  return 1;
+	 return 1;
      }
-     cell[sizeof(cell)-1] = '\0';
      if (cell[strlen(cell) - 1] == '\n')
 	 cell[strlen(cell) - 1] = '\0';
      thiscell = strdup (cell);
@@ -416,23 +622,21 @@ readsuidcell (const char *filename)
      char cell[256];
 
      f = fopen (filename, "r");
-     if (f == NULL) {
-	 fprintf(stderr, "Can't open file %s.\n", filename);
+     if (f == NULL)
 	 return 1;
-     }
 
      while (fgets (cell, sizeof(cell), f) != NULL) {
 	 int i;
 	 cell_entry *e;
 
-	 cell[sizeof(cell)-1] = '\0';
 	 i = strlen (cell);
 	 if (cell[i - 1] == '\n')
 	     cell[i - 1] = '\0';
 
 	 e = cell_get_by_name (cell);
 	 if (e == NULL) {
-	     fprintf (stderr, "cell %s doesn't exist in the db\n", cell);
+	     	log_log (cell_log, CDEBWARN,
+			 "cell %s doesn't exist in the db\n", cell);
 	 } else {
 	     e->suid_cell = SUID_CELL;
 	 }
@@ -448,17 +652,27 @@ readsuidcell (const char *filename)
 static int cell_inited = 0;
 
 void
-cell_init (int cellcachesize)
+cell_init (int cellcachesize, Log_method *log)
 {
     char *env;
     int ret;
 
+    assert (log);
+
     if (cell_inited) {
-	fprintf(stderr, "cell_init: Already initlized\n");
+	log_log (cell_log, CDEBWARN, "cell_init: Already initlized");
 	return;
     }
     cell_inited = 1;
 
+#ifdef HAVE_RES_INIT
+    res_init();
+#endif
+
+    cell_log = log_unit_init (log, "cell", celldebug_units, CDEBDEFAULT);
+    if (cell_log == NULL)
+	errx (1, "cell_init: log_unit_init failed");
+
     if (cellcachesize == 0)
 	cellcachesize = DEFCELLCACHESIZE;
 
@@ -471,32 +685,48 @@ cell_init (int cellcachesize)
 
     env = getenv ("AFSCELL");
     if (env != NULL) {
-	thiscell = strdup (env);
-	if (thiscell == NULL)
-	    err (1, "strdup");
+	thiscell = estrdup (env);
     } else if (readthiscell (SYSCONFDIR "/" THISCELLFILENAME)) {
-	fprintf(stderr, "Falling back on: " TRANSARCSYSCONFDIR "\n"); 
-	if (readthiscell(TRANSARCSYSCONFDIR "/" THISCELLFILENAME)) {
-	    fprintf(stderr, "Don't know where I am\n");
-	    exit(1);
-	}
+	if (readthiscell(TRANSARCSYSCONFDIR "/" THISCELLFILENAME))
+	    errx (1, "could not open "
+		  SYSCONFDIR "/" THISCELLFILENAME
+		  " nor "
+		  TRANSARCSYSCONFDIR "/" THISCELLFILENAME);
     }
 
     if (readcellservdb (SYSCONFDIR "/" CELLFILENAME)) {
-	fprintf(stderr, "Falling back on: " TRANSARCSYSCONFDIR "\n");
 	if (readcellservdb(TRANSARCSYSCONFDIR "/" CELLFILENAME)) {
-	    fprintf(stderr, "Can't read the CellServDB file," \
-		    "will use DNS AFSDB entries\n");
+	    log_log (cell_log, CDEBWARN,
+		     "Can't read the CellServDB file, "
+		     "will use DNS AFSDB entries");
 	}
     }
-    if (readsuidcell (SYSCONFDIR "/" SUIDCELLSFILENAME)) {
-	fprintf (stderr, "Falling back on: " TRANSARCSYSCONFDIR "\n");
-	if (readsuidcell (TRANSARCSYSCONFDIR "/" SUIDCELLSFILENAME))
-	    fprintf (stderr, "Cant read suid cells, ignoring\n");
-    }
     ret = add_special_dynroot_cell();
     if (ret)
-	fprintf (stderr, "adding dynroot cell failed with %d", ret);
+	log_log (cell_log, CDEBWARN, "adding dynroot cell failed with %d", ret);
+    
+    if (readthesecells (SYSCONFDIR "/" THESECELLFILENAME))
+	readthesecells (TRANSARCSYSCONFDIR "/" THESECELLFILENAME);
+    if (getenv("HOME") != NULL) {
+	char homedir[MAXPATHLEN];
+	snprintf (homedir, sizeof(homedir), 
+		  "%s/." THESECELLFILENAME,
+		  getenv("HOME"));
+	readthesecells (homedir);
+    }
+    addthesecell (thiscell);
+    if (readsuidcell (SYSCONFDIR "/" SUIDCELLSFILENAME))
+	readsuidcell (TRANSARCSYSCONFDIR "/" SUIDCELLSFILENAME);
+}
+
+/*
+ *
+ */
+
+const char **
+cell_thesecells (void)
+{
+    return (const char **)thesecells;
 }
 
 /*
@@ -505,23 +735,25 @@ cell_init (int cellcachesize)
  */
 
 const cell_db_entry *
-cell_dbservers (int32_t cell, int *num)
+cell_dbservers_by_id (int32_t id, int *num)
 {
-    cell_entry *data = cell_get_by_id (cell);
+    cell_entry *data = cell_get_by_id (id);
 
-    if (data == NULL) {
-	KODEB (KODEBMISC, ("Cannot find cell %d\n", cell));
+    if (data == NULL)
 	return NULL;
-    }
-    *num = data->ndbservers;
+
+    if (data->ndbservers == 0)
+	dns_lookup_cell (data->name, data);
+     if (data->ndbservers == 0 || data->active_hosts == 0)
+	 return NULL;
+
+    *num = data->active_hosts;
     return data->dbservers;
 }
 
 /*
- * Find the name of DB server to talk to for a given cell.
- *
- *   Warning: This is not really good since name could be something
- *     like NULL.
+ * Return the name of the first database server in `cell' or NULL (if
+ * the cell does not exist or it has no db servers).
  */
 
 const char *
@@ -529,14 +761,13 @@ cell_findnamedbbyname (const char *cell)
 {
      cell_entry *data = cell_get_by_name (cell);
 
-     if (data == NULL) {
-	  KODEB (KODEBMISC, ("Cannot find cell %d\n", cell));
+     if (data == NULL)
 	  return NULL;
-     }
-     if (data->ndbservers == 0) {
-	  KODEB (KODEBMISC, ("No DB servers for cell %d\n", cell));
-	  return NULL;
-     }
+     if (data->ndbservers == 0)
+	 dns_lookup_cell (cell, data);
+     if (data->ndbservers == 0 || data->active_hosts == 0)
+	 return NULL;
+
      return data->dbservers[0].name ;
 }
 
@@ -556,13 +787,10 @@ cell_getcellbyhost(const char *host)
     if (ptr)
 	return ptr;
 
-    ptr = host;
-    while(*ptr && *ptr != '.') ptr++;
-    
-    if (*ptr == '\0')
+    ptr = strchr (host, '.');
+    if (ptr == NULL)
 	return NULL;
-
-    return strdup(ptr); 
+    return ptr + 1;
 }
 
 /*
@@ -617,12 +845,9 @@ cell_setthiscell (const char *cell)
 {
     cell_entry *data;
 
-    if (!cell_inited)
-	cell_init (0);
-
     data = cell_get_by_name(cell);
     if (data == NULL) {
-	fprintf (stderr, "this cell doesn't exist: %s", cell);
+	log_log (cell_log, CDEBWARN, "this cell doesn't exist: %s", cell);
 	return 1;
     }
 
@@ -642,14 +867,15 @@ Bool
 cell_issuid (cell_entry *c)
 {
     assert (c);
-
     return c->suid_cell == SUID_CELL;
 }
 
 Bool
 cell_issuid_by_num (int32_t cell)
 {
-    cell_entry *c = cell_get_by_id (cell);
+    cell_entry *c;
+
+    c = cell_get_by_id (cell);
     if (c == NULL)
 	return FALSE;
     
@@ -659,7 +885,9 @@ cell_issuid_by_num (int32_t cell)
 Bool
 cell_issuid_by_name (const char *cell)
 {
-    cell_entry *c = cell_get_by_name (cell);
+    cell_entry *c;
+
+    c = cell_get_by_name (cell);
     if (c == NULL)
 	return FALSE;
     
@@ -669,7 +897,9 @@ cell_issuid_by_name (const char *cell)
 Bool
 cell_setsuid_by_num (int32_t cell)
 {
-    cell_entry *c = cell_get_by_id (cell);
+    cell_entry *c;
+
+    c = cell_get_by_id (cell);
     if (c == NULL)
 	return FALSE;
     
@@ -678,6 +908,46 @@ cell_setsuid_by_num (int32_t cell)
     return 0;
 }
 
+/*
+ *
+ */
+
+void
+cell_print_cell (cell_entry *c, FILE *out)
+{
+    int i;
+    char timestr[30];
+    struct tm tm;
+    time_t t;
+
+    fprintf (out, "name: %s id: %d type: %s\n",
+	     c->name, c->id, c->timeout ? "dynamic" : "static");
+    fprintf (out, "comment: %s\n", c->expl);
+    if (c->timeout) {
+	t = c->timeout;
+	strftime(timestr, sizeof(timestr),
+		 "%Y-%m-%d %H:%M:%S", localtime_r(&t, &tm));
+	fprintf (out, "timeout: %s\n", timestr);
+    }
+    fprintf (out, "num hosts: %d active hosts: %d\n",
+	     c->ndbservers, c->active_hosts);
+    for (i = 0; i < c->ndbservers; i++) {
+	char *buf;
+	if (c->dbservers[i].timeout == CELL_INVALID_HOST)
+	    buf = "invalid";
+	else if (c->dbservers[i].timeout == 0)
+	    buf = "no timeout";
+	else {
+	    t = c->timeout;
+	    strftime(timestr, sizeof(timestr),
+		     "%Y-%m-%d %H:%M:%S", localtime_r(&t, &tm));
+	    buf = timestr;
+	}
+	fprintf (out, " host: %s %s - %s\n", c->dbservers[i].name,
+		 inet_ntoa (c->dbservers[i].addr), buf);
+    }
+}
+
 /*
  * Iterate over all entries in the cell-db with `func'
  * (that takes the cellname and `arg' as arguments)
@@ -716,6 +986,45 @@ cell_foreach (int (*func) (const cell_entry *, void *), void *arg)
     return cia.ret;
 }
 
+/*
+ *
+ */
+
+struct cell_expand_arg {
+    const char *cell;
+    const char *fullcell;
+    int32_t last_found;
+};
+
+static int
+cell_expand_cell_name (const cell_entry *cell, void *data)
+{
+    struct cell_expand_arg *cea = (struct cell_expand_arg *) data;
+    if (strcasecmp (cell->name, cea->cell) == 0) {
+	cea->fullcell = cell->name;
+	return 1;
+    }
+    if (strstr (cell->name, cea->cell) && cea->last_found > cell->id) {
+	cea->last_found = cell->id;
+	cea->fullcell = cell->name;
+    }
+    return 0;
+}
+
+const char *
+cell_expand_cell (const char *cell)
+{
+    struct cell_expand_arg cea;
+    cea.cell = cell;
+    cea.fullcell = NULL;
+    cea.last_found = cellno + 1;
+
+    cell_foreach (cell_expand_cell_name, &cea);
+    if (cea.fullcell)
+	return cea.fullcell;
+    return cell;
+}
+
 /*
  *
  */
@@ -756,3 +1065,13 @@ add_special_dynroot_cell (void)
     hashtabadd (cellnumhtab, c);
     return 0;
 }
+
+/*
+ * Return TRUE if ``cell'' has a sane cellnumber.
+ */
+
+Bool
+cell_is_sanep (int cell)
+{
+    return cell < cellno;
+} 
