@@ -45,7 +45,7 @@ typedef struct QUEUE {
     struct Lock		lock;
 } queue;
 
-queue *init()
+static queue *init()
 {
     queue *q;
 
@@ -54,12 +54,12 @@ queue *init()
     return(q);
 }
 
-char empty(queue *q)
+static char empty(queue *q)
 {
     return (q->prev == q && q->next == q);
 }
 
-void insert(queue *q, char *s)
+static void insert(queue *q, char *s)
 {
     queue *new;
 
@@ -71,7 +71,7 @@ void insert(queue *q, char *s)
     new -> next = q;
 }
 
-char *Remove(queue *q)
+static char *Remove(queue *q)
 {
     queue *old;
     char *s;
