@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -55,7 +50,13 @@
 
 #include <dpart.h>
 
-RCSID("$Id: dpart.c,v 1.2 1999/07/12 08:32:11 map Exp $");
+RCSID("$Id: dpart.c,v 1.6 2000/12/29 20:12:35 tol Exp $");
+
+#ifdef MILKO_ROOT
+char *dpart_root = MILKO_ROOT;
+#else
+char *dpart_root = "";
+#endif
 
 /*
  * Allocate a dp_part structure for partition `num' and
@@ -67,20 +68,31 @@ dp_create (u_int32_t num, struct dp_part **dp)
 {
     struct dp_part *d;
     int ret;
+    char str[3], *ptr = str;
 
     assert (dp);
 
     *dp = NULL;
 
-    if (num > 'z' - 'a')
-	return EINVAL; /* XXX */
+    if (num > ('z' - 'a' * 'z' - 'a'))
+	return EINVAL;
 
-    d = calloc (sizeof (**dp), 1);
+    d = malloc (sizeof (*d));
     if (d == NULL)
-	return ENOMEM; /* XXX */
+	return ENOMEM;
+    memset (d, 0, sizeof(*d));
 
-    ret = asprintf (&d->part, "/vicep%c", num + 'a');
-    if (ret != 7) {
+    if (num > 'z' - 'a') {
+	ret = num / ('z' - 'a');
+	num -= ret * ('z' - 'a');
+	*ptr = ret + 'a';
+	ptr++;
+    }
+    ptr[0] = num + 'a';
+    ptr[1] = '\0';
+
+    ret = asprintf (&d->part, "%s/vicep%s", dpart_root, str);
+    if (ret < 0) {
 	free (d);
 	return EINVAL;
     }
@@ -136,23 +148,21 @@ dp_find (struct dp_part **dp)
     } else {	
 	num = (*dp)->num;
 	dp_free (*dp);
+	*dp = NULL;
     }
     do { 
 	++num;
 	
-	if (num > 'z' - 'a') {
-	    *dp = NULL;
-	    break;
-	}
-	    
+	if (*dp != NULL)
+	    dp_free (*dp);
+
 	ret = dp_create (num, dp);
 	if (ret)
 	    return ret;
 	
-	if (stat ((*dp)->part, &sb) == 0 &&
-	    S_ISDIR(sb.st_mode))
+	if (stat ((*dp)->part, &sb) == 0 && S_ISDIR(sb.st_mode))
 	    found = 1;
-	
+
     } while (!found);
 
     return 0;
@@ -194,3 +204,61 @@ dp_findvol (struct dp_part *d, void (*cb)(void *, int), void *data)
     closedir (dir);
     return 0;
 }
+
+/*
+ *
+ */
+
+int
+dp_parse (const char *str, u_int32_t *num)
+{
+    int len;
+    u_int32_t part = 0;
+
+    assert (str && num);
+
+    len = strlen (str);
+    if (len == 0)
+	return EINVAL;
+    if (len > 6 && strncmp (str, "/vicep", 6) == 0) {
+	str += 6;
+	len -= 6;
+    }
+    if (len > 2)
+	return EINVAL;
+    if (len == 2) {
+	if (*str < 'a' || *str > 'z')
+	    return EINVAL;
+	part += ('z' - 'a') * (*str - 'a' + 1);
+	str++;
+	len--;
+    }
+    if (*str < 'a' || *str > 'z')
+	return EINVAL;
+    part += *str - 'a';
+
+    *num = part;
+    return 0;
+}
+
+/*
+ *
+ */
+
+struct dp_part *
+dp_getpart (const char *str)
+{
+    int ret;
+    struct dp_part *dp;
+    u_int32_t part;
+
+    ret = dp_parse (str, &part);
+    if (ret)
+	return NULL;
+
+    ret = dp_create (part, &dp);
+    if (ret)
+	return NULL;
+
+    return dp;
+}
