@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1998, 1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -44,6 +39,8 @@
 #include <rx/rx.h>
 #include <rx/rx_null.h>
 
+#include <assert.h>
+
 #ifdef KERBEROS
 #include <des.h>
 #include <krb.h>
@@ -57,7 +54,6 @@
 #endif
 
 #include <err.h>
-#include <assert.h>
 #include <ctype.h>
 
 #ifndef HAVE_UNISTD_H
@@ -71,9 +67,21 @@
 #include "msecurity.h"
 #include "acl.h"
 
-RCSID("$Id: msecurity.c,v 1.2 1999/05/18 10:51:03 lha Exp $");
+RCSID("$Id: msecurity.c,v 1.9 2000/10/03 00:18:08 lha Exp $");
 
-char acl_file[] = "/usr/arla/etc/superuserlist"; /* XXX */
+static char acl_file[] = MILKO_SYSCONFDIR "/superuserlist"; /* XXX */
+
+/*
+ * Don't test for superuser if set
+ */
+
+static int superuser_check_disabled = FALSE;
+
+void
+sec_disable_superuser_check (void)
+{
+    superuser_check_disabled = TRUE;
+}
 
 /*
  * Return 0 if the user is not a superuser
@@ -88,7 +96,10 @@ sec_is_superuser(struct rx_call *call)
     char realm[REALM_SZ];
     char fullname[MAX_K_NAME_SZ];
 
-    if (sec_getname(call, aname, inst, realm))
+    if (superuser_check_disabled)
+	return 1;
+
+    if (sec_getname(call->conn, aname, inst, realm))
 	return 0;
 #ifdef KERBEROS
     if (call->conn->securityIndex == 2) {
@@ -102,13 +113,21 @@ sec_is_superuser(struct rx_call *call)
 int
 sec_add_superuser(char *user)
 {
+#ifdef KERBEROS
     return acl_add(acl_file, user);
+#else
+    return -1;
+#endif
 }
 
 int
 sec_del_superuser(char *user)
 {
+#ifdef KERBEROS
     return acl_delete(acl_file, user);
+#else
+    return -1;
+#endif
 }
 
 /*
@@ -118,19 +137,17 @@ sec_del_superuser(char *user)
  */
 
 int
-sec_getname(struct rx_call *call, char *name, char *instance, char *realm)
+sec_getname(struct rx_connection *conn,
+	    char *name, char *instance, char *realm)
 {
 #ifdef KERBEROS
-    if (call->conn->securityIndex == 2) {
+    if (conn->securityIndex == 2) {
 	serv_con_data *cdat;
 
-	cdat = call->conn->securityData;
-	strncpy(name, cdat->user->name, ANAME_SZ);
-	name[ANAME_SZ-1] = '\0';
-	strncpy(instance, cdat->user->instance, INST_SZ);
-	instance[INST_SZ-1] = '\0';
-	strncpy(realm, cdat->user->realm, REALM_SZ);
-	realm[REALM_SZ-1] = '\0';
+	cdat = conn->securityData;
+	strlcpy(name, cdat->user->name, ANAME_SZ);
+	strlcpy(instance, cdat->user->instance, INST_SZ);
+	strlcpy(realm, cdat->user->realm, REALM_SZ);
 	return 0;
     }
 #endif
