@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -53,7 +48,7 @@
 #include <roken.h>
 
 #ifdef RCSID
-RCSID("$Id: large-dir.c,v 1.5 1999/05/28 08:46:50 assar Exp $");
+RCSID("$Id: large-dir.c,v 1.7 2000/10/03 00:34:31 lha Exp $");
 #endif
 
 static int
@@ -131,8 +126,10 @@ creat_files (const char *dirname, int count)
     closedir (d);
     if (stat (".", &sb) < 0)
 	err (1, "stat .");
+#if 0
     if (sb.st_size != 2048)
 	errx (1, "size != 2048");
+#endif
     return 0;
 }
 
