@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- *
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -40,7 +35,11 @@
 #include <xfs/xfs_common.h>
 #include <xfs/xfs_deb.h>
 
-RCSID("$Id: xfs_common-bsd.c,v 1.9.2.1 1999/08/08 16:31:55 assar Exp $");
+RCSID("$Id: xfs_common-bsd.c,v 1.16 2000/10/02 23:31:37 lha Exp $");
+
+#ifdef MALLOC_DEFINE
+MALLOC_DEFINE(M_XFS, "xfs", "xfs buffer");
+#endif
 
 #ifdef XFS_DEBUG
 static u_int xfs_allocs;
@@ -49,20 +48,24 @@ static u_int xfs_frees;
 void *
 xfs_alloc(u_int size)
 {
+    void *ret;
+
     xfs_allocs++;
     XFSDEB(XDEBMEM, ("xfs_alloc: xfs_allocs - xfs_frees %d\n", 
 		     xfs_allocs - xfs_frees));
 
-    return malloc(size, M_TEMP, M_WAITOK); /* What kind? */
+    MALLOC(ret, void *, size, M_XFS, M_WAITOK);
+    return ret;
 }
 
 void
 xfs_free(void *ptr, u_int size)
 {
     xfs_frees++;
-    free(ptr, M_TEMP);
+    FREE(ptr, M_XFS);
 }
-#endif
+
+#endif /* XFS_DEBUG */
 
 int
 xfs_suser(struct proc *p)
@@ -82,3 +85,30 @@ memcpy (void *s1, const void *s2, size_t n)
     return s1;
 }
 #endif
+
+/*
+ * Print a `dev_t' in some readable format
+ */
+
+#ifdef HAVE_KERNEL_DEVTONAME
+
+const char *
+xfs_devtoname_r (dev_t dev, char *buf, size_t sz)
+{
+    return devtoname (dev);
+}
+
+#else /* !HAVE_KERNEL_DEVTONAME */
+
+const char *
+xfs_devtoname_r (dev_t dev, char *buf, size_t sz)
+{
+#ifdef HAVE_KERNEL_SNPRINTF
+    snprintf (buf, sz, "%u/%u", major(dev), minor(dev));
+    return buf;
+#else
+    return "<unknown device>";
+#endif
+}
+
+#endif /* HAVE_KERNEL_DEVTONAME */
