@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  *
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- *
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -41,7 +36,7 @@
 #include <xfs/xfs_fs.h>
 #include <xfs/xfs_deb.h>
 
-RCSID("$Id: xfs_node-osf.c,v 1.9 1999/02/26 00:46:45 lha Exp $");
+RCSID("$Id: xfs_node-osf.c,v 1.14.2.1 2001/01/25 08:41:22 lha Exp $");
 
 extern struct vnodeops *xfs_vnodeop_p;
 extern struct vfs_ubcops xfs_ubcops;
@@ -101,7 +96,7 @@ new_xfs_node(struct xfs *xfsp,
     }
 
     /* Init other fields */
-    xfs_attr2vattr(&node->attr, &result->attr);
+    xfs_attr2vattr(&node->attr, &result->attr, 1);
     result->vn->v_type = result->attr.va_type;
     XFS_TOKEN_SET(result, XFS_ATTR_R, XFS_ATTR_MASK);
     bcopy(node->id, result->id, sizeof(result->id));
@@ -129,7 +124,8 @@ free_xfs_node(struct xfs_node *node)
 {
     struct xfs *xfsp = XFS_FROM_XNODE(node);
 
-    XFSDEB(XDEBNODE, ("free_xfs_node starting: node = %p\n", node));
+    XFSDEB(XDEBNODE, ("free_xfs_node starting: node = %lx\n",
+		      (unsigned long)node));
 
     /* XXX Really need to put back dirty data first. */
 
@@ -190,8 +186,9 @@ xfs_node_find(struct xfs *xfsp, xfs_handle *handlep)
     struct vnode *t;
     struct xfs_node *xn = NULL;
 
-    XFSDEB(XDEBNODE, ("xfs_node_find: xfsp = %p handlep = %p\n", 
-		      xfsp, handlep));
+    XFSDEB(XDEBNODE, ("xfs_node_find: xfsp = %lx handlep = %lx\n", 
+		      (unsigned long)xfsp,
+		      (unsigned long)handlep));
 
     for(t = XFS_TO_VFS(xfsp)->m_mounth; 
 	t != NULL; 
@@ -265,9 +262,10 @@ vattr2xfs_attr(const struct vattr *va, struct xfs_attr *xa)
 #define SET_TIMEVAL(X, S, N) do { (X)->tv_sec = (S); (X)->tv_usec = (N) / 1000; } while(0)
 
 void
-xfs_attr2vattr(const struct xfs_attr *xa, struct vattr *va)
+xfs_attr2vattr(const struct xfs_attr *xa, struct vattr *va, int clear_node)
 {
-    VATTR_NULL(va);
+    if (clear_node)
+	VATTR_NULL(va);
     if (XA_VALID_MODE(xa))
 	va->va_mode = xa->xa_mode;
     if (XA_VALID_NLINK(xa))
@@ -397,12 +395,16 @@ xfs_dnlc_enter(struct vnode *dvp,
 	       xfs_componentname *ndp,
 	       struct vnode *vp)
 {
-    XFSDEB(XDEBDNLC, ("xfs_dnlc_enter(%p, %p, %p)\n", dvp, ndp, vp));
+    XFSDEB(XDEBDNLC, ("xfs_dnlc_enter(%lx, %lx, %lx)\n",
+		      (unsigned long)dvp,
+		      (unsigned long)ndp,
+		      (unsigned long)vp));
     XFSDEB(XDEBDNLC, ("xfs_dnlc_enter: v_id = %ld\n", dvp->v_id));
 
     XFSDEB(XDEBDNLC, ("xfs_dnlc_enter: calling cache_enter:"
-		      "dvp = %p, vp = %p, ndp = (%s, %ld)\n",
-		      dvp, vp, ndp->ni_ptr, ndp->ni_namelen));
+		      "dvp = %lx, vp = %lx, ndp = (%s, %ld)\n",
+		      (unsigned long)dvp,
+		      (unsigned long)vp, ndp->ni_ptr, ndp->ni_namelen));
 
     if (ndp->ni_namelen <= NCHNAMLEN)
     {
@@ -431,7 +433,7 @@ xfs_dnlc_enter(struct vnode *dvp,
     return 0;
 }
 
-void
+static void
 xfs_cnp_init (struct nameidata *ndp,
 	      const char *name, struct vnode *vp, struct vnode *dvp,
 	      struct proc *proc, struct ucred *cred,
@@ -462,7 +464,9 @@ xfs_dnlc_enter_name(struct vnode *dvp,
 {
     struct nameidata nd;
 
-    XFSDEB(XDEBDNLC, ("xfs_dnlc_enter_name(%p, \"%s\", %p)\n", dvp, name, vp));
+    XFSDEB(XDEBDNLC, ("xfs_dnlc_enter_name(%lx, \"%s\", %lx)\n",
+		      (unsigned long)dvp, name,
+		      (unsigned long)vp));
 
     xfs_cnp_init (&nd, name, vp, dvp, NULL, NULL, CREATE);
     return xfs_dnlc_enter (dvp, &nd, vp);
@@ -480,13 +484,15 @@ xfs_dnlc_lookup(struct vnode *dvp,
 {
     int error;
 
-    XFSDEB(XDEBDNLC, ("xfs_dnlc_lookup(%p, \"%s\")\n", dvp, cnp->cn_nameptr));
+    XFSDEB(XDEBDNLC, ("xfs_dnlc_lookup(%lx, \"%s\")\n",
+		      (unsigned long)dvp, cnp->cn_nameptr));
     
     XFSDEB(XDEBDNLC, ("xfs_dnlc_lookup: v_id = %ld\n", dvp->v_id));
     
     XFSDEB(XDEBDNLC, ("xfs_dnlc_lookup: calling cache_lookup:"
-		      "dvp = %p, cnp = (%s, %ld, %lu)\n",
-		      dvp, cnp->cn_nameptr, cnp->cn_namelen, cnp->cn_hash));
+		      "dvp = %lx, cnp = (%s, %ld, %lu)\n",
+		      (unsigned long)dvp,
+		      cnp->cn_nameptr, cnp->cn_namelen, cnp->cn_hash));
 
     cnp->ni_makeentry = 1;
 
@@ -495,7 +501,8 @@ xfs_dnlc_lookup(struct vnode *dvp,
     *res = cnp->ni_vp;
 
     XFSDEB(XDEBDNLC, ("xfs_dnlc_lookup: cache_lookup returned. "
-		      "error = %d, *res = %p\n", error, *res));
+		      "error = %d, *res = %lx\n", error,
+		      (unsigned long)*res));
 
     if (error == -1 || error == ENOENT)
 	return error;
@@ -552,7 +559,7 @@ xfs_dnlc_purge_mp (struct mount *mp)
  */
 
 int
-xfs_has_pag(const struct xfs_node *xn, pag_t pag)
+xfs_has_pag(const struct xfs_node *xn, xfs_pag_t pag)
 {
     int i;
 
