@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  *
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- *
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -38,7 +33,7 @@
 
 #include <xfs/xfs_locl.h>
 
-RCSID("$Id: xfs_syscalls-wrap-bsd.c,v 1.4 1999/06/09 20:30:53 joda Exp $");
+RCSID("$Id: xfs_syscalls-wrap-bsd.c,v 1.11 2000/12/12 11:01:09 lha Exp $");
 
 /*
  * XFS system calls.
@@ -53,7 +48,7 @@ RCSID("$Id: xfs_syscalls-wrap-bsd.c,v 1.4 1999/06/09 20:30:53 joda Exp $");
 
 #include <kafs.h>
 
-static int syscall_offset;
+int xfs_syscall_num;
 
 static struct sysent old_syscallent;
 
@@ -115,7 +110,7 @@ install_first_free_syscall (int *offset,
     int i;
 
     for (i = 1; i < SYS_MAXSYSCALL; ++i)
-	if (try_install_syscall (i, sysent, old_sysent)) {
+	if (try_install_syscall (i, sysent, old_sysent) == 0) {
 	    *offset = i;
 	    return 0;
 	}
@@ -126,10 +121,6 @@ install_first_free_syscall (int *offset,
  * Try AFS_SYSCALL first, if that fails, any free slot
  */
 
-/* XXX */
-
-#define AFS_SYSCALL 210
-
 int
 xfs_install_syscalls(void)
 {
@@ -154,16 +145,14 @@ xfs_install_syscalls(void)
 				  xfs_syscallent,
 				  &old_syscallent);
 	if (ret == 0)
-	    syscall_offset = AFS_SYSCALL;
+	    xfs_syscall_num = AFS_SYSCALL;
     }
 #endif
     if (ret != 0)
-	ret = install_first_free_syscall (&syscall_offset,
+	ret = install_first_free_syscall (&xfs_syscall_num,
 					  xfs_syscallent,
 					  &old_syscallent);
-    if (ret == 0)
-	XFSDEB(XDEBSYS, ("xfs_syscall installed at %d\n", syscall_offset));
-    else
+    if (ret != 0)
 	XFSDEB(XDEBSYS, ("failed installing xfs_syscall\n"));
     if (ret == 0) {
 	old_setgroups = sysent[SYS_setgroups];
@@ -179,17 +168,14 @@ xfs_install_syscalls(void)
 int
 xfs_uninstall_syscalls(void)
 {
-    if (syscall_offset) {
-	sysent[syscall_offset] = old_syscallent;
+    if (xfs_syscall_num) {
+	sysent[xfs_syscall_num] = old_syscallent;
+	sysent[SYS_setgroups] = old_setgroups;
 #ifdef HAVE_KERNEL_AOUT_SYSENT
-	aout_sysent[syscall_offset] = old_syscallent;
+	aout_sysent[xfs_syscall_num] = old_syscallent;
+	aout_sysent[SYS_setgroups] = old_setgroups;
 #endif
     }
-    sysent[SYS_setgroups] = old_setgroups;
-#ifdef HAVE_KERNEL_AOUT_SYSENT
-    aout_sysent[SYS_setgroups] = old_setgroups;
-#endif
-
     return 0;
 }
 
