@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  *
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- *
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -49,10 +44,11 @@
 #include <xfs/xfs_syscalls.h>
 #include <xfs/xfs_vnodeops.h>
 
-RCSID("$Id: xfs_vnodeops-common.c,v 1.34 1999/07/07 16:51:12 art Exp $");
+RCSID("$Id: xfs_vnodeops-common.c,v 1.66 2000/11/16 06:16:41 lha Exp $");
 
 int
-xfs_open_valid(struct vnode * vp, struct ucred * cred, u_int tok)
+xfs_open_valid(struct vnode *vp, struct ucred *cred, struct proc *p,
+	       u_int tok)
 {
     struct xfs *xfsp = XFS_FROM_VNODE(vp);
     struct xfs_node *xn = VNODE_TO_XNODE(vp);
@@ -69,7 +65,9 @@ xfs_open_valid(struct vnode * vp, struct ucred * cred, u_int tok)
 	    msg.cred.pag = xfs_get_pag(cred);
 	    msg.handle = xn->handle;
 	    msg.tokens = tok;
-	    error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
+
+	    error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg), p);
+
 	    if (error == 0)
 		error = ((struct xfs_message_wakeup *) & msg)->error;
 	} else {
@@ -84,12 +82,13 @@ done:
 }
 
 int
-xfs_attr_valid(struct vnode * vp, struct ucred * cred, u_int tok)
+xfs_attr_valid(struct vnode *vp, struct ucred *cred, struct proc *p,
+	       u_int tok)
 {
     struct xfs *xfsp = XFS_FROM_VNODE(vp);
     struct xfs_node *xn = VNODE_TO_XNODE(vp);
     int error = 0;
-    pag_t pag = xfs_get_pag(cred);
+    xfs_pag_t pag = xfs_get_pag(cred);
 
     do {
 	if (!XFS_TOKEN_GOT(xn, tok)) {
@@ -99,7 +98,7 @@ xfs_attr_valid(struct vnode * vp, struct ucred * cred, u_int tok)
 	    msg.cred.uid = cred->cr_uid;
 	    msg.cred.pag = pag;
 	    msg.handle = xn->handle;
-	    error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
+	    error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg), p);
 	    if (error == 0)
 		error = ((struct xfs_message_wakeup *) & msg)->error;
 	} else {
@@ -112,13 +111,13 @@ done:
 }
 
 int
-xfs_fetch_rights(struct vnode * vp, struct ucred * cred)
+xfs_fetch_rights(struct vnode *vp, struct ucred *cred, struct proc *p)
 {
     struct xfs *xfsp = XFS_FROM_VNODE(vp);
     struct xfs_node *xn = VNODE_TO_XNODE(vp);
     int error = 0;
 
-    pag_t pag = xfs_get_pag(cred);
+    xfs_pag_t pag = xfs_get_pag(cred);
 
     do {
 	if (!xfs_has_pag(xn, pag)) {
@@ -128,7 +127,7 @@ xfs_fetch_rights(struct vnode * vp, struct ucred * cred)
 	    msg.cred.uid = cred->cr_uid;
 	    msg.cred.pag = pag;
 	    msg.handle = xn->handle;
-	    error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
+	    error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg), p);
 	    if (error == 0)
 		error = ((struct xfs_message_wakeup *) & msg)->error;
 	} else {
@@ -142,7 +141,8 @@ done:
 }
 
 int
-xfs_data_valid(struct vnode * vp, struct ucred * cred, u_int tok)
+xfs_data_valid(struct vnode *vp, struct ucred *cred, struct proc *p,
+	       u_int tok)
 {
     struct xfs *xfsp = XFS_FROM_VNODE(vp);
     struct xfs_node *xn = VNODE_TO_XNODE(vp);
@@ -157,7 +157,9 @@ xfs_data_valid(struct vnode * vp, struct ucred * cred, u_int tok)
 	    msg.cred.pag = xfs_get_pag(cred);
 	    msg.handle = xn->handle;
 	    msg.tokens = tok;
-	    error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
+
+	    error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg), p);
+
 	    if (error == 0)
 		error = ((struct xfs_message_wakeup *) & msg)->error;
 	} else {
@@ -169,14 +171,46 @@ done:
     return error;
 }
 
+int
+xfs_open_common(struct vnode *vp,
+		int mode,
+		struct ucred *cred,
+		struct proc *p)
+{
+    struct xfs_node *xn = VNODE_TO_XNODE(vp);
+
+    XFSDEB(XDEBVNOPS, ("xfs_open\n"));
+
+    if (mode & FWRITE) {
+	if (xn->cred)
+	    crfree (xn->cred);
+	crhold (cred);
+	xn->cred = cred;
+	return xfs_open_valid(vp, cred, p, XFS_OPEN_NW);
+    } else {
+	return xfs_open_valid(vp, cred, p, XFS_OPEN_NR);
+    }
+}
+
 static int
-do_fsync(struct xfs * xfsp,
-	 struct xfs_node * xn,
-	 struct ucred * cred,
+do_fsync(struct xfs *xfsp,
+	 struct xfs_node *xn,
+	 struct ucred *cred,
+	 struct proc *p,
 	 u_int flag)
 {
     int error;
     struct xfs_message_putdata msg;
+#if 0
+    struct vnode *vp = XNODE_TO_VNODE(xn);
+    struct vnode *t  = DATA_FROM_XNODE(xn);
+
+    vinvalbuf (vp, V_SAVE, cred, p, 0, 0);
+
+    xfs_vfs_writelock(t, p);
+    vinvalbuf(t, V_SAVE, cred, p, 0, 0);
+    xfs_vfs_unlock(t, p);
+#endif
 
     msg.header.opcode = XFS_MSG_PUTDATA;
     if (cred != NOCRED) {
@@ -190,7 +224,8 @@ do_fsync(struct xfs * xfsp,
     vattr2xfs_attr(&xn->attr, &msg.attr);
     msg.flag   = flag;
 
-    error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
+    error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg), p);
+
     if (error == 0)
 	error = ((struct xfs_message_wakeup *) & msg)->error;
 
@@ -208,7 +243,7 @@ xfs_fsync_common(struct vnode *vp, struct ucred *cred,
     struct xfs_node *xn = VNODE_TO_XNODE(vp);
     int error = 0;
 
-    XFSDEB(XDEBVNOPS, ("xfs_fsync: %p\n", vp));
+    XFSDEB(XDEBVNOPS, ("xfs_fsync: %lx\n", (unsigned long)vp));
 
     /*
      * It seems that fsync is sometimes called after reclaiming a node.
@@ -220,8 +255,20 @@ xfs_fsync_common(struct vnode *vp, struct ucred *cred,
 	return 0;
     }
     
-    if (xn->flags & XFS_DATA_DIRTY)
-	error = do_fsync(xfsp, xn, cred, XFS_WRITE);
+#ifdef __APPLE__
+    ubc_pushdirty(vp);
+#endif
+    if (xn->flags & XFS_DATA_DIRTY) {
+#ifdef FSYNC_RECLAIM
+	/* writing back the data from this vnode failed */
+	if (waitfor & FSYNC_RECLAIM) {
+	    printf("xfs_fsync: data lost, failed to write back\n");
+	    xn->flags &= ~XFS_DATA_DIRTY;
+	    return 0;
+	}
+#endif    
+	error = do_fsync(xfsp, xn, cred, proc, XFS_WRITE | XFS_FSYNC);
+    }
 
     return error;
 }
@@ -234,22 +281,32 @@ xfs_close_common(struct vnode *vp, int fflag,
     struct xfs_node *xn = VNODE_TO_XNODE(vp);
     int error = 0;
     
-    XFSDEB(XDEBVNOPS, ("xfs_close cred = %p\n", cred));
+    XFSDEB(XDEBVNOPS,
+	   ("xfs_close cred = %lx, fflag = %x, xn->flags = %x\n",
+	    (unsigned long)cred, fflag, xn->flags));
 
     if (fflag & FWRITE && xn->flags & XFS_DATA_DIRTY)
-	error = do_fsync(xfsp, xn, cred, XFS_WRITE);
+	error = do_fsync(xfsp, xn, cred, proc, XFS_WRITE);
 
     return error;
 }
 
 int
-xfs_read_common(struct vnode *vp, struct uio *uio, int ioflag, struct ucred *cred)
+xfs_read_common(struct vnode *vp, struct uio *uio, int ioflag,
+		struct ucred *cred)
 {
     int error = 0;
+    int i;
 
     XFSDEB(XDEBVNOPS, ("xfs_read\n"));
 
-    error = xfs_data_valid(vp, cred, XFS_DATA_R);
+    error = xfs_data_valid(vp, cred, xfs_uio_to_proc(uio), XFS_DATA_R);
+
+    XFSDEB(XDEBVNOPS, ("xfs_read: iovcnt: %d\n", uio->uio_iovcnt));
+    for (i = 0; i < uio->uio_iovcnt; i++)
+      XFSDEB(XDEBVNOPS, ("  base: %lx len: %d\n",
+			 (unsigned long)uio->uio_iov[i].iov_base,
+			 uio->uio_iov[i].iov_len));
 
     if (error == 0) {
 	struct vnode *t = DATA_FROM_VNODE(vp);
@@ -257,18 +314,29 @@ xfs_read_common(struct vnode *vp, struct uio *uio, int ioflag, struct ucred *cre
 	xfs_vfs_readlock(t, xfs_uio_to_proc(uio));
 	xfs_vop_read(t, uio, ioflag, cred, error);
 	xfs_vfs_unlock(t, xfs_uio_to_proc(uio));
+
+	if (uio->uio_iovcnt && uio->uio_iov[0].iov_len > 0)
+	    XFSDEB(XDEBVNOPS, ("xfs_read: byte: %d\n",
+			       ((char*)uio->uio_iov[0].iov_base)[0]));
     }
+
+    XFSDEB(XDEBVNOPS, ("xfs_read offset: %lu resid: %d\n",
+		       (unsigned long)uio->uio_offset,
+		       uio->uio_resid));
+    XFSDEB(XDEBVNOPS, ("xfs_read error: %d\n", error));
+
     return error;
 }
 
 int
-xfs_write_common(struct vnode *vp, struct uio *uiop, int ioflag, struct ucred *cred)
+xfs_write_common(struct vnode *vp, struct uio *uiop, int ioflag,
+		 struct ucred *cred)
 {
     int error = 0;
 
     XFSDEB(XDEBVNOPS, ("xfs_write\n"));
 
-    error = xfs_data_valid(vp, cred, XFS_DATA_W);
+    error = xfs_data_valid(vp, cred, xfs_uio_to_proc(uiop), XFS_DATA_W);
 
     if (error == 0) {
 	struct xfs_node *xn = VNODE_TO_XNODE(vp);
@@ -284,11 +352,7 @@ xfs_write_common(struct vnode *vp, struct uio *uiop, int ioflag, struct ucred *c
 	if (error2 == 0) {
 	    xn->attr.va_size  = sub_attr.va_size;
 	    xn->attr.va_mtime = sub_attr.va_mtime;
-#ifdef UVM
-	    uvm_vnp_setsize(vp, sub_attr.va_size);
-#elif HAVE_KERNEL_VNODE_PAGER_SETSIZE
-	    vnode_pager_setsize(vp, sub_attr.va_size);
-#endif
+	    xfs_set_vp_size(vp, sub_attr.va_size);
 	}
 	xfs_vfs_unlock(t, xfs_uio_to_proc(uiop));
     }
@@ -297,7 +361,8 @@ xfs_write_common(struct vnode *vp, struct uio *uiop, int ioflag, struct ucred *c
 }
 
 int
-xfs_getattr_common(struct vnode *vp, struct vattr *vap, struct ucred *cred)
+xfs_getattr_common(struct vnode *vp, struct vattr *vap,
+		   struct ucred *cred, struct proc *p)
 {
     int error = 0;
 
@@ -305,14 +370,15 @@ xfs_getattr_common(struct vnode *vp, struct vattr *vap, struct ucred *cred)
 
     XFSDEB(XDEBVNOPS, ("xfs_getattr\n"));
 
-    error = xfs_attr_valid(vp, cred, XFS_ATTR_R);
+    error = xfs_attr_valid(vp, cred, p, XFS_ATTR_R);
     if (error == 0)
 	*vap = xn->attr;
     return error;
 }
 
 int
-xfs_setattr_common(struct vnode *vp, struct vattr *vap, struct ucred *cred)
+xfs_setattr_common(struct vnode *vp, struct vattr *vap,
+		   struct ucred *cred, struct proc *p)
 {
     struct xfs *xfsp = XFS_FROM_VNODE(vp);
     struct xfs_node *xn = VNODE_TO_XNODE(vp);
@@ -322,13 +388,13 @@ xfs_setattr_common(struct vnode *vp, struct vattr *vap, struct ucred *cred)
 
 #define CHECK_XFSATTR(A, cast) (vap->A == cast VNOVAL || vap->A == xn->attr.A)
 	if (CHECK_XFSATTR(va_mode,(mode_t)) &&
-	    CHECK_XFSATTR(va_nlink,) &&
-	    CHECK_XFSATTR(va_size,) &&
-	    CHECK_XFSATTR(va_uid,) &&
-	    CHECK_XFSATTR(va_gid,) &&
-	    CHECK_XFSATTR(va_mtime.tv_sec,) &&
-	    CHECK_XFSATTR(va_fileid,) &&
-	    CHECK_XFSATTR(va_type,))
+	    CHECK_XFSATTR(va_nlink,(short)) &&
+	    CHECK_XFSATTR(va_size,(u_quad_t)) &&
+	    CHECK_XFSATTR(va_uid,(uid_t)) &&
+	    CHECK_XFSATTR(va_gid,(gid_t)) &&
+	    CHECK_XFSATTR(va_mtime.tv_sec,(unsigned int)) &&
+	    CHECK_XFSATTR(va_fileid,(long)) &&
+	    CHECK_XFSATTR(va_type,(enum vtype)))
 		return 0;		/* Nothing to do */
 #undef CHECK_XFSATTR
 
@@ -350,9 +416,25 @@ xfs_setattr_common(struct vnode *vp, struct vattr *vap, struct ucred *cred)
 	}
 	msg.handle = xn->handle;
 	vattr2xfs_attr(vap, &msg.attr);
+	if (XFS_TOKEN_GOT(xn, XFS_DATA_R)) {
+	    if (vp->v_type == VREG) {
+		if (vap->va_size != (u_quad_t)VNOVAL)
+		    XA_SET_SIZE(&msg.attr, vap->va_size);
+		else
+		    XA_SET_SIZE(&msg.attr, xn->attr.va_size);
+#ifdef __APPLE__
+		if (UBCINFOEXISTS(vp))
+		    ubc_setsize(vp, msg.attr.xa_size);
+#endif
+	    }
+	    if (vap->va_mtime.tv_sec != (unsigned int)VNOVAL)
+		XA_SET_MTIME(&msg.attr, vap->va_mtime.tv_sec);
+	    else
+		XA_SET_MTIME(&msg.attr, xn->attr.va_mtime.tv_sec);
+	}
 
 	XFS_TOKEN_CLEAR(xn, XFS_ATTR_VALID, XFS_ATTR_MASK);
-	error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
+	error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg), p);
 	if (error == 0)
 	    error = ((struct xfs_message_wakeup *) & msg)->error;
     }
@@ -379,14 +461,15 @@ check_rights (u_char rights, int mode)
 }
 
 int
-xfs_access_common(struct vnode *vp, int mode, struct ucred *cred)
+xfs_access_common(struct vnode *vp, int mode, struct ucred *cred,
+		  struct proc *p)
 {
     int error = 0;
-    pag_t pag = xfs_get_pag(cred);
+    xfs_pag_t pag = xfs_get_pag(cred);
 
     XFSDEB(XDEBVNOPS, ("xfs_access mode = 0%o\n", mode));
 
-    error = xfs_attr_valid(vp, cred, XFS_ATTR_R);
+    error = xfs_attr_valid(vp, cred, p, XFS_ATTR_R);
     if (error == 0) {
 	struct xfs_node *xn = VNODE_TO_XNODE(vp);
 	int i;
@@ -398,7 +481,7 @@ xfs_access_common(struct vnode *vp, int mode, struct ucred *cred)
 
 	XFSDEB(XDEBVNOPS, ("xfs_access anonaccess failed\n"));
 
-	xfs_fetch_rights(vp, cred); /* ignore error */
+	xfs_fetch_rights(vp, cred, p); /* ignore error */
 
 	error = EACCES;		/* default to EACCES if pag isn't in xn->id */
 
@@ -429,13 +512,20 @@ xfs_lookup_common(struct vnode *dvp,
 
     *vpp = NULL;
 
+    if (cnp->cn_namelen >= XFS_MAX_NAME)
+	return ENAMETOOLONG;
+	
     if (dvp->v_type != VDIR)
 	return ENOTDIR;
-    
- again:
 
+    if (cnp->cn_namelen == 1 && cnp->cn_nameptr[0] == '.') {
+	*vpp = dvp;
+	VREF(*vpp);
+	return 0;
+    }
+    
     do {
-	xfs_vop_access(dvp, VEXEC, cred, xfs_cnp_to_proc(cnp), error);
+	xfs_vop_access(dvp, VEXEC, cred, proc, error);
 	if (error != 0)
 	    goto done;
 
@@ -463,23 +553,19 @@ xfs_lookup_common(struct vnode *dvp,
 		msg.cred.pag = XFS_ANONYMOUSID;
 	    }
 	    msg.parent_handle = d->handle;
-
-	    bcopy(cnp->cn_nameptr, msg.name, cnp->cn_namelen);
+	    memcpy(msg.name, cnp->cn_nameptr, cnp->cn_namelen);
 	    msg.name[cnp->cn_namelen] = '\0';
-
-	    error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
-
+	    error = xfs_message_rpc(xfsp->fd, &msg.header,
+				    sizeof(msg), proc);
 	    if (error == 0)
 		error = ((struct xfs_message_wakeup *) & msg)->error;
 	    if(error == ENOENT && cnp->cn_nameiop != CREATE) {
-		XFSDEB(XDEBVNOPS, ("xfs_lookup: neg cache %p (%s, %ld)\n",
-				   dvp,
+		XFSDEB(XDEBVNOPS, ("xfs_lookup: neg cache %lx (%s, %ld)\n",
+				   (unsigned long)dvp,
 				   cnp->cn_nameptr, cnp->cn_namelen));
 		xfs_dnlc_enter (dvp, cnp, NULL);
 	    }
 	} else if (error == -1) {
-	    if (xfs_do_vget(*vpp, LK_EXCLUSIVE, xfs_cnp_to_proc(cnp)))
-		    goto again;
 	    error = 0;
 	    goto done;
 	}
@@ -489,36 +575,26 @@ done:
     return error;
 }
 
-int
-xfs_lookup_name(struct vnode *dvp, 
-		const char *name,
-		struct proc *proc,
-		struct ucred *cred,
-		struct vnode **vpp)
-{
-    xfs_componentname cn;
-
-    xfs_cnp_init (&cn, name, NULL, dvp, proc, cred, CREATE);
-    return xfs_lookup_common (dvp, &cn, vpp);
-}    
-
 int
 xfs_create_common(struct vnode *dvp,
 		  const char *name,
 		  struct vattr *vap, 
-		  struct ucred *cred)
+		  struct ucred *cred,
+		  struct proc *p)
 {
     struct xfs *xfsp = XFS_FROM_VNODE(dvp);
     struct xfs_node *xn = VNODE_TO_XNODE(dvp);
     int error = 0;
 
-    XFSDEB(XDEBVNOPS, ("xfs_create: (%s)\n", name));
+    XFSDEB(XDEBVNOPS, ("xfs_create: (%lx, %s)\n",
+		       (unsigned long)dvp, name));
     {
 	struct xfs_message_create msg;
 
 	msg.header.opcode = XFS_MSG_CREATE;
 	msg.parent_handle = xn->handle;
-	strncpy(msg.name, name, 256);
+	if (strlcpy(msg.name, name, sizeof(msg.name)) >= XFS_MAX_NAME)
+	    return ENAMETOOLONG;
 	vattr2xfs_attr(vap, &msg.attr);
 
 	msg.mode = 0;		       /* XXX - mode */
@@ -530,7 +606,9 @@ xfs_create_common(struct vnode *dvp,
 	    msg.cred.pag = XFS_ANONYMOUSID;
 	}
 
-	error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
+
+	error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg), p);
+
 	if (error == 0)
 	    error = ((struct xfs_message_wakeup *) & msg)->error;
     }
@@ -547,7 +625,8 @@ int
 xfs_remove_common(struct vnode *dvp,
 		  struct vnode *vp,
 		  const char *name,
-		  struct ucred *cred)
+		  struct ucred *cred,
+		  struct proc *p)
 {
     struct xfs *xfsp  = XFS_FROM_VNODE(dvp);
     struct xfs_node *xn = VNODE_TO_XNODE(dvp);
@@ -558,10 +637,13 @@ xfs_remove_common(struct vnode *dvp,
 
     msg.header.opcode = XFS_MSG_REMOVE;
     msg.parent_handle = xn->handle;
-    strncpy(msg.name, name, 256);
     msg.cred.uid = cred->cr_uid;
     msg.cred.pag = xfs_get_pag(cred);
-    error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
+    
+    if (strlcpy(msg.name, name, sizeof(msg.name)) >= XFS_MAX_NAME)
+	error = ENAMETOOLONG;
+    else
+	error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg), p);
     if (error == 0)
 	error = ((struct xfs_message_wakeup *) &msg)->error;
 
@@ -586,7 +668,8 @@ xfs_rename_common(struct vnode *fdvp,
 		  struct vnode *tdvp,
 		  struct vnode *tvp,
 		  const char *tname,
-		  struct ucred *cred)
+		  struct ucred *cred,
+		  struct proc *p)
 {
     struct xfs *xfsp = XFS_FROM_VNODE(fdvp);
     int error;
@@ -603,12 +686,14 @@ xfs_rename_common(struct vnode *fdvp,
 
 	msg.header.opcode = XFS_MSG_RENAME;
 	msg.old_parent_handle = VNODE_TO_XNODE(fdvp)->handle;
-	strncpy(msg.old_name, fname, 256);
+	if (strlcpy(msg.old_name, fname, sizeof(msg.old_name)) >= XFS_MAX_NAME)
+	    return ENAMETOOLONG;
 	msg.new_parent_handle = VNODE_TO_XNODE(tdvp)->handle;
-	strncpy(msg.new_name, tname, 256);
+	if (strlcpy(msg.new_name, tname, sizeof(msg.new_name)) >= XFS_MAX_NAME)
+	    return ENAMETOOLONG;
 	msg.cred.uid = cred->cr_uid;
 	msg.cred.pag = xfs_get_pag(cred);
-	error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
+	error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg), p);
 	if (error == 0)
 	    error = ((struct xfs_message_wakeup *) &msg)->error;
 
@@ -623,7 +708,8 @@ int
 xfs_mkdir_common(struct vnode *dvp, 
 		 const char *name,
 		 struct vattr *vap, 
-		 struct ucred *cred)
+		 struct ucred *cred,
+		 struct proc *p)
 {
     struct xfs *xfsp = XFS_FROM_VNODE(dvp);
     struct xfs_node *xn = VNODE_TO_XNODE(dvp);
@@ -635,7 +721,8 @@ xfs_mkdir_common(struct vnode *dvp,
 
 	msg.header.opcode = XFS_MSG_MKDIR;
 	msg.parent_handle = xn->handle;
-	strncpy(msg.name, name, 256);
+	if (strlcpy(msg.name, name, sizeof(msg.name)) >= XFS_MAX_NAME)
+	    return ENAMETOOLONG;
 	vattr2xfs_attr(vap, &msg.attr);
 	if (cred != NOCRED) {
 	    msg.cred.uid = cred->cr_uid;
@@ -644,7 +731,7 @@ xfs_mkdir_common(struct vnode *dvp,
 	    msg.cred.uid = 0;
 	    msg.cred.pag = XFS_ANONYMOUSID;
 	}
-	error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
+	error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg), p);
 	if (error == 0)
 	    error = ((struct xfs_message_wakeup *) & msg)->error;
     }
@@ -656,7 +743,8 @@ int
 xfs_rmdir_common(struct vnode *dvp,
 		 struct vnode *vp,
 		 const char *name,
-		 struct ucred *cred)
+		 struct ucred *cred,
+		 struct proc *p)
 {
     struct xfs *xfsp  = XFS_FROM_VNODE(dvp);
     struct xfs_node *xn = VNODE_TO_XNODE(dvp);
@@ -667,10 +755,12 @@ xfs_rmdir_common(struct vnode *dvp,
 
     msg.header.opcode = XFS_MSG_RMDIR;
     msg.parent_handle = xn->handle;
-    strncpy(msg.name, name, 256);
     msg.cred.uid = cred->cr_uid;
     msg.cred.pag = xfs_get_pag(cred);
-    error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
+    if (strlcpy(msg.name, name, sizeof(msg.name)) >= XFS_MAX_NAME)
+	error = ENAMETOOLONG;
+    else
+	error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg), p);
     if (error == 0)
 	error = ((struct xfs_message_wakeup *) &msg)->error;
 
@@ -685,6 +775,8 @@ xfs_rmdir_common(struct vnode *dvp,
     vput(dvp);
 #endif
 
+    XFSDEB(XDEBVNOPS, ("xfs_rmdir error: %d\n", error));
+
     return error;
 }
 
@@ -692,6 +784,7 @@ int
 xfs_readdir_common(struct vnode *vp, 
 		   struct uio *uiop, 
 		   struct ucred *cred,
+		   struct proc *p,
 		   int *eofflag)
 {
     int error = 0;
@@ -700,14 +793,20 @@ xfs_readdir_common(struct vnode *vp,
 
     if(eofflag)
 	*eofflag = 0;
-    error = xfs_data_valid(vp, cred, XFS_DATA_R);
+    error = xfs_data_valid(vp, cred, xfs_uio_to_proc(uiop), XFS_DATA_R);
     if (error == 0) {
 	struct vnode *t = DATA_FROM_VNODE(vp);
 
 	xfs_vfs_readlock(t, xfs_uio_to_proc(uiop));
 	xfs_vop_read(t, uiop, 0, cred, error);
-	if (eofflag)
-	    *eofflag = VNODE_TO_XNODE(vp)->attr.va_size <= uiop->uio_offset;
+	if (eofflag) {
+	    struct vattr t_attr;
+	    int error2;
+
+	    xfs_vop_getattr(t, &t_attr, cred, xfs_uio_to_proc(uiop), error2);
+	    if (error2 == 0)
+		*eofflag = t_attr.va_size <= uiop->uio_offset;
+	}
 	xfs_vfs_unlock(t, xfs_uio_to_proc(uiop));
     }
     return error;
@@ -717,7 +816,8 @@ int
 xfs_link_common(struct vnode *dvp, 
 		struct vnode *vp, 
 		const char *name,
-		struct ucred *cred)
+		struct ucred *cred,
+		struct proc *p)
 {
     struct xfs *xfsp = XFS_FROM_VNODE(dvp);
     struct xfs_node *xn = VNODE_TO_XNODE(dvp);
@@ -730,11 +830,12 @@ xfs_link_common(struct vnode *dvp,
     msg.header.opcode = XFS_MSG_LINK;
     msg.parent_handle = xn->handle;
     msg.from_handle   = xn2->handle;
-    strncpy(msg.name, name, 256);
+    if (strlcpy(msg.name, name, sizeof(msg.name)) >= XFS_MAX_NAME)
+	return ENAMETOOLONG;
     msg.cred.uid = cred->cr_uid;
     msg.cred.pag = xfs_get_pag(cred);
 
-    error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
+    error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg), p);
     if (error == 0)
 	error = ((struct xfs_message_wakeup *) & msg)->error;
     
@@ -744,40 +845,47 @@ xfs_link_common(struct vnode *dvp,
 int
 xfs_symlink_common(struct vnode *dvp,
 		   struct vnode **vpp,
-		   const char *name,
-		   struct proc *proc,
-		   struct ucred *cred,
+		   xfs_componentname *cnp,
 		   struct vattr *vap,
 		   char *target)
 {
     struct xfs *xfsp = XFS_FROM_VNODE(dvp);
     struct xfs_node *xn = VNODE_TO_XNODE(dvp);
+    struct proc *proc  = xfs_cnp_to_proc(cnp);
+    struct ucred *cred = xfs_proc_to_cred(proc);
     struct xfs_message_symlink msg;
+    const char *name = cnp->cn_nameptr;
     int error = 0;
 
     XFSDEB(XDEBVNOPS, ("xfs_symlink: %s\n", name));
 
     msg.header.opcode = XFS_MSG_SYMLINK;
     msg.parent_handle = xn->handle;
-    strncpy(msg.name, name, sizeof(msg.name));
-    msg.name[sizeof(msg.name) - 1] = '\0';
     vattr2xfs_attr(vap, &msg.attr);
     msg.cred.uid = cred->cr_uid;
     msg.cred.pag = xfs_get_pag(cred);
-    strncpy (msg.contents, target, sizeof(msg.contents));
-    msg.contents[sizeof(msg.contents) - 1] = '\0';
-
-    error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
+    if (strlcpy (msg.contents, target, sizeof(msg.contents)) >= XFS_MAX_SYMLINK_CONTENT) {
+	error = ENAMETOOLONG;
+	goto done;
+    }
+    if (strlcpy(msg.name, name, sizeof(msg.name)) >= XFS_MAX_NAME) {
+	error = ENAMETOOLONG;
+	goto done;
+    }
+    error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg), proc);
     if (error == 0)
 	error = ((struct xfs_message_wakeup *) & msg)->error;
 
     if (error == 0) {
-	error = xfs_lookup_name(dvp, name, proc, cred, vpp);
+	error = xfs_lookup_common(dvp, cnp, vpp);
+#if !defined(__FreeBSD__) || __FreeBSD_version < 400012
 	if (error == 0)
 	    vput (*vpp);
+#endif
     }
 
-#if !defined(__FreeBSD__) || __FreeBSD_version < 300000
+ done:
+#if !defined(__FreeBSD__)
     vput(dvp);
 #endif
 
@@ -791,7 +899,7 @@ xfs_readlink_common(struct vnode *vp, struct uio *uiop, struct ucred *cred)
 
     XFSDEB(XDEBVNOPS, ("xfs_readlink\n"));
 
-    error = xfs_data_valid(vp, cred, XFS_DATA_R);
+    error = xfs_data_valid(vp, cred, xfs_uio_to_proc(uiop), XFS_DATA_R);
     if (error == 0) {
 	struct vnode *t = DATA_FROM_VNODE(vp);
 
@@ -805,11 +913,11 @@ xfs_readlink_common(struct vnode *vp, struct uio *uiop, struct ucred *cred)
 int
 xfs_inactive_common(struct vnode *vp, struct proc *p)
 {
-    struct xfs_message_inactivenode msg;
-    struct xfs *xfsp = XFS_FROM_VNODE(vp);
+    int error;
     struct xfs_node *xn = VNODE_TO_XNODE(vp);
 
-    XFSDEB(XDEBVNOPS, ("xfs_inactive, %p\n", vp));
+    XFSDEB(XDEBVNOPS, ("xfs_inactive, %lx\n",
+		       (unsigned long)vp));
 
     /*
      * This seems rather bogus, but sometimes we get an already
@@ -821,17 +929,25 @@ xfs_inactive_common(struct vnode *vp, struct proc *p)
 	return 0;
     }
 
-    xn->tokens = 0;
-    msg.header.opcode = XFS_MSG_INACTIVENODE;
-    msg.handle = xn->handle;
-    msg.flag   = XFS_NOREFS;
-    xfs_message_send(xfsp->fd, &msg.header, sizeof(msg));
+    /* xn->cred not set -> NOCRED */
+
+    error = xfs_fsync_common(vp, xn->cred, /* XXX */ 0, p);
+    if (error) {
+	printf ("xfs_inactive: failed writing back data: %d\n", error);
+    }
 
 #ifndef __osf__
     xfs_vfs_unlock(vp, p);
+    /* If this node is no longer valid, recycle immediately. */
+    if (!XFS_TOKEN_GOT(xn, XFS_ATTR_R | XFS_ATTR_W)
+	|| (xn->flags & XFS_STALE) == XFS_STALE) {
+        XFSDEB(XDEBVNOPS, ("xfs_inactive: vrecycle\n"));
+        vrecycle(vp, 0, p);
+    }
 #else
     /* XXX ? */
 #endif
+    xn->flags &= ~XFS_STALE;
 
     XFSDEB(XDEBVNOPS, ("return: xfs_inactive\n"));
 
@@ -845,7 +961,18 @@ xfs_reclaim_common(struct vnode *vp)
     struct xfs *xfsp = XFS_FROM_VNODE(vp);
     struct xfs_node *xn = VNODE_TO_XNODE(vp);
 
-    XFSDEB(XDEBVNOPS, ("xfs_reclaim: %p\n", vp));
+    XFSDEB(XDEBVNOPS, ("xfs_reclaim: %lx",
+		       (unsigned long)vp));
+
+    XFS_TOKEN_CLEAR(xn,
+		    ~0,
+		    XFS_OPEN_MASK | XFS_ATTR_MASK |
+		    XFS_DATA_MASK | XFS_LOCK_MASK);
+    /* Release, data if we still have it. */
+    if (DATA_FROM_XNODE(xn) != 0) {
+        vrele(DATA_FROM_XNODE(xn));
+	DATA_FROM_XNODE(xn) = 0;
+    }
 
     msg.header.opcode = XFS_MSG_INACTIVENODE;
     msg.handle = xn->handle;
@@ -854,7 +981,6 @@ xfs_reclaim_common(struct vnode *vp)
 
     xfs_dnlc_purge(vp);
     free_xfs_node(xn);
-
     return 0;
 }
 
@@ -890,7 +1016,7 @@ xfs_advlock_common(struct vnode *dvp,
 	    msg.cred.uid = 0;
 	    msg.cred.pag = XFS_ANONYMOUSID;
 	}
-	error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
+	error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg), p);
 	if (error == 0)
 	    error = ((struct xfs_message_wakeup *) & msg)->error;
     }
