@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -42,28 +37,57 @@
 #include <xfs/xfs_common.h>
 #include <xfs/xfs_fs.h>
 #include <xfs/xfs_deb.h>
-#include <xfs/xfs_cache.h>
 
-RCSID("$Id: xfs_node.c,v 1.31.2.1 1999/08/21 23:44:38 map Exp $");
+RCSID("$Id: xfs_node.c,v 1.51.2.1 2001/01/30 01:48:11 lha Exp $");
+
+/*
+ * Copy the attributes from `attr' into `inode', setting the fields
+ * that weren't set in `attr' to reasonable defaults.
+ */
 
 void
-xfs_attr2inode(const struct xfs_attr *attr, struct inode *inode)
+xfs_attr2inode(const struct xfs_attr *attr, struct inode *inode,
+	       int clear_node)
 {
-    inode->i_mode = XA_VALID_MODE(attr) ? attr->xa_mode : 0;
-    inode->i_uid = XA_VALID_UID(attr) ? attr->xa_uid : 0;
-    inode->i_gid = XA_VALID_GID(attr) ? attr->xa_gid : 0;
-    inode->i_nlink = XA_VALID_NLINK(attr) ? attr->xa_nlink : 1;
-    inode->i_size = XA_VALID_SIZE(attr) ? attr->xa_size : 0;
-    inode->i_atime = XA_VALID_ATIME(attr) ? attr->xa_atime : 0;
-    inode->i_mtime = XA_VALID_MTIME(attr) ? attr->xa_mtime : 0;
-    inode->i_ctime = XA_VALID_CTIME(attr) ? attr->xa_ctime : 0;
-    inode->i_blocks = inode->i_size / I_BLOCKS_UNIT;
-#ifdef HAVE_GET_EMPTY_INODE
+    if (clear_node) {
+	inode->i_mode   = 0;
+	inode->i_uid    = 0;
+	inode->i_gid    = 0;
+	inode->i_nlink  = 1;
+	inode->i_size   = 0;
+	inode->i_atime  = 0;
+	inode->i_mtime  = 0;
+	inode->i_ctime  = 0;
+	inode->i_blocks = 0;
+    }
+    if (XA_VALID_MODE(attr))
+	inode->i_mode   = attr->xa_mode;
+    if (XA_VALID_UID(attr))
+	inode->i_uid    = attr->xa_uid;
+    if (XA_VALID_GID(attr))
+	inode->i_gid    = attr->xa_gid;
+    if (XA_VALID_NLINK(attr))
+	inode->i_nlink  = attr->xa_nlink;
+    if (XA_VALID_SIZE(attr)) {
+	inode->i_size   = attr->xa_size;
+    inode->i_blocks = inode->i_size >> I_BLOCKS_BITS; /* / I_BLOCKS_UNIT */
+    }
+    if (XA_VALID_ATIME(attr))
+	inode->i_atime  = attr->xa_atime;
+    if (XA_VALID_MTIME(attr))
+	inode->i_mtime  = attr->xa_mtime;
+    if (XA_VALID_CTIME(attr))
+	inode->i_ctime  = attr->xa_ctime;
     if (XA_VALID_FILEID(attr))
 	inode->i_ino = attr->xa_fileid;
-#endif
 }
 
+/*
+ * Allocate a new inode (of the file system identified by `sb') and
+ * return it, associated with `newnode'.  Return the `inode' or NULL.
+ * The reference count on `inode' is incremented.
+ */
+
 static struct inode *
 xfs_iget(struct super_block *sb, struct xfs_msg_node *node,
 	 struct xfs_node *newnode)
@@ -76,7 +100,6 @@ xfs_iget(struct super_block *sb, struct xfs_msg_node *node,
      	printk(KERN_EMERG "XFS Panic: xfs_iget: super block is NULL\n");
      	return NULL;
     }
-#ifdef HAVE_GET_EMPTY_INODE
     inode = get_empty_inode ();
     if (inode == NULL) {
     	printk(KERN_EMERG "XFS Panic: xfs_iget: get_empty_inode failed\n");
@@ -84,26 +107,42 @@ xfs_iget(struct super_block *sb, struct xfs_msg_node *node,
     }
     inode->i_sb  = sb;
     inode->i_dev = sb->s_dev;
-#else
-    inode = iget (sb, (ino_t)newnode);
-    if (inode == NULL) {
-    	printk(KERN_EMERG "XFS Panic: xfs_iget: iget failed\n");
-    	return NULL;
-    }
-#endif
 
-    xfs_attr2inode (attr, inode);
-    
-    if (!XA_VALID_TYPE(attr))
-	inode->i_op = &xfs_dead_inode_operations;
-    else if (attr->xa_type == XFS_FILE_REG)
-	inode->i_op = &xfs_file_inode_operations;
-    else if (attr->xa_type == XFS_FILE_DIR)
-	inode->i_op = &xfs_dir_inode_operations;
-    else if (attr->xa_type == XFS_FILE_LNK)
-	inode->i_op = &xfs_link_inode_operations;
-    else
-	inode->i_op = &xfs_dead_inode_operations;
+    if (!XA_VALID_TYPE(attr)) {
+	inode->i_op  = &xfs_dead_inode_operations;
+#ifndef HAVE_STRUCT_INODE_OPERATIONS_DEFAULT_FILE_OPS
+	inode->i_fop = &xfs_dead_operations;
+#endif
+    } else if (attr->xa_type == XFS_FILE_REG) {
+	inode->i_op  = &xfs_file_inode_operations;
+#ifndef HAVE_STRUCT_INODE_OPERATIONS_DEFAULT_FILE_OPS
+	inode->i_fop = &xfs_file_operations;
+#endif
+#ifdef LINUX2_3
+        inode->i_mapping->a_ops = &xfs_aops;
+#endif
+    } else if (attr->xa_type == XFS_FILE_DIR) {
+	inode->i_op  = &xfs_dir_inode_operations;
+#ifndef HAVE_STRUCT_INODE_OPERATIONS_DEFAULT_FILE_OPS
+	inode->i_fop = &xfs_dir_operations;
+#endif
+#ifdef LINUX2_3
+        inode->i_mapping->a_ops = &xfs_aops;
+#endif
+    } else if (attr->xa_type == XFS_FILE_LNK) {
+	inode->i_op  = &xfs_link_inode_operations;
+#ifndef HAVE_STRUCT_INODE_OPERATIONS_DEFAULT_FILE_OPS
+	inode->i_fop = &xfs_link_operations;
+#endif
+#ifdef LINUX2_3
+	inode->i_mapping->a_ops = &xfs_aops;
+#endif
+    } else {
+	inode->i_op  = &xfs_dead_inode_operations;
+#ifndef HAVE_STRUCT_INODE_OPERATIONS_DEFAULT_FILE_OPS
+	inode->i_fop = &xfs_dead_operations;
+#endif
+    }
     
     inode->u.generic_ip = newnode;
 
@@ -111,15 +150,9 @@ xfs_iget(struct super_block *sb, struct xfs_msg_node *node,
 }
 
 /*
- * Create a new xfs_node and make a VN_HOLD()!
- *
- * Also prevents creation of duplicates. This happens
- * whenever there are more than one name to a file,
- * "." and ".." are common cases.
- */
-/*
- * xfsp = pointer to the current xfs file system (maps onto super_block)
- *
+ * Find the node identified by `node->handle' belong to the filesystem
+ * `xfsp' or create a new one.  The node is returned with incremented
+ * reference count.  Always return a valid node.
  */
 
 struct xfs_node *
@@ -156,18 +189,19 @@ new_xfs_node(struct xfs *xfsp, struct xfs_msg_node *node)
 	result->handle = node->handle;
 	result->flags = 0;
 	result->tokens = 0;
+	INIT_LIST_HEAD(&result->inactive_list);
 	
 	v=xfs_iget(XFS_TO_VFS(xfsp),node,result);
 	result->vn = v;
 	/* XXX - handle this case better */
     } else {
 	/* Node is already cached */
-	XNODE_TO_VNODE(result)->i_count++;
+	xfs_iref(XNODE_TO_VNODE(result));
     }
     
     /* Init other fields */
     result->attr = node->attr;
-    xfs_attr2inode (&result->attr, result->vn);
+    xfs_attr2inode (&result->attr, result->vn, 1);
     XFS_TOKEN_SET(result, XFS_ATTR_R, XFS_ATTR_MASK);
     memmove(result->id, node->id, sizeof(result->id));
     memmove(result->rights, node->rights, sizeof(result->rights));
@@ -175,13 +209,31 @@ new_xfs_node(struct xfs *xfsp, struct xfs_msg_node *node)
     return result;
 }
 
+/*
+ * Free the memory used by `node' and remove it from the list of nodes.
+ */
+
 void
 free_xfs_node(struct xfs_node *node)
+{
+    if (node == NULL)
+	return;
+
+    clear_xfs_node (node);
+    xfs_free(node, XFS_MEM_XNODE);
+}
+
+/*
+ * remove everything about `node'
+ */
+
+void
+clear_xfs_node(struct xfs_node *node)
 {
     struct xfs *xfsp;
     struct xfs_node *t;
     
-    XFSDEB(XDEBNODE, ("free_xfs_node starting\n"));
+    XFSDEB(XDEBNODE, ("clear_xfs_node starting\n"));
     
     if (node == NULL)
 	return;
@@ -199,48 +251,51 @@ free_xfs_node(struct xfs_node *node)
 		t->next = t->next->next;
 		goto done;
 	    }
-	printk(KERN_EMERG "XFS Panic: free_xfs_node(0x%p) failed?\n", node);
+	printk(KERN_EMERG "XFS Panic: clear_xfs_node(0x%p) failed?\n", node);
 	return;
     }
     
 done:
     /* XXX Really need to put back dirty data first. */
+    XFS_TOKEN_CLEAR(node, ~0,
+		    XFS_OPEN_MASK | XFS_ATTR_MASK |
+		    XFS_DATA_MASK | XFS_LOCK_MASK);
     if (DATA_FROM_XNODE(node)) {
-#ifndef LINUX2_1
-	iput(DATA_FROM_XNODE(node));
-	DATA_FROM_XNODE(node) = NULL;
-#else
+	XFS_RESET_I_MAPPING(XNODE_TO_VNODE(node));
 	dput(DATA_FROM_XNODE(node));
 	DATA_FROM_XNODE(node) = NULL;
-#endif /* LINUX2_1 */
     }
+    node->flags = 0;
     xfsp->nnodes--;
-    XFSDEB(XDEBNODE, ("free_xfs_node xnode: %p inode:%p\n",
+    XFSDEB(XDEBNODE, ("clear_xfs_node xnode: %p inode:%p\n",
 		      node, XNODE_TO_VNODE(node)));
     XNODE_TO_VNODE(node)->u.generic_ip = NULL;
-    xfs_free(node, XFS_MEM_XNODE);
     
-    XFSDEB(XDEBNODE, ("free_xfs_node done\n"));
+    XFSDEB(XDEBNODE, ("clear_xfs_node done\n"));
 }
 
+/*
+ * Throw all nodes of the filesystem `xfsp'.
+ */
+
 void
 free_all_xfs_nodes(struct xfs *xfsp)
 {
     struct xfs_node *xnode;
     XFSDEB(XDEBNODE, ("free_all_xfs_nodes starting\n"));
     
-#ifndef LINUX2_1
-    xfs_cache_purge();
-#endif /* LINUX2_1 */
-
     /* There might still be a few nodes out there, invalidate them */
     for (xnode = xfsp->nodes; xnode; xnode = xnode->next) {
-	xfs_invalid_xnode(xnode);
+	xfs_force_invalid_xnode(xnode);
     }
     
     XFSDEB(XDEBNODE, ("free_all_xfs_nodes done\n"));
 }
 
+/*
+ * Return the node identifed by `handlep' in `xfsp' or NULL.
+ */
+
 struct xfs_node *
 xfs_node_find(struct xfs *xfsp, xfs_handle *handlep)
 {
@@ -248,22 +303,18 @@ xfs_node_find(struct xfs *xfsp, xfs_handle *handlep)
     
     XFSDEB(XDEBNODE, ("xfs_node_find\n"));
     
-    for(x = xfsp->nodes;
-	x != NULL;
-	x = x->next)
+    for(x = xfsp->nodes; x != NULL; x = x->next)
 	if (xfs_handle_eq(&x->handle, handlep))
 	    break;
-    if(x != NULL)
-	return x;
-    else
-	return NULL;
+    return x;
 }
 
 /*
  * Returns 1 if pag has any rights set in the node
  */
+
 int
-xfs_has_pag(const struct xfs_node *xn, pag_t pag)
+xfs_has_pag(const struct xfs_node *xn, xfs_pag_t pag)
 {
     int i;
     
@@ -273,3 +324,22 @@ xfs_has_pag(const struct xfs_node *xn, pag_t pag)
     
     return 0;
 }
+
+/*
+ * Return the number of users of the node `xn'.
+ */
+
+int
+xfs_node_users(struct xfs_node *xnode)
+{
+    struct inode *inode = XNODE_TO_VNODE(xnode);
+    struct list_head *pos;
+    int users = 0;
+    
+    list_for_each(pos, &inode->i_dentry) {
+	struct dentry *dentry = list_entry(pos, struct dentry, d_alias);
+	if (xfs_dcount(dentry))
+	    users++;
+    }
+    return users;
+}
