@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -40,7 +35,6 @@
 
 #include <xfs/xfs_locl.h>
 #include <xfs/xfs_dev.h>
-#include <asm/unistd.h>
 #include <kafs.h>
 #include <xfs/xfs_syscalls.h>
 #include <xfs/xfs_common.h>
@@ -48,7 +42,7 @@
 #include <linux/smp_lock.h>
 #include <linux/smp.h>
 
-RCSID("$Id: xfs_syscalls.c,v 1.47.2.1 1999/08/21 23:45:49 map Exp $");
+RCSID("$Id: xfs_syscalls.c,v 1.64 2000/10/02 23:47:54 lha Exp $");
 
 /* 
  * Def pag:
@@ -64,25 +58,10 @@ RCSID("$Id: xfs_syscalls.c,v 1.47.2.1 1999/08/21 23:45:49 map Exp $");
 static gid_t pag_part_one = XFS_PAG1_LLIM ;
 static gid_t pag_part_two = XFS_PAG2_LLIM ;
 
-#ifndef LINUX2_1
-static int xfs_get_ngroups(void)
-{
-    int i;
-
-    for (i = 0 ; current->groups[i] != NOGROUP && i < NGROUPS; i++);
-    return i;
-}
-#endif /* LINUX2_1 */
-
 static int xfs_is_pag(void)
 {
     
-#ifndef LINUX2_1
-    if (current->groups[0] != NOGROUP &&
-	current->groups[1] != NOGROUP &&
-#else
     if (current->ngroups >= 2 &&
-#endif /* LINUX2_1 */
 	current->groups[0] >= XFS_PAG1_LLIM &&
 	current->groups[0] <= XFS_PAG1_ULIM &&
 	current->groups[1] >= XFS_PAG2_LLIM &&
@@ -94,7 +73,7 @@ static int xfs_is_pag(void)
 }
 
 
-pag_t xfs_get_pag(void)
+xfs_pag_t xfs_get_pag(void)
 {
     if (xfs_is_pag()) {
 	
@@ -105,52 +84,6 @@ pag_t xfs_get_pag(void)
 	return current->uid;
 }
 
-#ifndef LINUX2_1
-
-static int
-store_pag (gid_t part1, gid_t part2)
-{
-    int i;
-
-    if (!xfs_is_pag ()) {
-	int ngroups = xfs_get_ngroups ();
-
-	if (ngroups + 2 >= NGROUPS)
-	    return -E2BIG;
-	for (i = ngroups - 1; i >= 0; i--) {
-	    current->groups[i + 2] = current->groups[i];
-	}
-	ngroups += 2;
-	if (ngroups < NGROUPS)
-	    current->groups[ngroups] = NOGROUP;
-    }
-    current->groups[0] = part1;
-    current->groups[1] = part2;
-    return 0;
-}
-
-/* remove the current process's PAG (i.e. go back to default PAG) */
-
-static int
-unpag (void)
-{
-    int i;
-
-    while(xfs_is_pag ()) {
-	int ngroups = xfs_get_ngroups ();
-
-	for (i = 0; i < ngroups - 2; i++) {
-	    current->groups[i] = current->groups[i+2];
-	}
-	ngroups -= 2;
-	current->groups[ngroups] = NOGROUP;
-    }
-    return 0;
-}
-
-
-#else /* LINUX2_1 */
-
 static int
 store_pag (gid_t part1, gid_t part2)
 {
@@ -187,7 +120,6 @@ unpag (void)
     }
     return 0;
 }
-#endif /* LINUX2_1 */
 
 static int
 xfs_setpag_call(void)
@@ -197,11 +129,7 @@ xfs_setpag_call(void)
 
 #ifdef DEBUG
     XFSDEB(XDEBSYS, ("xfs_setpag_call groups: "));
-#ifndef LINUX2_1
-    for (i = 0 ; current->groups[i] != NOGROUP && i < NGROUPS; i++)
-#else
     for (i = 0 ; i < current->ngroups; i++)
-#endif /* LINUX2_1 */
 	XFSDEB(XDEBSYS, ("%d ",current->groups[i]));
     XFSDEB(XDEBSYS, ("\n"));
 #endif
@@ -230,14 +158,28 @@ typedef int (*sys_afs_function)(int operation,
 
 typedef int (*sys_setgroups_function)(int, gid_t *);
 
+#ifdef ARLA_NR_setgroups16
+typedef int (*sys_setgroups16_function)(int, old_gid_t *);
+#endif
+
 typedef int (*sys_call_function)(void);
 
 extern sys_call_function sys_call_table[];
+#ifdef NEED_VICEIOCTL32
+extern u_int32_t sys_call_table32[];
+#endif
 
 static sys_call_function old_afs_syscall=NULL;
+#ifdef NEED_VICEIOCTL32
+static u_int32_t old_afs_syscall32=0;
+#endif
 
 static sys_call_function old_setgroups = NULL;
 
+#ifdef ARLA_NR_setgroups16
+static sys_call_function old_setgroups16 = NULL;
+#endif
+
 /*
  * A wrapper around sys_setgroups that tries to preserve the pag. If
  * the pag wouldn't fit after setting the group list, we return E2BIG.
@@ -286,46 +228,73 @@ xfs_setgroups (int gidsetsize, gid_t *grouplist)
     }
 }
 
+#ifdef ARLA_NR_setgroups16
+
+/*
+ * Linux 2.3.39 and above has 2 setgroups() system calls on arm, i386,
+ * m68k, sh, and sparc32. We call the old one setgroups16() because it
+ * uses a 16-bit gid_t (old_gid_t).
+ * We need to fix it up too.
+ */
+
+static asmlinkage int
+xfs_setgroups16 (int gidsetsize, old_gid_t *grouplist)
+{
+    sys_setgroups16_function setgroups16 = (sys_setgroups16_function)old_setgroups16;
+
+    if (xfs_is_pag ()) {
+	gid_t part1, part2;
+	int ret;
+
+	if(gidsetsize + 2 >= NGROUPS)
+	    return -E2BIG;
+
+	part1 = current->groups[0];
+	part2 = current->groups[1];
+	ret = (*setgroups16) (gidsetsize, grouplist);
+	if (ret) {
+	    if(xfs_is_pag ())
+		store_pag (part1, part2);
+
+	    return ret;
+	}
+	return store_pag (part1, part2);
+    } else {
+	int ret;
+	ret = (*setgroups16) (gidsetsize, grouplist);
+
+	/* Prevent attaching to somebody's PAG */
+	if (xfs_is_pag ()) {
+	    unpag();
+	    return -EINVAL;
+	}
+	return ret;
+    }
+}
+#endif /* ARLA_NR_setgroups16 */
+
 struct file_handle {
-    unsigned int dev_index;
+    kdev_t dev;
     ino_t inode;
+    __u32 gen;
 };
 
-struct super_block *(sb_list[100]);
-unsigned int sb_list_counter = 0;
-
-#ifdef LINUX2_1
 static int
-fhget_call(struct ViceIoctl *vice_ioctl, struct DENTRY *dentry)
+fhget_call(struct ViceIoctl *vice_ioctl, struct dentry *dentry)
 {
     struct file_handle fh;
     struct xfs_cache_handle temp_fh;
-    int dev_index = -1;
-    int i;
 
-    for (i = 0; i < sb_list_counter ; i++) {
-	if (sb_list[i]->s_dev == dentry->d_inode->i_dev) {
-	    dev_index = i;
-	}
-    }
-    if (dev_index == -1) {
-	dev_index = sb_list_counter;
-	sb_list[dev_index] = dentry->d_sb;
-	sb_list_counter++;
-    }
-    if (dev_index >= 100) {
-	DPUT(dentry);
-	return -ENOMEM;
-    }
-
-    fh.dev_index = dev_index;
+    fh.dev   = dentry->d_inode->i_dev;
     fh.inode = dentry->d_inode->i_ino;
+    fh.gen   = dentry->d_inode->i_generation;
 
-    XFSDEB(XDEBVFOPS, ("fhget_call: dev: %u inode: %u\n",
-		       fh.dev_index, (unsigned int) fh.inode));
+    XFSDEB(XDEBVFOPS, ("fhget_call: dev: %u inode: %u gen: %u\n",
+		       (unsigned int) fh.dev, (unsigned int) fh.inode,
+		       (unsigned int) fh.gen));
 
-    if (vice_ioctl->out_size < sizeof(xfs_cache_handle)) {
-	DPUT(dentry);
+    if (vice_ioctl->out_size < sizeof(struct xfs_cache_handle)) {
+	dput(dentry);
 	return -EINVAL;
     }
     memset(&temp_fh, 0, sizeof(temp_fh));
@@ -337,16 +306,20 @@ fhget_call(struct ViceIoctl *vice_ioctl, struct DENTRY *dentry)
 
     if (copy_to_user (vice_ioctl->out,
 		      &temp_fh,
-		      sizeof(xfs_cache_handle)) != 0) {
-	DPUT(dentry);
+		      sizeof(struct xfs_cache_handle)) != 0) {
+	dput(dentry);
 	return -EFAULT;
     }
 
-    DPUT(dentry);
+    dput(dentry);
 
     return 0;
 }
 
+/*
+ * lookup file identified by `handle' and return a dentry or an error.
+ */
+
 struct dentry * xfs_fh_to_dentry(struct xfs_cache_handle *handle)
 {
     struct file_handle fh;
@@ -357,12 +330,11 @@ struct dentry * xfs_fh_to_dentry(struct xfs_cache_handle *handle)
     memcpy(&fh, handle, sizeof(fh));
 
     XFSDEB(XDEBVFOPS, ("xfs_fh_to_dentry: dev: %u inode: %u\n",
-		       fh.dev_index, (unsigned int) fh.inode));
-
-    if (fh.dev_index >= sb_list_counter)
-	return ERR_PTR(-EINVAL);
+		       (unsigned int) fh.dev, (unsigned int) fh.inode));
     
-    sb = sb_list[fh.dev_index];
+    sb = get_super (fh.dev);
+    if (sb == NULL)
+	return ERR_PTR(-ENXIO);
 
     inode = iget(sb, fh.inode);
     if (inode == NULL) {
@@ -370,6 +342,14 @@ struct dentry * xfs_fh_to_dentry(struct xfs_cache_handle *handle)
 	return ERR_PTR(-ENOENT);
     }
 
+    if (inode->i_generation != fh.gen) {
+	iput (inode);
+	XFSDEB(XDEBVFOPS, ("xfs_fh_to_dentry: bad generation %u != %u\n",
+			   (unsigned int)inode->i_generation,
+			   (unsigned int)fh.gen));
+	return ERR_PTR(-ENOENT);
+    }
+
     dentry = d_alloc(NULL, &(const struct qstr) { "xfs cache file", 14, 0});
     dentry->d_sb = sb;
     dentry->d_parent = dentry;
@@ -379,54 +359,72 @@ struct dentry * xfs_fh_to_dentry(struct xfs_cache_handle *handle)
     return dentry;
 }
 
+/*
+ * open the file identified by the file handle in `vice_ioctl' with
+ * `mode'.
+ * Return an fd or an error.
+ */
+
 static int
 fhopen_call(struct ViceIoctl *vice_ioctl, int mode)
 {
     struct dentry *dentry;
+    struct xfs_cache_handle handle;
     int fd;
-    int error;
+    struct inode *inode;
 
     if (!suser())
         return -EPERM;
     
-    if (vice_ioctl->in_size < sizeof(xfs_cache_handle))
+    if (vice_ioctl->in_size < sizeof(struct xfs_cache_handle))
 	return -EINVAL;
 
-    dentry = xfs_fh_to_dentry((struct xfs_cache_handle *) vice_ioctl->in);
-    if (dentry == NULL)
-	return -ENOENT;
+    if (copy_from_user(&handle, vice_ioctl->in, sizeof(handle)) != 0)
+	return -EFAULT;
+
+    dentry = xfs_fh_to_dentry(&handle);
     if (IS_ERR(dentry))
 	return PTR_ERR(dentry);
+    inode = dentry->d_inode;
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,99)
     fd = open_dentry(dentry, O_RDWR);
     if (fd >= 0) {
-	error = get_write_access(dentry->d_inode);
+	int error = get_write_access(inode);
 	if (error) {
+	    dput(dentry);
 	    sys_close(fd);
 	    return error;
 	}
     }
-    
-    DPUT(dentry);
+    dput(dentry);
+#else
+    {
+	struct file *f;
+	int error;
 
+	error = get_write_access (inode);
+	if (error) {
+	    dput (dentry);
+	    return error;
+	}
+
+	f = dentry_open(dentry, NULL, O_RDWR);
+	if (IS_ERR(f))
+	    return PTR_ERR(f);
+	fd = get_unused_fd();
+	if (fd < 0) {
+	    put_filp(f);
+	    dput(dentry);
+	    return fd;
+	}
+	fd_install(fd, f);
+    }
+#endif
     XFSDEB(XDEBSYS, ("fhopen_call: returns fd: %d\n", fd));
 
     return fd;
 }
-#else
-int
-fhget_call(struct ViceIoctl *vice_ioctl, struct DENTRY *dentry)
-{
-    DPUT(dentry);
-    return -EINVAL;
-}
-
-int
-fhopen_call(struct ViceIoctl *vice_ioctl, int mode)
-{
-    return -EINVAL;
-}
-#endif
 
 static int
 xfs_debug (struct ViceIoctl *vice_ioctl)
@@ -464,7 +462,7 @@ xfs_debug (struct ViceIoctl *vice_ioctl)
 }
 
 static int
-xfs_debug_print (struct ViceIoctl *vice_ioctl)
+xfs_debug_print (struct ViceIoctl *vice_ioctl, struct dentry *node)
 {
     if (!suser())
         return -EPERM;
@@ -487,6 +485,14 @@ xfs_debug_print (struct ViceIoctl *vice_ioctl)
 	case XDEBMSG:
 	    xfs_print_sleep_queue();
 	    return 0;
+	case XDEBNODE:
+	    if (node) {
+		print_aliases(node->d_inode);
+		print_childs(node);
+	    } else {
+		print_nodes(0);
+	    }
+	    return 0;
 	default:
 	    return -EINVAL;
 	}
@@ -495,20 +501,64 @@ xfs_debug_print (struct ViceIoctl *vice_ioctl)
     return 0;
 }
 
+/*
+ * convert the path `user_path' (in user memory) into an dentry.
+ * follow symlinks iff `follow'
+ */
+
+#ifdef LINUX2_3
+
+static struct dentry *
+user_path2dentry (char *user_path, int follow)
+{
+    char *kname;
+    int flags;
+    struct nameidata nd;
+    int error;
+
+    kname = getname (user_path);
+    if (IS_ERR(kname))
+	return ERR_PTR(PTR_ERR(kname));
+    flags = LOOKUP_POSITIVE;
+    if (follow)
+	flags |= LOOKUP_FOLLOW;
+
+    error = 0;
+    if (path_init (kname, flags, &nd))
+	error = path_walk(kname, &nd);
+    putname(kname);
+    if (error)
+	return ERR_PTR(error);
+    return nd.dentry;
+}
+
+#else /* !LINUX2_3 */
+
+static struct dentry *
+user_path2dentry (char *user_path, int follow)
+{
+    if (follow)
+	return namei (user_path);
+    else
+	return lnamei (user_path);
+}
+
+#endif /* !LINUX2_3 */
+
 #define MIN(a,b) ((a<b)?a:b)
 
 asmlinkage int
-sys_afs (int operation,
-	 char *a_pathP,
-	 int a_opcode,
-	 struct ViceIoctl *a_paramsP,
-	 int a_followSymlinks)
+sys_afs_int (int operation,
+	     char *a_pathP,
+	     int a_opcode,
+	     struct ViceIoctl *a_paramsP,
+	     int a_followSymlinks)
 {
     int error = 0;
     struct ViceIoctl vice_ioctl; 
     struct xfs_message_pioctl msg;
     struct xfs_message_wakeup_data *msg2;
-    struct DENTRY *dentry = NULL;
+    struct dentry *dentry = NULL;
     
 #ifdef __SMP__
     XFSDEB(XDEBSYS, ("sys_afs locking kernel; cpu: %d\n", smp_processor_id()));
@@ -527,10 +577,7 @@ sys_afs (int operation,
     switch (operation) {
     case AFSCALL_PIOCTL:
 	XFSDEB(XDEBSYS, ("xfs_pioctl\n"));
-	if(copy_from_user(&vice_ioctl,a_paramsP,sizeof(*a_paramsP)) != 0) {
-	    error = -EFAULT;
-	    goto unlock;
-	}
+	memcpy(&vice_ioctl,a_paramsP,sizeof(*a_paramsP));
 	if (vice_ioctl.in_size > 2048) {
 	    XFSDEB(XDEBSYS, ("xfs_pioctl_call: got a humongous in packet: "
 			     "opcode: %d (size %d)",
@@ -548,25 +595,7 @@ sys_afs (int operation,
 	}
 	if (a_pathP != NULL) {
 	    XFSDEB(XDEBMSG, ("xfs_syscall: looking up: %p\n", a_pathP));
-
-#ifndef LINUX2_1
-	    if (a_followSymlinks)
-		error = namei(a_pathP, &dentry);
-	    else
-		error = lnamei(a_pathP, &dentry);
-	    if (dentry)
-		XFSDEB(XDEBMSG,("xfs_syscall: inode: %p inodenum: %lx\n",
-				dentry, dentry->i_ino));
-	    if (error) {
-		XFSDEB(XDEBMSG, ("xfs_syscall: error during namei: %d\n",
-				 error));
-		goto unlock;
-	    }
-#else
-	    if (a_followSymlinks)
-		dentry = namei(a_pathP);
-	    else
-		dentry = lnamei(a_pathP);
+	    dentry = user_path2dentry (a_pathP, a_followSymlinks);
 	    if (!dentry) {
 		error = -EINVAL;
 		goto unlock;
@@ -579,22 +608,42 @@ sys_afs (int operation,
 	    }
 	    XFSDEB(XDEBMSG,("xfs_syscall: inode: %p inodenum: %lx\n",
 			    dentry->d_inode, dentry->d_inode->i_ino));
-#endif /* LINUX2_1 */
 	}
 
 	switch (a_opcode) {
 	case VIOC_FHGET:
+#ifdef VIOC_FHGET_32
+	case VIOC_FHGET_32:
+#endif
 	    error = fhget_call(&vice_ioctl, dentry);
+	    dentry = NULL;
 	    goto unlock;
 	case VIOC_FHOPEN:
-	    DPUT(dentry);
+#ifdef VIOC_FHOPEN_32
+	case VIOC_FHOPEN_32:
+#endif
+	    if (dentry)
+		dput(dentry);
+	    dentry = NULL;
 	    error = fhopen_call(&vice_ioctl, a_followSymlinks);
 	    goto unlock;
 	case VIOC_XFSDEBUG:
+#ifdef VIOC_XFSDEBUG_32
+	case VIOC_XFSDEBUG_32:
+#endif
 	    error = xfs_debug (&vice_ioctl);
+	    if (dentry)
+		dput(dentry);
+	    dentry = NULL;
 	    goto unlock;
 	case VIOC_XFSDEBUG_PRINT:
-	    error = xfs_debug_print (&vice_ioctl);
+#ifdef VIOC_XFSDEBUG_PRINT_32
+	case VIOC_XFSDEBUG_PRINT_32:
+#endif
+	    error = xfs_debug_print (&vice_ioctl, dentry);
+	    if (dentry)
+		dput(dentry);
+	    dentry = NULL;
 	    goto unlock;
 	}
 
@@ -603,22 +652,29 @@ sys_afs (int operation,
 	    if (strcmp(DENTRY_TO_INODE(dentry)->i_sb->s_type->name,
 		       "xfs") != 0) {
 		XFSDEB(XDEBMSG, ("xfs_syscall: not in afs\n"));
-		DPUT(dentry);
+		dput(dentry);
 		dentry = NULL;
 		error = -EINVAL;
 		goto unlock;
 	    }
 	    xn = VNODE_TO_XNODE(DENTRY_TO_INODE(dentry));
+	    if (xn == NULL) {
+		XFSDEB(XDEBMSG, ("xfs_syscall: is an xfs dentry, but has no xnode\n"));
+		dput(dentry);
+		dentry = NULL;
+		error = -EINVAL;
+		goto unlock;
+	    }
 	    msg.handle = xn->handle;
-	    DPUT(dentry);
+	    dput(dentry);
 	    dentry = NULL;
 	}
 
 	msg.header.opcode = XFS_MSG_PIOCTL;
 	msg.opcode = a_opcode;
 	
-	msg.insize = vice_ioctl.in_size;
-	msg.outsize = vice_ioctl.out_size;
+	msg.insize   = vice_ioctl.in_size;
+	msg.outsize  = vice_ioctl.out_size;
 	msg.cred.uid = current->uid;
 	msg.cred.pag = xfs_get_pag();
 	
@@ -626,7 +682,8 @@ sys_afs (int operation,
 	msg2 = (struct xfs_message_wakeup_data *) &msg;
 	if (error == 0)
 	    error = msg2->error;
-	else
+
+	if (error == -ENODEV)
 	    error = -EINVAL;
 
 	if (error == 0 && msg2->header.opcode == XFS_MSG_WAKEUP_DATA) {
@@ -660,13 +717,66 @@ sys_afs (int operation,
     return error;
 }
 
+#ifdef NEED_VICEIOCTL32
+asmlinkage int
+sys32_afs (int operation,
+	   char *a_pathP,
+	   int a_opcode,
+	   struct ViceIoctl32 *a_paramsP,
+	   int a_followSymlinks)
+{
+    struct ViceIoctl vice_ioctl;
+    struct ViceIoctl32 vice_ioctl32;
+
+    if (operation == AFSCALL_PIOCTL) {
+	if(copy_from_user(&vice_ioctl32, a_paramsP, sizeof(*a_paramsP)) != 0)
+	    return -EFAULT;
+
+	vice_ioctl.in = (caddr_t) (u_int64_t) vice_ioctl32.in;
+	vice_ioctl.out = (caddr_t) (u_int64_t) vice_ioctl32.out;
+	vice_ioctl.in_size = vice_ioctl32.in_size;
+	vice_ioctl.out_size = vice_ioctl32.out_size;
+    }
+
+    return sys_afs_int(operation, a_pathP, a_opcode, &vice_ioctl,
+		       a_followSymlinks);
+}
+#endif
+
+asmlinkage int
+sys_afs (int operation,
+	 char *a_pathP,
+	 int a_opcode,
+	 struct ViceIoctl *a_paramsP,
+	 int a_followSymlinks)
+{
+    struct ViceIoctl vice_ioctl;
+
+    if (operation == AFSCALL_PIOCTL) {
+	if(copy_from_user(&vice_ioctl, a_paramsP, sizeof(*a_paramsP)) != 0)
+	    return -EFAULT;
+    }
+
+    return sys_afs_int(operation, a_pathP, a_opcode, &vice_ioctl,
+		       a_followSymlinks);
+}
+
+
 void
 install_afs_syscall (void)
 {
     old_afs_syscall = sys_call_table[__NR_afs_syscall];
     sys_call_table[__NR_afs_syscall] = (sys_call_function)&sys_afs;
-    old_setgroups = sys_call_table[__NR_setgroups];
-    sys_call_table[__NR_setgroups] = (sys_call_function)&xfs_setgroups;
+#ifdef NEED_VICEIOCTL32
+    old_afs_syscall32 = sys_call_table32[__NR_afs_syscall];
+    sys_call_table32[__NR_afs_syscall] = (u_int32_t)&sys32_afs;
+#endif
+    old_setgroups = sys_call_table[ARLA_NR_setgroups];
+    sys_call_table[ARLA_NR_setgroups] = (sys_call_function)&xfs_setgroups;
+#ifdef ARLA_NR_setgroups16
+    old_setgroups16 = sys_call_table[ARLA_NR_setgroups16];
+    sys_call_table[ARLA_NR_setgroups16] = (sys_call_function)&xfs_setgroups16;
+#endif
 }
 
 void
@@ -676,8 +786,20 @@ restore_afs_syscall (void)
 	sys_call_table[__NR_afs_syscall] = old_afs_syscall;
 	old_afs_syscall = NULL;
     }
+#ifdef NEED_VICEIOCTL32
+    if (old_afs_syscall32) {
+	sys_call_table32[__NR_afs_syscall] = old_afs_syscall32;
+	old_afs_syscall32 = 0;
+    }
+#endif
     if (old_setgroups) {
-	sys_call_table[__NR_setgroups] = old_setgroups;
+	sys_call_table[ARLA_NR_setgroups] = old_setgroups;
 	old_setgroups = NULL;
     }
+#ifdef ARLA_NR_setgroups16
+    if (old_setgroups16) {
+	sys_call_table[ARLA_NR_setgroups16] = old_setgroups16;
+	old_setgroups16 = NULL;
+    }
+#endif
 }
