@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -41,153 +36,184 @@
 #include <xfs/xfs_fs.h>
 #include <xfs/xfs_deb.h>
 
-RCSID("$Id: xfs_node.c,v 1.8 1999/06/16 20:23:39 assar Exp $");
+RCSID("$Id: xfs_node.c,v 1.14.2.1 2001/01/30 02:14:01 lha Exp $");
 
 /*
  * Create a new xfs_node and make a VN_HOLD()!
  *
- * Also prevents creation of duplicates. This happens
- * whenever there are more than one name to a file,
- * "." and ".." are common cases.
+ * Also prevents creation of duplicates. This happens whenever there
+ * are more than one name to a file, "." and ".." are common
+ * cases. For this case the `dp' argument was added. `dp' is a the
+ * locked parent node, or NULL if there are none.
  */
+
 struct xfs_node *
-new_xfs_node(struct xfs *xfsp, struct xfs_msg_node *node)
+new_xfs_node(struct xfs *xfsp, struct xfs_node *dp, struct xfs_msg_node *node)
 {
-  struct xfs_node *result;
-
-  XFSDEB(XDEBNODE, ("new_xfs_node: node = %x\n",
-		    (int)node));
-
-  XFSDEB(XDEBNODE, ("new_xfs_node %d.%d.%d.%d\n",
-		   node->handle.a,
-		   node->handle.b,
-		   node->handle.c,
-		   node->handle.d));
-
-  /* Does not allow duplicates */
-  result = xfs_node_find(xfsp, &node->handle);
-  if (result == 0)
-    {
-      result = xfs_alloc(sizeof(*result));
-      if (result == 0)
-	{
-	  printf("xfs_alloc(%d) failed\n", (int)sizeof(*result));
-	  panic("new_xfs_node: You Loose!");
+    struct xfs_node *result;
+    
+    XFSDEB(XDEBNODE, ("new_xfs_node: node = %x\n",
+		      (int)node));
+    
+    XFSDEB(XDEBNODE, ("new_xfs_node %d.%d.%d.%d\n",
+		      node->handle.a,
+		      node->handle.b,
+		      node->handle.c,
+		      node->handle.d));
+    
+    /* Does not allow duplicates */
+    if (dp && xfs_handle_eq(&dp->handle, &node->handle)) {
+	/* Node is parent, ie . */
+	result = dp;
+	VN_HOLD(XNODE_TO_VNODE(result));
+    } else if ((result = xfs_node_find(xfsp, &node->handle)) == 0) {
+	result = xfs_alloc(sizeof(*result));
+	if (result == 0) {
+	    printf("xfs_alloc(%d) failed\n", (int)sizeof(*result));
+	    panic("new_xfs_node: You Loose!");
 	}
+	
+	/* Init vnode part */
+	result->vn.v_vfsmountedhere = 0;
+	result->vn.v_op = &xfs_vnodeops;
+	result->vn.v_filocks = 0;
+	DATA_FROM_XNODE(result) = (struct vnode *) 0;
+	xfs_attr2vattr (&node->attr, &result->attr, 1);
+	VN_INIT(&result->vn, xfsp->vfsp,
+		result->attr.va_type, result->attr.va_rdev);
+	
+	result->handle = node->handle;
+	result->flags = 0;
+	result->tokens = 0;
+	result->anonrights = node->anonrights;
+	
+	mutex_init (&result->node_lock, "xfs_node", MUTEX_DRIVER, NULL);
+	mutex_enter(&result->node_lock);
 
-      /* Init vnode part */
-      result->vn.v_vfsmountedhere = 0;
-      result->vn.v_op = &xfs_vnodeops;
-      result->vn.v_filocks = 0;
-      DATA_FROM_XNODE(result) = (struct vnode *) 0;
-      xfs_attr2vattr (&node->attr, &result->attr);
-      VN_INIT(&result->vn, xfsp->vfsp,
-	      result->attr.va_type, result->attr.va_rdev);
-      
-      result->handle = node->handle;
-      result->flags = 0;
-      result->tokens = 0;
-      result->anonrights = node->anonrights;
-      
-      /* Save reference on list */
-      result->next = xfsp->nodes;
-      xfsp->nodes = result;
-      xfsp->nnodes++;
-    }
-  else
-    {
-      /* Node is already cached */
-      VN_HOLD(XNODE_TO_VNODE(result));
+	/* Save reference on list */
+	mutex_enter(&xfsp->nodes_modify);
+	result->next = xfsp->nodes;
+	xfsp->nodes = result;
+	xfsp->nnodes++;
+	mutex_exit(&xfsp->nodes_modify);
+    } else {
+	/* Node is already cached */
+	VN_HOLD(XNODE_TO_VNODE(result));
     }
+    
+    /* Init other fields */
+    xfs_attr2vattr (&node->attr, &result->attr);
+    XFS_TOKEN_SET(result, XFS_ATTR_R, XFS_ATTR_MASK);
+    bcopy((caddr_t)node->id,
+	  (caddr_t)result->id, sizeof(result->id));
+    bcopy((caddr_t)node->rights,
+	  (caddr_t)result->rights, sizeof(result->rights));
 
-  /* Init other fields */
-  xfs_attr2vattr (&node->attr, &result->attr);
-  XFS_TOKEN_SET(result, XFS_ATTR_R, XFS_ATTR_MASK);
-  bcopy((caddr_t)node->id,
-	(caddr_t)result->id, sizeof(result->id));
-  bcopy((caddr_t)node->rights,
-	(caddr_t)result->rights, sizeof(result->rights));
-
-  return result;
+    return result;
 }
 
 void
 free_xfs_node(struct xfs_node *node)
 {
-  struct xfs *xfsp = XFS_FROM_XNODE(node);
-  struct xfs_node *t;
+    struct xfs *xfsp = XFS_FROM_XNODE(node);
+    struct xfs_node *t;
+    
+    XFSDEB(XDEBNODE, ("free_xfs_node starting\n"));
+    
+    ASSERT(mutex_owned(&node->node_lock));
 
-  XFSDEB(XDEBNODE, ("free_xfs_node starting\n"));
+    mutex_enter(&xfsp->nodes_modify);
 
-  /* First remove from chain. */
-  if (node == xfsp->nodes)
-    {
-      xfsp->nodes = node->next;
+    /* 
+     * If we are the next on the iteration list, 
+     * move the pointer to the node after us.
+     */
+
+    if (xfsp->next_node == node)
+	xfsp->next_node = node->next;
+
+    /* First remove from chain. */
+    if (node == xfsp->nodes) {
+	xfsp->nodes = node->next;
+    } else {
+	for (t = xfsp->nodes; t->next; t = t->next)
+	    if (t->next == node) {
+		t->next = t->next->next;
+		goto done;
+	    }
+	mutex_exit(&xfsp->nodes_modify);
+	mutex_exit(&node->node_lock);
+	printf("XFS PANIC Error: free_xfs_node(0x%x) failed?\n", (int) node);
+	return;
     }
-  else
-    {
-      for (t = xfsp->nodes; t->next; t = t->next)
-	if (t->next == node)
-	  {
-	    t->next = t->next->next;
-	    goto done;
-	  }
-      printf("XFS PANIC Error: free_xfs_node(0x%x) failed?\n", (int) node);
-      return;
-    }
-
+    
  done:
-  /* XXX Really need to put back dirty data first. */
-  if (DATA_FROM_XNODE(node))
-    {
-      VN_RELE(DATA_FROM_XNODE(node));
-    }
-  xfsp->nnodes--;
-  xfs_free(node, sizeof(*node));
+    xfsp->nnodes--;
+    mutex_exit(&xfsp->nodes_modify);
 
-  XFSDEB(XDEBNODE, ("free_xfs_node done\n"));
+    /* XXX Really need to put back dirty data first. */
+    if (DATA_FROM_XNODE(node))
+	VN_RELE(DATA_FROM_XNODE(node));
+
+    mutex_exit(&node->node_lock);
+    mutex_destroy(&node->node_lock);
+
+    xfs_free(node, sizeof(*node));
+
+    XFSDEB(XDEBNODE, ("free_xfs_node done\n"));
+}
+
+struct xfs_node *
+xfs_node_iter_start (struct xfs *xfsp)
+{
+    struct xfs_node *n;
+    mutex_enter(&xfsp->nodes_iter);
+    mutex_enter(&xfsp->nodes_modify);
+    ASSERT(xfsp->next_node == NULL);
+    if (xfsp->nodes != NULL)
+	xfsp->next_node = xfsp->nodes->next;
+    n = xfsp->nodes;
+    mutex_exit(&xfsp->nodes_modify);
+    return n;
+}
+
+struct xfs_node *
+xfs_node_iter_next (struct xfs *xfsp)
+{
+    struct xfs_node *n;
+    ASSERT(mutex_owned(&xfsp->nodes_iter));
+    mutex_enter(&xfsp->nodes_modify);
+    n = xfsp->next_node;
+    if (n)
+	xfsp->next_node = n->next;
+    else
+	xfsp->next_node = NULL;
+    mutex_exit(&xfsp->nodes_modify);
+    return n;
 }
 
 void
-free_all_xfs_nodes(struct xfs *xfsp)
+xfs_node_iter_stop (struct xfs *xfsp)
 {
-  struct xfs_node *t;
-
-  XFSDEB(XDEBNODE, ("free_all_xfs_nodes starting\n"));
-
-  xfs_dnlc_purge();		/* This is really a bit brutal! */
-  XFSDEB(XDEBNODE, ("free_all_xfs_nodes now removing root\n"));
-  if (xfsp->root)
-    {
-      VN_RELE(XNODE_TO_VNODE(xfsp->root));
-      xfsp->root = 0;
-    }
-
-  /* There might still be a few nodes out there, invalidate them */
-  for (t = xfsp->nodes; t; t = t->next)
-    {
-      t->tokens = 0;
-      if (DATA_FROM_XNODE(t))
-	{
-	  VN_RELE(DATA_FROM_XNODE(t));
-	  DATA_FROM_XNODE(t) = (struct vnode *) 0;
-	}
-    }
-
-  XFSDEB(XDEBNODE, ("free_all_xfs_nodes done\n"));
+    xfsp->next_node = NULL;
+    mutex_exit(&xfsp->nodes_iter);
 }
 
+
 struct xfs_node *
 xfs_node_find(struct xfs *xfsp, xfs_handle *handlep)
 {
-  struct xfs_node *t;
-
-  XFSDEB(XDEBNODE, ("xfs_node_find\n"));
-
-  for (t = xfsp->nodes; t != 0; t = t->next)
-    if (xfs_handle_eq(&t->handle, handlep))
-      break;
-  return t;
+    struct xfs_node *t;
+    
+    XFSDEB(XDEBNODE, ("xfs_node_find\n"));
+    
+    for (t = xfs_node_iter_start(xfsp); t != 0; t = xfs_node_iter_next(xfsp))
+	if (xfs_handle_eq(&t->handle, handlep)) {
+	    mutex_enter(&t->node_lock);
+	    break;
+	}
+    xfs_node_iter_stop(xfsp);
+    return t;
 }
 
 struct long_entry {
@@ -198,6 +224,50 @@ struct long_entry {
 
 static struct long_entry tbl;
 
+static void
+xfs_dnlc_purge(struct vfs *vfsp)
+{
+    XFSDEB(XDEBDNLC, ("xfs_dnlc_purge\n"));
+    
+    dnlc_purge_vfsp(vfsp, 0);
+    
+    if (tbl.dvp) {
+	VN_RELE(tbl.dvp);
+	tbl.dvp = 0;
+	VN_RELE(tbl.vp);
+	tbl.vp = 0;
+    }
+}
+
+void
+free_all_xfs_nodes(struct xfs *xfsp)
+{
+    struct xfs_node *t;
+    
+    XFSDEB(XDEBNODE, ("free_all_xfs_nodes starting\n"));
+
+    xfs_dnlc_purge(XFS_TO_VFS(xfsp));		/* This is really a bit brutal! */
+    XFSDEB(XDEBNODE, ("free_all_xfs_nodes now removing root\n"));
+    if (xfsp->root) {
+	VN_RELE(XNODE_TO_VNODE(xfsp->root));
+	xfsp->root = 0;
+    }
+    
+    /* There might still be a few nodes out there, invalidate them */
+
+    for (t = xfs_node_iter_start(xfsp); t != 0; t = xfs_node_iter_next(xfsp)) {
+	t->tokens = 0;
+	if (DATA_FROM_XNODE(t)) {
+	    VN_RELE(DATA_FROM_XNODE(t));
+	    DATA_FROM_XNODE(t) = (struct vnode *) 0;
+	}
+    }
+    xfs_node_iter_stop(xfsp);
+
+    
+    XFSDEB(XDEBNODE, ("free_all_xfs_nodes done\n"));
+}
+
 int
 xfs_dnlc_enter(struct vnode *dvp, char *nm, struct vnode *vp)
 {
@@ -215,8 +285,10 @@ xfs_dnlc_enter(struct vnode *dvp, char *nm, struct vnode *vp)
     return 0;
   }
 
-  /* Be careful to HOLD first because dvp and vp
-   * might be aliased to tbl.dvp or tbl.vp. */
+  /*
+   * Be careful to HOLD first because dvp and vp
+   * might be aliased to tbl.dvp or tbl.vp.
+   */
   VN_HOLD(dvp);
   VN_HOLD(vp);
   if (tbl.dvp != 0)
@@ -232,6 +304,11 @@ xfs_dnlc_enter(struct vnode *dvp, char *nm, struct vnode *vp)
   return 0;
 }
 
+/*
+ * lookup `vp, nm' in the name cache.  either return an holded vnode
+ * or NULL
+ */
+
 struct vnode *
 xfs_dnlc_lookup(struct vnode *dvp, char *nm)
 {
@@ -244,28 +321,13 @@ xfs_dnlc_lookup(struct vnode *dvp, char *nm)
     return res;
   else if (   (tbl.dvp == dvp)
 	   && (*tbl.nm == *nm)
-	   && (strncmp(tbl.nm, nm, tbl.len + 1) == 0))
+	      && (strncmp(tbl.nm, nm, tbl.len + 1) == 0)) {
+    VN_HOLD(tbl.vp);
     return tbl.vp;
-  else
+  } else
     return 0;
 }
 
-void
-xfs_dnlc_purge(void)
-{
-  XFSDEB(XDEBDNLC, ("xfs_dnlc_purge()\n"));
-
-  dnlc_purge();
-
-  if (tbl.dvp)
-    {
-      VN_RELE(tbl.dvp);
-      tbl.dvp = 0;
-      VN_RELE(tbl.vp);
-      tbl.vp = 0;
-    }
-}
-
 void
 xfs_dnlc_remove(vnode_t *dvp, char *name)
 {
@@ -335,9 +397,10 @@ vattr2xfs_attr(const struct vattr *va, struct xfs_attr *xa)
 }
 
 void
-xfs_attr2vattr(const struct xfs_attr *xa, struct vattr *va)
+xfs_attr2vattr(const struct xfs_attr *xa, struct vattr *va, , int clear_node)
 {
-    bzero((caddr_t)va, sizeof(*va));
+    if (clear_node)
+	bzero((caddr_t)va, sizeof(*va));
     if (XA_VALID_MODE(xa)) {
 	va->va_mode  = xa->xa_mode;
 	va->va_mask |= AT_MODE;
@@ -413,10 +476,9 @@ xfs_attr2vattr(const struct xfs_attr *xa, struct vattr *va)
 }
 
 /*
- * Returns 1 if pag has any rights set in the node
- */
+ * Returns 1 if pag has any rights set in the node */
 int
-xfs_has_pag(const struct xfs_node *xn, pag_t pag)
+xfs_has_pag(const struct xfs_node *xn, xfs_pag_t pag)
 {
     int i;
     
