@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2001 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -51,70 +46,70 @@
 #include <sys/flock.h>
 #include <sys/fs_subr.h>
 
-RCSID("$Id: xfs_vnodeops.c,v 1.22 1999/06/16 20:24:07 assar Exp $");
+RCSID("$Id: xfs_vnodeops.c,v 1.38 2001/01/07 01:10:05 lha Exp $");
 
 static int
 xfs_open_valid(struct vnode *vp, struct cred *cred, u_int tok)
 {
-  struct xfs *xfsp = XFS_FROM_VNODE(vp);
-  struct xfs_node *xn = VNODE_TO_XNODE(vp);
-  int error = 0;
+    struct xfs *xfsp = XFS_FROM_VNODE(vp);
+    struct xfs_node *xn = VNODE_TO_XNODE(vp);
+    int error = 0;
   
-  XFSDEB(XDEBVFOPS, ("xfs_open_valid\n"));
+    XFSDEB(XDEBVFOPS, ("xfs_open_valid\n"));
 
-  do {
-    if (!XFS_TOKEN_GOT(xn, tok))
-      {
-	struct xfs_message_open msg;
+    do {
+	if (!XFS_TOKEN_GOT(xn, tok))
+	{
+	    struct xfs_message_open msg;
 
-	msg.header.opcode = XFS_MSG_OPEN;
-	msg.cred.uid = cred->cr_uid;
-	msg.cred.pag = xfs_get_pag(cred);
-	msg.handle = xn->handle;
-	msg.tokens = tok;
-	error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
-	if (error == 0)
-	  error = ((struct xfs_message_wakeup *) &msg)->error;
-      }
-    else
-      {
-	goto done;
-      }
-  } while (error == 0);
+	    msg.header.opcode = XFS_MSG_OPEN;
+	    msg.cred.uid = cred->cr_uid;
+	    msg.cred.pag = xfs_get_pag(cred);
+	    msg.handle = xn->handle;
+	    msg.tokens = tok;
+	    error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
+	    if (error == 0)
+		error = ((struct xfs_message_wakeup *) &msg)->error;
+	}
+	else
+	{
+	    goto done;
+	}
+    } while (error == 0);
 
  done:
-  return error;
+    return error;
 }
 
 static int
 xfs_attr_valid(struct vnode *vp, struct cred *cred, u_int tok)
 {
-  struct xfs *xfsp = XFS_FROM_VNODE(vp);
-  struct xfs_node *xn = VNODE_TO_XNODE(vp);
-  int error = 0;
+    struct xfs *xfsp = XFS_FROM_VNODE(vp);
+    struct xfs_node *xn = VNODE_TO_XNODE(vp);
+    int error = 0;
   
-  pag_t pag = xfs_get_pag(cred);
+    xfs_pag_t pag = xfs_get_pag(cred);
 
-  do {
-    if (!XFS_TOKEN_GOT(xn, tok))
-      {
-	struct xfs_message_getattr msg;
-	msg.header.opcode = XFS_MSG_GETATTR;
-	msg.cred.uid = cred->cr_uid;
-	msg.cred.pag = pag;
-	msg.handle = xn->handle;
-	error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
-	if (error == 0)
-	  error = ((struct xfs_message_wakeup *) &msg)->error;
-      }
-    else
-      {
-	goto done;
-      }
-  } while (error == 0);
+    do {
+	if (!XFS_TOKEN_GOT(xn, tok))
+	{
+	    struct xfs_message_getattr msg;
+	    msg.header.opcode = XFS_MSG_GETATTR;
+	    msg.cred.uid = cred->cr_uid;
+	    msg.cred.pag = pag;
+	    msg.handle = xn->handle;
+	    error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
+	    if (error == 0)
+		error = ((struct xfs_message_wakeup *) &msg)->error;
+	}
+	else
+	{
+	    goto done;
+	}
+    } while (error == 0);
 
  done:
-  return error;
+    return error;
 }
 
 static int
@@ -124,7 +119,7 @@ xfs_fetch_rights(struct vnode *vp, struct cred *cred)
     struct xfs_node *xn = VNODE_TO_XNODE(vp);
     int error = 0;
 
-    pag_t pag = xfs_get_pag(cred);
+    xfs_pag_t pag = xfs_get_pag(cred);
 
     struct xfs_message_getattr msg;
 
@@ -142,31 +137,31 @@ xfs_fetch_rights(struct vnode *vp, struct cred *cred)
 static int
 xfs_data_valid(struct vnode *vp, struct cred *cred, u_int tok)
 {
-  struct xfs *xfsp = XFS_FROM_VNODE(vp);
-  struct xfs_node *xn = VNODE_TO_XNODE(vp);
-  int error = 0;
+    struct xfs *xfsp = XFS_FROM_VNODE(vp);
+    struct xfs_node *xn = VNODE_TO_XNODE(vp);
+    int error = 0;
   
-  do {
-    if (!XFS_TOKEN_GOT(xn, tok))
-      {
-	struct xfs_message_getdata msg;
-	msg.header.opcode = XFS_MSG_GETDATA;
-	msg.cred.uid = cred->cr_uid;
-	msg.cred.pag = xfs_get_pag(cred);
-	msg.handle = xn->handle;
-	msg.tokens = tok;
-	error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
-	if (error == 0)
-	  error = ((struct xfs_message_wakeup *) &msg)->error;
-      }
-    else
-      {
-	goto done;
-      }
-  } while (error == 0);
+    do {
+	if (!XFS_TOKEN_GOT(xn, tok))
+	{
+	    struct xfs_message_getdata msg;
+	    msg.header.opcode = XFS_MSG_GETDATA;
+	    msg.cred.uid = cred->cr_uid;
+	    msg.cred.pag = xfs_get_pag(cred);
+	    msg.handle = xn->handle;
+	    msg.tokens = tok;
+	    error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
+	    if (error == 0)
+		error = ((struct xfs_message_wakeup *) &msg)->error;
+	}
+	else
+	{
+	    goto done;
+	}
+    } while (error == 0);
 
  done:
-  return error;
+    return error;
 }
 
 static int
@@ -199,16 +194,16 @@ xfs_open(struct vnode **vpp,
 	 int flag,
 	 struct cred *cred)
 {
-  int error = 0;
+    int error = 0;
   
-  XFSDEB(XDEBVNOPS, ("xfs_open\n"));
+    XFSDEB(XDEBVNOPS, ("xfs_open\n"));
   
-  if (flag & FWRITE)
-    error = xfs_open_valid(*vpp, cred, XFS_OPEN_NW);
-  else
-    error = xfs_open_valid(*vpp, cred, XFS_OPEN_NR);
+    if (flag & FWRITE)
+	error = xfs_open_valid(*vpp, cred, XFS_OPEN_NW);
+    else
+	error = xfs_open_valid(*vpp, cred, XFS_OPEN_NR);
   
-  return error;
+    return error;
 }
 
 static int
@@ -218,18 +213,18 @@ xfs_close(struct vnode *vp,
 	  offset_t offset,
 	  struct cred *cred)
 {
-  struct xfs *xfsp = XFS_FROM_VNODE(vp);
-  struct xfs_node *xn = VNODE_TO_XNODE(vp);
-  int error = 0;
+    struct xfs *xfsp = XFS_FROM_VNODE(vp);
+    struct xfs_node *xn = VNODE_TO_XNODE(vp);
+    int error = 0;
 
-  XFSDEB(XDEBVNOPS, ("xfs_close\n"));
+    XFSDEB(XDEBVNOPS, ("xfs_close\n"));
   
-  if (flag & FWRITE && xn->flags & XFS_DATA_DIRTY)
-    error = do_fsync (xfsp, xn, cred, XFS_WRITE);
+    if (flag & FWRITE && xn->flags & XFS_DATA_DIRTY)
+	error = do_fsync (xfsp, xn, cred, XFS_WRITE);
   
-  XFSDEB(XDEBVNOPS, ("xfs_close: %d\n", error));
+    XFSDEB(XDEBVNOPS, ("xfs_close: %d\n", error));
 
-  return error;
+    return error;
 }
 
 static int
@@ -305,8 +300,8 @@ xfs_ioctl(struct vnode *vp,
 	  struct cred *cred,
 	  int *result)
 {
-  XFSDEB(XDEBVNOPS, ("xfs_ioctl\n"));
-  return ENOSYS;
+    XFSDEB(XDEBVNOPS, ("xfs_ioctl\n"));
+    return ENOSYS;
 }
 
 static int
@@ -315,8 +310,8 @@ xfs_setfl(struct vnode *vp,
 	  int nflags,
 	  struct cred *cred)
 {
-  XFSDEB(XDEBVNOPS, ("xfs_setfl\n"));
-  return fs_setfl (vp, oflags, nflags, cred);
+    XFSDEB(XDEBVNOPS, ("xfs_setfl\n"));
+    return fs_setfl (vp, oflags, nflags, cred);
 }
 
 static int
@@ -325,18 +320,21 @@ xfs_getattr(struct vnode *vp,
 	    int flags,
 	    struct cred *cred)     
 {
-  int error = 0;
+    int error = 0;
   
-  struct xfs_node *xn = VNODE_TO_XNODE(vp);
+    struct xfs_node *xn = VNODE_TO_XNODE(vp);
 
-  XFSDEB(XDEBVNOPS, ("xfs_getattr\n"));
+    XFSDEB(XDEBVNOPS, ("xfs_getattr\n"));
   
-  error = xfs_attr_valid(vp, cred, XFS_ATTR_R);
-  if (error == 0) {
-      *vap = xn->attr;
-  }
+    error = xfs_attr_valid(vp, cred, XFS_ATTR_R);
+    if (error == 0) {
+	*vap = xn->attr;
+    }
   
-  return error;
+    XFSDEB(XDEBVNOPS, ("xfs_getattr: size: 0x%lx\n",
+		       (unsigned long)xn->attr.va_size));
+
+    return error;
 }
 
 static int
@@ -345,31 +343,43 @@ xfs_setattr(struct vnode *vp,
 	    int flags,
 	    struct cred *cred)
 {
-  struct xfs *xfsp = XFS_FROM_VNODE(vp);
-  struct xfs_node *xn = VNODE_TO_XNODE(vp);
-  int error = 0;
+    struct xfs *xfsp = XFS_FROM_VNODE(vp);
+    struct xfs_node *xn = VNODE_TO_XNODE(vp);
+    int error = 0;
   
-  XFSDEB(XDEBVNOPS, ("xfs_setattr\n"));
-  if (XFS_TOKEN_GOT(xn, XFS_ATTR_W)) {
-      /* Update attributes and mark them dirty. */
-      VNODE_TO_XNODE(vp)->flags |= XFS_ATTR_DIRTY;
-      error = ENOSYS;		/* XXX not yet implemented */
-      goto done;
-  } else {
-      struct xfs_message_putattr msg;
-      msg.header.opcode = XFS_MSG_PUTATTR;
-      msg.cred.uid = cred->cr_uid;
-      msg.cred.pag = xfs_get_pag(cred);
-      msg.handle = xn->handle;
-      vattr2xfs_attr (vap, &msg.attr);
-      XFS_TOKEN_CLEAR(xn, XFS_ATTR_VALID, XFS_ATTR_MASK);
-      error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
-      if (error == 0)
-	error = ((struct xfs_message_wakeup *) &msg)->error;
-  }
+    XFSDEB(XDEBVNOPS, ("xfs_setattr\n"));
+    if (XFS_TOKEN_GOT(xn, XFS_ATTR_W)) {
+	/* Update attributes and mark them dirty. */
+	VNODE_TO_XNODE(vp)->flags |= XFS_ATTR_DIRTY;
+	error = ENOSYS;		/* XXX not yet implemented */
+	goto done;
+    } else {
+	struct xfs_message_putattr msg;
+	msg.header.opcode = XFS_MSG_PUTATTR;
+	msg.cred.uid = cred->cr_uid;
+	msg.cred.pag = xfs_get_pag(cred);
+	msg.handle = xn->handle;
+	vattr2xfs_attr (vap, &msg.attr);
+	if (XFS_TOKEN_GOT(xn, XFS_DATA_R)) {
+	    if (vp->v_type == VREG) {
+		if (vap->va_mask & AT_SIZE)
+		    XA_SET_SIZE(&msg.attr,  vap->va_size);
+		else
+		    XA_SET_SIZE(&msg.attr,  xn->attr.va_size);
+	    }
+	    if (vap->va_mask & AT_MTIME)
+		XA_SET_MTIME(&msg.attr, vap->va_mtime.tv_sec);
+	    else
+		XA_SET_MTIME(&msg.attr, xn->attr.va_mtime.tv_sec);
+	}
+	XFS_TOKEN_CLEAR(xn, XFS_ATTR_VALID, XFS_ATTR_MASK);
+	error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
+	if (error == 0)
+	    error = ((struct xfs_message_wakeup *) &msg)->error;
+    }
 
  done:
-  return error;
+    return error;
 }
 
 static int
@@ -395,36 +405,36 @@ xfs_access(struct vnode *vp,
 	   int flags,
 	   struct cred *cred)     
 {
-  int error = 0;
-  pag_t pag = xfs_get_pag(cred);
+    int error = 0;
+    xfs_pag_t pag = xfs_get_pag(cred);
 
-  XFSDEB(XDEBVNOPS, ("xfs_access mode = 0%o\n", mode));
+    XFSDEB(XDEBVNOPS, ("xfs_access mode = 0%o\n", mode));
 
-  error = xfs_attr_valid(vp, cred, XFS_ATTR_R);
-  if (error == 0) {
-      struct xfs_node *xn = VNODE_TO_XNODE(vp);
-      int i;
+    error = xfs_attr_valid(vp, cred, XFS_ATTR_R);
+    if (error == 0) {
+	struct xfs_node *xn = VNODE_TO_XNODE(vp);
+	int i;
 
-      error = check_rights (xn->anonrights, mode);
+	error = check_rights (xn->anonrights, mode);
 	
-      if (error == 0)
-	  goto done;
+	if (error == 0)
+	    goto done;
 
-      XFSDEB(XDEBVNOPS, ("xfs_access anonaccess failed\n"));
+	XFSDEB(XDEBVNOPS, ("xfs_access anonaccess failed\n"));
 
-      if (error != 0)
-	  xfs_fetch_rights(vp, cred); /* ignore error */
+	if (error != 0)
+	    xfs_fetch_rights(vp, cred); /* ignore error */
 	
-      error = EACCES;
+	error = EACCES;
 	
-      for (i = 0; i < MAXRIGHTS; i++)
-	  if (xn->id[i] == pag) {
-	      error = check_rights (xn->rights[i], mode);
-	      break;
-	  }
-  }
+	for (i = 0; i < MAXRIGHTS; i++)
+	    if (xn->id[i] == pag) {
+		error = check_rights (xn->rights[i], mode);
+		break;
+	    }
+    }
 
-done:
+ done:
     XFSDEB(XDEBVNOPS, ("xfs_access(0%o) = %d\n", mode, error));
     return error;
 
@@ -439,44 +449,52 @@ xfs_lookup(struct vnode *dvp,
 	   struct vnode *rdir,
 	   struct cred *cred)
 {
-  struct xfs_message_getnode msg;
-  struct xfs *xfsp = XFS_FROM_VNODE(dvp);
-  int error = 0;
+    struct xfs_message_getnode msg;
+    struct xfs *xfsp = XFS_FROM_VNODE(dvp);
+    int error = 0;
 
-  struct xfs_node *d = VNODE_TO_XNODE(dvp);
-  struct vnode *v;
+    struct xfs_node *d = VNODE_TO_XNODE(dvp);
+    struct vnode *v;
   
-  XFSDEB(XDEBVNOPS, ("xfs_lookup\n"));
+    XFSDEB(XDEBVNOPS, ("xfs_lookup (%s)\n", nm));
   
-  do {
-#ifdef notdef_but_correct
-    error = xfs_access(dvp, VEXEC, cred);
-    if (error != 0)
-      goto done;
-#endif
-    v = xfs_dnlc_lookup(dvp, nm);
-    if (!v)
-      {
-	msg.header.opcode = XFS_MSG_GETNODE;
-	msg.cred.uid = cred->cr_uid;
-	msg.cred.pag = xfs_get_pag(cred);
-	msg.parent_handle = d->handle;
-	strncpy(msg.name, nm, sizeof(msg.name));
-	error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
-	if (error == 0)
-	  error = ((struct xfs_message_wakeup *) &msg)->error;
-      }
-    else
-      {
-	*vpp = v;
-	VN_HOLD(v);
+    if (*nm == '\0') {
+	VN_HOLD(dvp);
+	*vpp = dvp;
+	error = 0;
 	goto done;
-      }
-  } while (error == 0);
+    }
+
+    do {
+#ifdef notdef_but_correct
+	error = xfs_access(dvp, VEXEC, cred);
+	if (error != 0)
+	    goto done;
+#endif
+	v = xfs_dnlc_lookup(dvp, nm);
+	if (!v)
+	{
+	    msg.header.opcode = XFS_MSG_GETNODE;
+	    msg.cred.uid = cred->cr_uid;
+	    msg.cred.pag = xfs_get_pag(cred);
+	    msg.parent_handle = d->handle;
+	    if (strlcpy(msg.name, nm, sizeof(msg.name)) >= XFS_MAX_NAME)
+		error = ENAMETOOLONG;
+	    else
+		error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
+	    if (error == 0)
+		error = ((struct xfs_message_wakeup *) &msg)->error;
+	}
+	else
+	{
+	    *vpp = v;
+	    goto done;
+	}
+    } while (error == 0);
 
  done:
-  XFSDEB(XDEBVNOPS, ("xfs_lookup() = %d\n", error));
-  return error;
+    XFSDEB(XDEBVNOPS, ("xfs_lookup() = %d\n", error));
+    return error;
 }
 
 static int
@@ -490,44 +508,45 @@ xfs_create(struct vnode *dvp,
 #ifdef _LARGEFILE64_SOURCE
 	   ,int file_awareness  /* Solaris 2.6+ */
 #endif
-	   )     
+    )     
 {
-  struct xfs *xfsp = XFS_FROM_VNODE(dvp);
-  struct xfs_node *xn = VNODE_TO_XNODE(dvp);
-  int error = 0;
-  int do_trunc = 0;
+    struct xfs *xfsp = XFS_FROM_VNODE(dvp);
+    struct xfs_node *xn = VNODE_TO_XNODE(dvp);
+    int error = 0;
+    int do_trunc = 0;
   
-  XFSDEB(XDEBVNOPS, ("xfs_create\n"));
-  {
-    struct xfs_message_create msg;
-    msg.header.opcode = XFS_MSG_CREATE;
-    msg.parent_handle = xn->handle;
-    strncpy(msg.name, nm, 256);
-    vattr2xfs_attr (va, &msg.attr);
+    XFSDEB(XDEBVNOPS, ("xfs_create\n"));
+    {
+	struct xfs_message_create msg;
+	msg.header.opcode = XFS_MSG_CREATE;
+	msg.parent_handle = xn->handle;
+	if (strlcpy(msg.name, nm, sizeof(msg.name)) >= XFS_MAX_NAME)
+	    return ENAMETOOLONG;
+	vattr2xfs_attr (va, &msg.attr);
 #if 0
-    msg.exclusive = exclusive;
+	msg.exclusive = exclusive;
 #endif
-    msg.mode = mode;
-    msg.cred.uid = cred->cr_uid;
-    msg.cred.pag = xfs_get_pag(cred);
-    error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
+	msg.mode = mode;
+	msg.cred.uid = cred->cr_uid;
+	msg.cred.pag = xfs_get_pag(cred);
+	error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
+	if (error == 0)
+	    error = ((struct xfs_message_wakeup *) &msg)->error;
+    }
+
+    if (error == EEXIST) {
+	do_trunc = 1;
+	error = 0;
+    }
+
     if (error == 0)
-      error = ((struct xfs_message_wakeup *) &msg)->error;
-  }
+	error = xfs_lookup(dvp, nm, vpp, /*pnp*/ NULL, /*flags*/ 0,
+			   /*rdir*/ NULL, cred);
 
-  if (error == EEXIST) {
-      do_trunc = 1;
-      error = 0;
-  }
+    if (error == 0 && do_trunc)
+	error = xfs_setattr (*vpp, va, 0, cred);
 
-  if (error == 0)
-      error = xfs_lookup(dvp, nm, vpp, /*pnp*/ NULL, /*flags*/ 0,
-			 /*rdir*/ NULL, cred);
-
-  if (error == 0 && do_trunc)
-      error = xfs_setattr (*vpp, va, 0, cred);
-
-  return error;
+    return error;
 }
 
 static int
@@ -535,23 +554,26 @@ xfs_remove(struct vnode *dvp,
 	   char *nm,
 	   struct cred *cred)
 {
-  struct xfs *xfsp = XFS_FROM_VNODE(dvp);
-  struct xfs_node *xn = VNODE_TO_XNODE(dvp);
-  struct xfs_message_remove msg;
-  int error;
+    struct xfs *xfsp = XFS_FROM_VNODE(dvp);
+    struct xfs_node *xn = VNODE_TO_XNODE(dvp);
+    struct xfs_message_remove msg;
+    int error;
 
-  XFSDEB(XDEBVNOPS, ("xfs_remove: %s\n", nm));
+    XFSDEB(XDEBVNOPS, ("xfs_remove: %s\n", nm));
 
-  msg.header.opcode = XFS_MSG_REMOVE;
-  msg.parent_handle = xn->handle;
-  strncpy(msg.name, nm, 256);
-  msg.cred.uid = cred->cr_uid;
-  msg.cred.pag = xfs_get_pag(cred);
-  error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
-  if (error == 0)
-      error = ((struct xfs_message_wakeup *) &msg)->error;
+    msg.header.opcode = XFS_MSG_REMOVE;
+    msg.parent_handle = xn->handle;
+    if (strlcpy(msg.name, nm, sizeof(msg.name)) >= XFS_MAX_NAME)
+	return ENAMETOOLONG;
+    msg.cred.uid = cred->cr_uid;
+    msg.cred.pag = xfs_get_pag(cred);
+    error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
+    if (error == 0)
+	error = ((struct xfs_message_wakeup *) &msg)->error;
+    if (error == 0)
+	dnlc_remove(dvp, nm);
 
-  return error;
+    return error;
 }
 
 static int
@@ -571,7 +593,8 @@ xfs_link(struct vnode *tdvp,
     msg.header.opcode = XFS_MSG_LINK;
     msg.parent_handle = xn->handle;
     msg.from_handle   = xn2->handle;
-    strncpy(msg.name, tnm, 256);
+    if (strlcpy(msg.name, tnm, sizeof(msg.name)) >= XFS_MAX_NAME)
+	return ENAMETOOLONG;
     msg.cred.uid = cred->cr_uid;
     msg.cred.pag = xfs_get_pag(cred);
 
@@ -589,44 +612,47 @@ xfs_rename(struct vnode *sdvp,
 	   char *nnm,
 	   struct cred *cred)     
 {
-  struct xfs_message_rename msg;
-  struct vnode *vp;
-  int error = 0;
+    struct xfs_message_rename msg;
+    struct vnode *vp;
+    int error = 0;
 
-  XFSDEB(XDEBVNOPS, ("xfs_rename\n"));
+    XFSDEB(XDEBVNOPS, ("xfs_rename\n"));
 
-  error = xfs_lookup(tdvp, nnm, &vp, /*pnp*/ NULL, /*flags*/ 0,
-		     /*rdir*/ NULL, cred);
-  if (error != ENOENT) {
-      /* the filename being moved to already exists */
-      struct xfs_message_remove remmsg;
+    error = xfs_lookup(tdvp, nnm, &vp, /*pnp*/ NULL, /*flags*/ 0,
+		       /*rdir*/ NULL, cred);
+    if (error != ENOENT) {
+	/* the filename being moved to already exists */
+	struct xfs_message_remove remmsg;
       
-      remmsg.header.opcode = XFS_MSG_REMOVE;
-      remmsg.parent_handle = VNODE_TO_XNODE(tdvp)->handle;
-      strncpy(remmsg.name, nnm, 256);
-      remmsg.cred.uid = cred->cr_uid;
-      remmsg.cred.pag = xfs_get_pag(cred);
-      error = xfs_message_rpc(XFS_FROM_VNODE(tdvp)->fd, &remmsg.header,
-			      sizeof(remmsg));
-      if (error == 0)
-	  error = ((struct xfs_message_wakeup *) & remmsg)->error;
-      if (error != 0)
-	  return error;
-  }
+	remmsg.header.opcode = XFS_MSG_REMOVE;
+	remmsg.parent_handle = VNODE_TO_XNODE(tdvp)->handle;
+	if (strlcpy(remmsg.name, nnm, sizeof(remmsg.name)) >= XFS_MAX_NAME)
+	    return ENAMETOOLONG;
+	remmsg.cred.uid = cred->cr_uid;
+	remmsg.cred.pag = xfs_get_pag(cred);
+	error = xfs_message_rpc(XFS_FROM_VNODE(tdvp)->fd, &remmsg.header,
+				sizeof(remmsg));
+	if (error == 0)
+	    error = ((struct xfs_message_wakeup *) & remmsg)->error;
+	if (error != 0 && error != ENOENT)
+	    return error;
+    }
 
-  msg.header.opcode = XFS_MSG_RENAME;
-  msg.old_parent_handle = VNODE_TO_XNODE(sdvp)->handle;
-  strncpy(msg.old_name, onm, 256);
-  msg.new_parent_handle = VNODE_TO_XNODE(tdvp)->handle;
-  strncpy(msg.new_name, nnm, 256);
-  msg.cred.uid = cred->cr_uid;
-  msg.cred.pag = xfs_get_pag(cred);
-  error = xfs_message_rpc(XFS_FROM_VNODE(sdvp)->fd, &msg.header,
-			  sizeof(msg));
-  if (error == 0)
-      error = ((struct xfs_message_wakeup *) & msg)->error;
+    msg.header.opcode = XFS_MSG_RENAME;
+    msg.old_parent_handle = VNODE_TO_XNODE(sdvp)->handle;
+    if (strlcpy(msg.old_name, onm, sizeof(msg.old_name)) >= XFS_MAX_NAME)
+	return ENAMETOOLONG;
+    msg.new_parent_handle = VNODE_TO_XNODE(tdvp)->handle;
+    if (strlcpy(msg.new_name, nnm, sizeof(msg.new_name)) >= XFS_MAX_NAME)
+	return ENAMETOOLONG;
+    msg.cred.uid = cred->cr_uid;
+    msg.cred.pag = xfs_get_pag(cred);
+    error = xfs_message_rpc(XFS_FROM_VNODE(sdvp)->fd, &msg.header,
+			    sizeof(msg));
+    if (error == 0)
+	error = ((struct xfs_message_wakeup *) & msg)->error;
   
-  return error;
+    return error;
 }
 
 static int
@@ -636,27 +662,28 @@ xfs_mkdir(struct vnode *dvp,
 	  struct vnode **vpp,
 	  struct cred *cred)     
 {
-  struct xfs *xfsp = XFS_FROM_VNODE(dvp);
-  struct xfs_node *xn = VNODE_TO_XNODE(dvp);
-  int error = 0;
+    struct xfs *xfsp = XFS_FROM_VNODE(dvp);
+    struct xfs_node *xn = VNODE_TO_XNODE(dvp);
+    int error = 0;
   
-  XFSDEB(XDEBVNOPS, ("xfs_mkdir\n"));
-  {
-    struct xfs_message_mkdir msg;
-    msg.header.opcode = XFS_MSG_MKDIR;
-    msg.parent_handle = xn->handle;
-    strncpy(msg.name, nm, 256);
-    vattr2xfs_attr (va, &msg.attr);
-    msg.cred.uid = cred->cr_uid;
-    msg.cred.pag = xfs_get_pag(cred);
-    error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
+    XFSDEB(XDEBVNOPS, ("xfs_mkdir\n"));
+    {
+	struct xfs_message_mkdir msg;
+	msg.header.opcode = XFS_MSG_MKDIR;
+	msg.parent_handle = xn->handle;
+	if (strlcpy(msg.name, nm, sizeof(msg.name)) >= XFS_MAX_NAME)
+	    return ENAMETOOLONG;
+	vattr2xfs_attr (va, &msg.attr);
+	msg.cred.uid = cred->cr_uid;
+	msg.cred.pag = xfs_get_pag(cred);
+	error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
+	if (error == 0)
+	    error = ((struct xfs_message_wakeup *) &msg)->error;
+    }
     if (error == 0)
-      error = ((struct xfs_message_wakeup *) &msg)->error;
-  }
-  if (error == 0)
-      error = xfs_lookup(dvp, nm, vpp, /*pnp*/ NULL, /*flags*/ 0,
-			 /*rdir*/ NULL, cred);
-  return error;
+	error = xfs_lookup(dvp, nm, vpp, /*pnp*/ NULL, /*flags*/ 0,
+			   /*rdir*/ NULL, cred);
+    return error;
 }
 
 static int
@@ -675,12 +702,15 @@ xfs_rmdir(struct vnode *dvp,
 
     msg.header.opcode = XFS_MSG_RMDIR;
     msg.parent_handle = xn->handle;
-    strncpy(msg.name, nm, 256);
+    if (strlcpy(msg.name, nm, sizeof(msg.name)) >= XFS_MAX_NAME)
+	return ENAMETOOLONG;
     msg.cred.uid = cred->cr_uid;
     msg.cred.pag = xfs_get_pag(cred);
     error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
     if (error == 0)
 	error = ((struct xfs_message_wakeup *) &msg)->error;
+    if (error == 0)
+	dnlc_remove(dvp, nm);
 
     return error;
 }
@@ -691,21 +721,21 @@ xfs_readdir(struct vnode *vp,
 	    struct cred *cred,
 	    int *eofp)		/* XXX */
 {
-  int error = 0;
+    int error = 0;
 
-  XFSDEB(XDEBVNOPS, ("xfs_readdir\n"));
+    XFSDEB(XDEBVNOPS, ("xfs_readdir\n"));
 
-  error = xfs_data_valid(vp, cred, XFS_DATA_R);
-  if (error == 0)
+    error = xfs_data_valid(vp, cred, XFS_DATA_R);
+    if (error == 0)
     {
-      struct vnode *t = DATA_FROM_VNODE(vp);
-      ASSERT(t != NULL);
-      VOP_RWLOCK(t, 0);
-      error = VOP_READ(t, uiop, 0, cred);
-      VOP_RWUNLOCK(t, 0);
+	struct vnode *t = DATA_FROM_VNODE(vp);
+	ASSERT(t != NULL);
+	VOP_RWLOCK(t, 0);
+	error = VOP_READ(t, uiop, 0, cred);
+	VOP_RWUNLOCK(t, 0);
     }
 
-  return error;
+    return error;
 }
 
 static int
@@ -724,8 +754,10 @@ xfs_symlink(struct vnode *dvp,
    
     msg.header.opcode = XFS_MSG_SYMLINK;
     msg.parent_handle = xn->handle;
-    strncpy(msg.name, lnm, 256);
-    strncpy(msg.contents, tnm, sizeof(msg.contents));
+    if (strlcpy(msg.name, lnm, sizeof(msg.name)) >= XFS_MAX_NAME)
+	return ENAMETOOLONG;
+    if (strlcpy(msg.contents, tnm, sizeof(msg.contents)) >= XFS_MAX_SYMLINK_CONTENT)
+	return ENAMETOOLONG;
     vattr2xfs_attr (tva, &msg.attr);
     msg.cred.uid = cred->cr_uid;
     msg.cred.pag = xfs_get_pag(cred);
@@ -742,21 +774,21 @@ xfs_readlink(struct vnode *vp,
 	     struct uio *uiop,
 	     struct cred *cred)
 {
-  int error = 0;
+    int error = 0;
 
-  XFSDEB(XDEBVNOPS, ("xfs_readlink\n"));
+    XFSDEB(XDEBVNOPS, ("xfs_readlink\n"));
 
-  error = xfs_data_valid(vp, cred, XFS_DATA_R);
-  if (error == 0)
+    error = xfs_data_valid(vp, cred, XFS_DATA_R);
+    if (error == 0)
     {
-      struct vnode *t = DATA_FROM_VNODE(vp);
-      ASSERT(t != NULL);
-      VOP_RWLOCK(t, 0);
-      error = VOP_READ(t, uiop, 0, cred);
-      VOP_RWUNLOCK(t, 0);
+	struct vnode *t = DATA_FROM_VNODE(vp);
+	ASSERT(t != NULL);
+	VOP_RWLOCK(t, 0);
+	error = VOP_READ(t, uiop, 0, cred);
+	VOP_RWUNLOCK(t, 0);
     }
 
-  return error;
+    return error;
 }
 
 static int
@@ -764,54 +796,58 @@ xfs_fsync(struct vnode *vp,
 	  int syncflag,
 	  struct cred *cred)
 {
-  struct xfs *xfsp = XFS_FROM_VNODE(vp);
-  struct xfs_node *xn = VNODE_TO_XNODE(vp);
-  int error = 0;
+    struct xfs *xfsp = XFS_FROM_VNODE(vp);
+    struct xfs_node *xn = VNODE_TO_XNODE(vp);
+    int error = 0;
   
-  XFSDEB(XDEBVNOPS, ("xfs_fsync\n"));
+    XFSDEB(XDEBVNOPS, ("xfs_fsync\n"));
 
-  if (xn->flags & XFS_DATA_DIRTY)
-      error = do_fsync (xfsp, xn, cred, XFS_WRITE);
-  return error;
+    if (xn->flags & XFS_DATA_DIRTY)
+	error = do_fsync (xfsp, xn, cred, XFS_WRITE | XFS_FSYNC);
+    return error;
 }
 
 static void
 xfs_inactive(struct vnode *vp,
 	     struct cred *cred)
 {
-  struct xfs_message_inactivenode msg;
-  struct xfs *xfsp = XFS_FROM_VNODE(vp);
-  struct xfs_node *xn = VNODE_TO_XNODE(vp);
+    struct xfs_message_inactivenode msg;
+    struct xfs *xfsp = XFS_FROM_VNODE(vp);
+    struct xfs_node *xn = VNODE_TO_XNODE(vp);
+    int error;
 
-  XFSDEB(XDEBVNOPS, ("xfs_inactive: 0x%x\n", (int)vp));
+    XFSDEB(XDEBVNOPS, ("xfs_inactive: 0x%x\n", (int)vp));
 
-  msg.header.opcode = XFS_MSG_INACTIVENODE;
-  msg.handle = xn->handle;
-  msg.flag   = XFS_NOREFS | XFS_DELETE;
-  free_xfs_node(VNODE_TO_XNODE(vp));
-  xfs_message_send(xfsp->fd, &msg.header, sizeof(msg));
+    mutex_enter(&xn->node_lock);
+
+    msg.header.opcode = XFS_MSG_INACTIVENODE;
+    msg.handle = xn->handle;
+    msg.flag   = XFS_NOREFS | XFS_DELETE;
+    VNODE_TO_XNODE(vp) = NULL;
+    free_xfs_node(xn);
+    xfs_message_send(xfsp->fd, &msg.header, sizeof(msg));
 }
 
 static int
 xfs_fid(struct vnode *vp,
 	struct fid *fid)
 {
-  XFSDEB(XDEBVNOPS, ("xfs_fid\n"));
-  return ENOSYS;
+    XFSDEB(XDEBVNOPS, ("xfs_fid\n"));
+    return ENOSYS;
 }
 
 static void
 xfs_rwlock(struct vnode *vp,
 	   int write_lock)
 {
-  XFSDEB(XDEBVNOPS, ("xfs_rwlock\n"));
+    XFSDEB(XDEBVNOPS, ("xfs_rwlock\n"));
 }
 
 static void
 xfs_rwunlock(struct vnode *vp,
 	     int write_lock)
 {
-  XFSDEB(XDEBVNOPS, ("xfs_rwunlock\n"));
+    XFSDEB(XDEBVNOPS, ("xfs_rwunlock\n"));
 }
 
 static int
@@ -819,15 +855,15 @@ xfs_seek(struct vnode *vp,
 	 offset_t offset,
 	 offset_t *roffset)
 {
-  XFSDEB(XDEBVNOPS, ("xfs_seek\n"));
-  return 0;
+    XFSDEB(XDEBVNOPS, ("xfs_seek\n"));
+    return 0;
 }
 
 static int
 xfs_cmp(struct vnode *vp1, struct vnode *vp2)
 {
-  XFSDEB(XDEBVNOPS, ("xfs_cmp\n"));
-  return vp1 == vp2;
+    XFSDEB(XDEBVNOPS, ("xfs_cmp\n"));
+    return vp1 == vp2;
 }
 
 static int
@@ -842,8 +878,8 @@ xfs_frlock(struct vnode *vp,
 	   offset_t off,
 	   struct cred *cred)
 {
-  XFSDEB(XDEBVNOPS, ("xfs_frlock\n"));
-  return ENOSYS;
+    XFSDEB(XDEBVNOPS, ("xfs_frlock\n"));
+    return ENOSYS;
 }
 
 static int
@@ -858,39 +894,39 @@ xfs_space(struct vnode *vp,
 	  offset_t offset,
 	  struct cred *cred)
 {
-  int error = 0;
-  struct vattr attr;
+    int error = 0;
+    struct vattr attr;
 
-  XFSDEB(XDEBVNOPS, ("xfs_space\n"));
+    XFSDEB(XDEBVNOPS, ("xfs_space\n"));
 
-  if (cmd != F_FREESP) {
-      error = EINVAL;
-      goto done;
-  }
+    if (cmd != F_FREESP) {
+	error = EINVAL;
+	goto done;
+    }
       
-  error = convoff (vp, fl, 0, offset);
-  if (error)
-      goto done;
+    error = convoff (vp, fl, 0, offset);
+    if (error)
+	goto done;
 
-  if (fl->l_len != 0) {
-      error = EINVAL;
-      goto done;
-  }
+    if (fl->l_len != 0) {
+	error = EINVAL;
+	goto done;
+    }
 
-  attr.va_mask = AT_SIZE;
-  attr.va_size = fl->l_start;
-  error = xfs_setattr (vp, &attr, 0, cred);
-done:
-  XFSDEB(XDEBVNOPS, ("xfs_space: %d\n", error));
-  return error;
+    attr.va_mask = AT_SIZE;
+    attr.va_size = fl->l_start;
+    error = xfs_setattr (vp, &attr, 0, cred);
+ done:
+    XFSDEB(XDEBVNOPS, ("xfs_space: %d\n", error));
+    return error;
 }
 
 static int
 xfs_realvp(struct vnode *vp,
 	   struct vnode **vpp)
 {
-  XFSDEB(XDEBVNOPS, ("xfs_realvp\n"));
-  return ENOSYS;
+    XFSDEB(XDEBVNOPS, ("xfs_realvp\n"));
+    return ENOSYS;
 }
 
 static int
@@ -905,8 +941,23 @@ xfs_getpage(struct vnode *vp,
 	    enum seg_rw rw,
 	    struct cred *cred)
 {
-  XFSDEB(XDEBVNOPS, ("xfs_getpage\n"));
-  return ENOSYS;
+    struct vnode *t = DATA_FROM_VNODE(vp);
+    int error;
+
+    XFSDEB(XDEBVNOPS, ("xfs_getpage\n"));
+
+
+    if (vp->v_flag & VNOMAP)	/* File doesn't allow mapping */
+	return (ENOSYS);
+
+    error = xfs_data_valid(vp, cred, XFS_DATA_R);
+    if (error)
+	return error;
+
+    error = VOP_GETPAGE(t, off, len, protp, pl, plsz, seg, addr, rw, cred);
+
+    XFSDEB(XDEBVNOPS, ("xfs_getpage: return %d\n", error));
+    return error;
 }
 
 static int
@@ -916,8 +967,24 @@ xfs_putpage(struct vnode *vp,
 	    int flags,
 	    struct cred *cred)
 {
-  XFSDEB(XDEBVNOPS, ("xfs_putpage\n"));
-  return ENOSYS;
+    struct vnode *t = DATA_FROM_VNODE(vp);
+    int error;
+
+    XFSDEB(XDEBVNOPS, ("xfs_putpage\n"));
+
+    if (vp->v_flag & VNOMAP)	/* File doesn't allow mapping */
+	return (ENOSYS);
+
+    error = xfs_data_valid(vp, cred, XFS_DATA_W);
+    if (error)
+	return error;
+
+    VNODE_TO_XNODE(vp)->flags |= XFS_DATA_DIRTY;
+
+    error = VOP_PUTPAGE(t, off, len, flags, cred);
+
+    XFSDEB(XDEBVNOPS, ("xfs_getpage: return %d\n", error));
+    return error;
 }
 
 static int
@@ -931,46 +998,78 @@ xfs_map(struct vnode *vp,
 	uint_t flags,
 	struct cred *cred)
 {
-  int error = 0;
+    struct segvn_crargs vn_a;
+    int error = 0;
+    
+    XFSDEB(XDEBVNOPS,
+	   ("xfs_map"
+	    "(0x%x, 0x%lx, 0x%x, 0x%x, 0x%lx, 0x%x, 0x%x, 0x%x, 0x%x)\n",
+	    (int) vp, (unsigned long)off,
+	    (int) as, (int) addrp, (unsigned long)len, prot, maxprot,
+	    flags, (int) cred));
+    
+    
+    if (vp->v_flag & VNOMAP)
+	return ENOSYS;
 
-  XFSDEB(XDEBVNOPS,
-	 ("xfs_map(0x%x, 0x%lx, 0x%x, 0x%x, 0x%lx, 0x%x, 0x%x, 0x%x, 0x%x)\n",
-	  (int) vp, (unsigned long)off,
-	  (int) as, (int) addrp, (unsigned long)len, prot, maxprot,
-	  flags, (int) cred));
+    if (off < (offset_t)0 || (off + len) < (offset_t)0)
+	return EINVAL;
 
-  error = xfs_attr_valid (vp, cred, XFS_ATTR_R);
-  if (error)
-      goto out;
+    if (vp->v_type != VREG)
+	return ENODEV;
 
-  if ((prot & PROT_WRITE) && (flags & MAP_SHARED))
-    error = xfs_data_valid(vp, cred, XFS_DATA_W);
-  else
-    error = xfs_data_valid(vp, cred, XFS_DATA_R);
-  XFSDEB(XDEBVNOPS, ("xfs_map: size = %u\n",
-		     (unsigned)VNODE_TO_XNODE(vp)->attr.va_size));
+    if (vp->v_filocks != NULL)
+	return EAGAIN;
 
-  if (error != 0)
-    /* Can't map today */;
-  else if (off + len > VNODE_TO_XNODE(vp)->attr.va_size)
-    error = EINVAL;
+    error = xfs_attr_valid (vp, cred, XFS_ATTR_R);
+    if (error)
+	goto out;
+    
+    if ((prot & PROT_WRITE) && (flags & MAP_SHARED))
+	error = xfs_data_valid(vp, cred, XFS_DATA_W);
+    else
+	error = xfs_data_valid(vp, cred, XFS_DATA_R);
+    
+    XFSDEB(XDEBVNOPS, ("xfs_map: size = %u\n",
+		       (unsigned)VNODE_TO_XNODE(vp)->attr.va_size));
+    
+    if (error != 0)
+	/* Can't map today */;
 #if 0
-  else if ((prot & PROT_WRITE) && (flags & MAP_SHARED))
-    error = EROFS;		/* XXX This is currently not supported */
+    else if ((prot & PROT_WRITE) && (flags & MAP_SHARED))
+	error = EROFS;		/* XXX This is currently not supported */
 #endif
-  else
+    else
     {
-      struct vnode *t = DATA_FROM_VNODE(vp);
-      ASSERT(t != NULL);
-      VOP_RWLOCK(t, 1);
-      error = VOP_MAP(t, off, as, addrp, len, prot, maxprot, flags, cred);
-      /* XXX Patch vnode so that we can intercept get/putpage and inactive. */
-      VOP_RWUNLOCK(t, 1);
+	as_rangelock(as);
+	
+	
+	if ((flags & MAP_FIXED) == 0) {
+	    map_addr(addrp, len, off, 1, flags);
+	    if (*addrp == NULL) {
+		as_rangeunlock(as);
+		return ENOMEM;
+	    }
+	} else {
+	    as_unmap(as, *addrp, len);
+	}
+	
+	vn_a.vp = vp;
+	vn_a.offset = (u_offset_t)off;
+	vn_a.type = flags & MAP_TYPE;
+	vn_a.prot = prot;
+	vn_a.maxprot = maxprot;
+	vn_a.cred = cred;
+	vn_a.amp = NULL;
+	vn_a.flags = flags & ~MAP_TYPE;
+	
+	error = as_map(as, *addrp, len, segvn_create, &vn_a);
+	as_rangeunlock(as);
     }
-
-out:
-  XFSDEB(XDEBVNOPS, ("xfs_map: %d\n", error));
-  return error;
+    
+ out:
+    XFSDEB(XDEBVNOPS, ("xfs_map: %d\n", error));
+    return error;
 }
 
 static int
@@ -984,9 +1083,8 @@ xfs_addmap(struct vnode *vp,
 	   uint_t flags,
 	   struct cred *cred)
 {
-  XFSDEB(XDEBVNOPS, ("xfs_addmap\n"));
-  return ENOSYS;
-
+    XFSDEB(XDEBVNOPS, ("xfs_addmap\n"));
+    return 0;
 }
 
 static int
@@ -1000,9 +1098,19 @@ xfs_delmap(struct vnode *vp,
 	   uint_t flags,
 	   struct cred *cred)
 {
-  XFSDEB(XDEBVNOPS, ("xfs_delmap\n"));
-  return ENOSYS;
+    struct xfs *xfsp = XFS_FROM_VNODE(vp);
+    struct xfs_node *xn = VNODE_TO_XNODE(vp);
+    int error = 0;
 
+    XFSDEB(XDEBVNOPS, ("xfs_delmap\n"));
+
+    if (xn->flags & XFS_DATA_DIRTY) {
+	XFSDEB(XDEBVNOPS, ("xfs_delmap: data dirty\n"));
+	error = do_fsync (xfsp, xn, cred, XFS_WRITE | XFS_FSYNC);
+    }
+
+    XFSDEB(XDEBVNOPS, ("xfs_delmap: %d\n", error));
+    return error;
 }
 
 
@@ -1013,8 +1121,8 @@ xfs_poll(struct vnode *vp,
 	 short *revents,
 	 struct pollhead **ph)
 {
-  XFSDEB(XDEBVNOPS, ("xfs_poll\n"));
-  return fs_poll(vp, events, anyyet, revents, ph);
+    XFSDEB(XDEBVNOPS, ("xfs_poll\n"));
+    return fs_poll(vp, events, anyyet, revents, ph);
 }
 
 static int
@@ -1023,8 +1131,8 @@ xfs_dump(struct vnode *dumpvp,
 	 int bn,
 	 int count)
 {
-  XFSDEB(XDEBVNOPS, ("xfs_dump\n"));
-  return ENOSYS;
+    XFSDEB(XDEBVNOPS, ("xfs_dump\n"));
+    return ENOSYS;
 }
 
 static int
@@ -1033,8 +1141,8 @@ xfs_pathconf(struct vnode *vp,
 	     u_long *valp,
 	     struct cred *cred)
 {
-  XFSDEB(XDEBVNOPS, ("xfs_pathconf\n"));
-  return fs_pathconf (vp, cmd, valp, cred);
+    XFSDEB(XDEBVNOPS, ("xfs_pathconf\n"));
+    return fs_pathconf (vp, cmd, valp, cred);
 }
 
 static int
@@ -1049,16 +1157,16 @@ xfs_pageio(struct vnode *vp,
 	   int flags,
 	   struct cred *cred)
 {
-  XFSDEB(XDEBVNOPS, ("xfs_pageio\n"));
-  return ENOSYS;
+    XFSDEB(XDEBVNOPS, ("xfs_pageio\n"));
+    return ENOSYS;
 }
 
 static int
 xfs_dumpctl(struct vnode *vp,
 	    int flag)
 {
-  XFSDEB(XDEBVNOPS, ("xfs_dumpctl\n"));
-  return ENOSYS;
+    XFSDEB(XDEBVNOPS, ("xfs_dumpctl\n"));
+    return ENOSYS;
 }
 
 static void
@@ -1068,7 +1176,7 @@ xfs_dispose(struct vnode *vp,
 	    int b,
 	    struct cred *cred)
 {
-  XFSDEB(XDEBVNOPS, ("xfs_dispose\n"));
+    XFSDEB(XDEBVNOPS, ("xfs_dispose\n"));
 }
 
 static int
@@ -1077,8 +1185,8 @@ xfs_setsecattr(struct vnode *vp,
 	       int flag,
 	       struct cred *cred)
 {
-  XFSDEB(XDEBVNOPS, ("xfs_setsecattr\n"));
-  return ENOSYS;
+    XFSDEB(XDEBVNOPS, ("xfs_setsecattr\n"));
+    return ENOSYS;
 }
 
 static int
@@ -1087,52 +1195,52 @@ xfs_getsecattr(struct vnode *vp,
 	       int flag,
 	       struct cred *cred)
 {
-  XFSDEB(XDEBVNOPS, ("xfs_getsecattr\n"));
-  return fs_fab_acl(vp, attr, flag, cred);
+    XFSDEB(XDEBVNOPS, ("xfs_getsecattr\n"));
+    return fs_fab_acl(vp, attr, flag, cred);
 }
 
 struct vnodeops xfs_vnodeops = {
-        xfs_open,
-        xfs_close,
-        xfs_read,
-        xfs_write,
-        xfs_ioctl,
-	xfs_setfl,
-        xfs_getattr,
-        xfs_setattr,
-        xfs_access,
-        xfs_lookup,
-        xfs_create,
-        xfs_remove,
-        xfs_link,
-        xfs_rename,
-        xfs_mkdir,
-        xfs_rmdir,
-        xfs_readdir,
-        xfs_symlink,
-        xfs_readlink,
-        xfs_fsync,
-        xfs_inactive,
-	xfs_fid,
-	xfs_rwlock,
-	xfs_rwunlock,
-	xfs_seek,
-	xfs_cmp,
-	xfs_frlock,
-	xfs_space,
-	xfs_realvp,
-        xfs_getpage,
-        xfs_putpage,
-        xfs_map,
-	xfs_addmap,
-	xfs_delmap,
-	xfs_poll,
-        xfs_dump,
-	xfs_pathconf,
-	xfs_pageio,
-	xfs_dumpctl,
-	xfs_dispose,
-	xfs_setsecattr,
-	xfs_getsecattr
-	/* xfs_shrlock */
+    xfs_open,
+    xfs_close,
+    xfs_read,
+    xfs_write,
+    xfs_ioctl,
+    xfs_setfl,
+    xfs_getattr,
+    xfs_setattr,
+    xfs_access,
+    xfs_lookup,
+    xfs_create,
+    xfs_remove,
+    xfs_link,
+    xfs_rename,
+    xfs_mkdir,
+    xfs_rmdir,
+    xfs_readdir,
+    xfs_symlink,
+    xfs_readlink,
+    xfs_fsync,
+    xfs_inactive,
+    xfs_fid,
+    xfs_rwlock,
+    xfs_rwunlock,
+    xfs_seek,
+    xfs_cmp,
+    xfs_frlock,
+    xfs_space,
+    xfs_realvp,
+    xfs_getpage,
+    xfs_putpage,
+    xfs_map,
+    xfs_addmap,
+    xfs_delmap,
+    xfs_poll,
+    xfs_dump,
+    xfs_pathconf,
+    xfs_pageio,
+    xfs_dumpctl,
+    xfs_dispose,
+    xfs_setsecattr,
+    xfs_getsecattr
+    /* xfs_shrlock */
 };
