@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -50,7 +45,7 @@
 #include <xfs/xfs_dev.h>
 #include <xfs/xfs_syscalls.h>
 
-RCSID("$Id: xfs_wrap.c,v 1.6 1999/08/02 11:44:26 assar Exp $");
+RCSID("$Id: xfs_wrap.c,v 1.8 2000/10/02 23:53:44 lha Exp $");
 
 extern struct modlsys xfs_modlsys;
 
@@ -95,6 +90,9 @@ _fini(void)
 {
     int ret;
 
+    if (xfs_unloadable())
+	return EBUSY;
+
     ret = mod_remove(&xfs_modlinkage);
     if (ret)
 	return ret;
