@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,7 +33,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: output.c,v 1.58 1999/07/24 17:54:37 assar Exp $");
+RCSID("$Id: output.c,v 1.73 2000/11/25 22:38:20 lha Exp $");
 #endif
 
 #include <stdio.h>
@@ -58,6 +53,13 @@ RCSID("$Id: output.c,v 1.58 1999/07/24 17:54:37 assar Exp $");
 
 char *package = "";
 
+/*
+ * This is the list of packages so we know how to generate
+ * all Execute_package().
+ */
+
+List *packagelist = NULL;
+
 /*
  * Add this in front of the real functions implementing the server
  * functions called.
@@ -69,7 +71,7 @@ char *prefix = "";
  * File handles for the generated files themselves.
  */
 
-fileblob headerfile,
+ydr_file headerfile,
     clientfile,
     serverfile,
     clienthdrfile,
@@ -88,7 +90,12 @@ char *error_function = "";
 
 typedef enum { ENCODE_RX, DECODE_RX, ENCODE_MEM, DECODE_MEM } EncodeType;
 
-static void print_type (char *name, Type *type, FILE *f);
+typedef enum { CLIENT, SERVER } Side;
+
+typedef enum { FDECL, VDECL } DeclType;
+
+static void print_type (char *name, Type *type, enum argtype argtype, 
+			DeclType decl, FILE *f);
 static Bool print_entry (List *list, Listitem *item, void *i);
 static void generate_hdr_struct (Symbol *s, FILE *f);
 static void generate_hdr_enum (Symbol *s, FILE *f);
@@ -97,21 +104,22 @@ static void generate_hdr_typedef (Symbol *s, FILE *f);
 static int sizeof_type (Type *type);
 static int sizeof_symbol (Symbol *);
 static void encode_type (char *name, Type *type, FILE *f, 
-			 EncodeType encodetype);
-static void print_type (char *name, Type *type, FILE *f);
+			 EncodeType encodetype, Side side);
 static void display_type (char *where, char *name, Type *type, FILE *f);
 static Bool encode_entry (List *list, Listitem *item, void *arg);
 static void encode_struct (Symbol *s, char *name, FILE *f, 
-			   EncodeType encodetype);
-static void encode_enum (Symbol *s, char *name, FILE *f, EncodeType encodetype);
+			   EncodeType encodetype, Side side);
+static void encode_enum (Symbol *s, char *name, FILE *f,
+			 EncodeType encodetype, Side side);
 static void encode_typedef (Symbol *s, char *name, FILE *f, 
-			    EncodeType encodetype);
+			    EncodeType encodetype, Side side);
 static void encode_symbol (Symbol *s, char *name, FILE *f, 
-			   EncodeType encodetype);
+			   EncodeType encodetype, Side side);
 static void print_symbol (char *where, Symbol *s, char *name, FILE *f);
 
 static void
-print_type (char *name, Type *type, FILE *f)
+print_type (char *name, Type *type, enum argtype argtype, 
+	    DeclType decl, FILE *f)
 {
      switch (type->type) {
 	  case TCHAR :
@@ -133,8 +141,10 @@ print_type (char *name, Type *type, FILE *f)
 	       fprintf (f, "u_int32_t %s", name);
 	       break;
 	  case TSTRING :
-	       if (type->size)
+	       if (type->size && decl == VDECL)
 		   fprintf (f, "char %s[%d]", name, type->size);
+	       else if (argtype != TIN && type->size == 0)
+		   fprintf (f, "char **%s", name);
 	       else
 		   fprintf (f, "char *%s", name);
 	       break;
@@ -145,7 +155,7 @@ print_type (char *name, Type *type, FILE *f)
 	       tmp = (char *)emalloc (strlen(name) + 2);
 	       *tmp = '*';
 	       strcpy (tmp+1, name);
-	       print_type (tmp, type->subtype, f);
+	       print_type (tmp, type->subtype, argtype, decl, f);
 	       free (tmp);
 	       break;
 	  }
@@ -166,18 +176,18 @@ print_type (char *name, Type *type, FILE *f)
 
 	       fprintf (f, "struct {\n");
 	       if (type->indextype)
-		    print_type ("len", type->indextype, f);
+		    print_type ("len", type->indextype, argtype, decl, f);
 	       else
 		    fprintf (f, "unsigned %s", "len");
 	       fprintf (f, ";\n");
 	       strcpy(s + strlen(s) - 3, "val");
-	       print_type ("*val", type->subtype, f);
+	       print_type ("*val", type->subtype, argtype, decl, f);
 	       fprintf (f, ";\n} %s", name);
 	       free(s);
 	       break;
 	  }
 	  case TARRAY :
-	       print_type (name, type->subtype, f);
+	       print_type (name, type->subtype, argtype, decl, f);
 	       fprintf (f, "[ %d ]", type->size);
 	       break;
 	  case TOPAQUE :
@@ -195,7 +205,7 @@ print_entry (List *list, Listitem *item, void *i)
      FILE *f = (FILE *)i;
 
      fprintf (f, "     ");
-     print_type (s->name, s->type, f);
+     print_type (s->name, s->type, TIN, VDECL, f);
      fprintf (f, ";\n");
      return FALSE;
 }
@@ -301,7 +311,8 @@ static void
 generate_hdr_struct (Symbol *s, FILE *f)
 {
      fprintf (f, "struct %s {\n", s->name);
-     listiter (s->u.list, print_entry, f);
+     if (s->u.list)
+	 listiter (s->u.list, print_entry, f);
      fprintf (f, "};\ntypedef struct %s %s;\n", s->name, s->name);
 }
 
@@ -334,7 +345,7 @@ static void
 generate_hdr_typedef (Symbol *s, FILE *f)
 {
      fprintf (f, "typedef ");
-     print_type (s->name, s->u.type, f);
+     print_type (s->name, s->u.type, TIN, VDECL, f);
      fprintf (f, ";\n");
 }
 
@@ -563,9 +574,16 @@ encode_short (char *name, Type *type, FILE *f, EncodeType encodetype)
  */
 
 static void
-encode_string (char *name, Type *type, FILE *f, EncodeType encodetype)
+encode_string (char *name, Type *type, FILE *f, EncodeType encodetype,
+	       Side side)
 {
      Type lentype = {TULONG};
+     char *nname;
+
+     asprintf (&nname, "(%s%s)",
+	       ((type->size == 0) && side == CLIENT 
+		&& (encodetype == ENCODE_RX || encodetype == ENCODE_MEM))
+	       ? "*" : "", name);
 
      switch (encodetype) {
 	  case ENCODE_RX :
@@ -575,7 +593,7 @@ encode_string (char *name, Type *type, FILE *f, EncodeType encodetype)
 			"len = strlen(%s);\n"
 			"padlen = (4 - (len %% 4)) %% 4;\n",
 			name);
-	       encode_type ("len", &lentype, f, encodetype);
+	       encode_type ("len", &lentype, f, encodetype, side);
 	       fprintf (f,
 			"if(rx_Write(call, %s, len) != len)\n"
 			"goto fail;\n"
@@ -587,14 +605,16 @@ encode_string (char *name, Type *type, FILE *f, EncodeType encodetype)
 	       fprintf (f, "{ unsigned len;\n"
 			"unsigned padlen;\n"
 			"char zero[4] = {0, 0, 0, 0};\n");
-	       encode_type ("len", &lentype, f, encodetype);
+	       encode_type ("len", &lentype, f, encodetype, side);
 	       if (type->size != 0) {
 		   fprintf (f,
-			    "if (len >= %u)\n"
-			    "abort();\n",
+			    "if (len >= %u) {\n"
+			    "rx_Error(call) = ENOMEM;\n"
+			    "goto fail;\n"
+			    "}\n",
 			    type->size);
 	       } else {
-		   fprintf(f, "%s = malloc(len + 1);\n", name);
+		   fprintf(f, "%s = malloc(len + 1);\n", nname);
 	       }
 
 	       fprintf (f, 
@@ -604,7 +624,7 @@ encode_string (char *name, Type *type, FILE *f, EncodeType encodetype)
 			"padlen = (4 - (len %% 4)) %% 4;\n"
 			"if(rx_Read(call, zero, padlen) != padlen)\n"
 			"goto fail;\n"
-			"}\n", name, name);
+			"}\n", nname, nname);
 	       break;
 	  case ENCODE_MEM :
 	       fprintf (f,
@@ -612,7 +632,7 @@ encode_string (char *name, Type *type, FILE *f, EncodeType encodetype)
 			"if (*total_len < len) goto fail;\n"
 			"*total_len -= len;\n",
 			name);
-	       encode_type ("len", &lentype, f, encodetype);
+	       encode_type ("len", &lentype, f, encodetype, side);
 	       fprintf (f, "strncpy (ptr, %s, len);\n", name);
 	       fprintf (f, "ptr += len + (4 - (len %% 4) %% 4);\n"
 			"*total_len -= len + (4 - (len %% 4) %% 4);\n}\n");
@@ -620,28 +640,29 @@ encode_string (char *name, Type *type, FILE *f, EncodeType encodetype)
 	  case DECODE_MEM :
 	       fprintf (f,
 		   "{\nunsigned len;\n");
-	       encode_type ("len", &lentype, f, encodetype);
+	       encode_type ("len", &lentype, f, encodetype, side);
 	       fprintf (f,
 			"if (*total_len < len) goto fail;\n"
 			"*total_len -= len;\n");
 	       if (type->size != 0) {
 		   fprintf (f,
 			    "if(len >= %u)\n"
-			    "abort();\n",
+			    "goto fail;\n",
 			    type->size);
 	       } else {
-		   fprintf (f, "%s = malloc(len + 1);\n", name);
+		   fprintf (f, "%s = malloc(len + 1);\n", nname);
 	       }
 	       fprintf (f,
 			"memcpy (%s, ptr, len);\n"
 			"%s[len] = '\\0';\n"
 			"ptr += len + (4 - (len %% 4)) %% 4;\n"
 			"*total_len -= len + (4 - (len %% 4) %% 4);\n}\n",
-			name, name);
+			nname, nname);
 	       break;
 	  default :
 	       abort ();
      }
+     free (nname);
 }	       
 
 /*
@@ -660,7 +681,8 @@ print_string (char *where, char *name, Type *type, FILE *f)
  */
 
 static void
-encode_array (char *name, Type *type, FILE *f, EncodeType encodetype)
+encode_array (char *name, Type *type, FILE *f, EncodeType encodetype,
+	      Side side)
 {
      if (type->subtype->type == TOPAQUE) {
 	  if (type->size % 4 != 0)
@@ -703,7 +725,9 @@ encode_array (char *name, Type *type, FILE *f, EncodeType encodetype)
 		   "++i%lu){\n", tmpcnt, tmpcnt, tmpcnt, type->size,tmpcnt);
 	  snprintf(tmp, sizeof(tmp)-1, "%s[i%lu]", name, tmpcnt);
 	  tmpcnt++;
-	  encode_type (tmp , type->subtype, f, encodetype);
+	  if (type->flags)
+	      type->subtype->flags |= type->flags;
+	  encode_type (tmp , type->subtype, f, encodetype, side);
 	  tmpcnt--;
 	  fprintf (f, "}\n}\n");
      }
@@ -754,7 +778,8 @@ print_array (char *where, char *name, Type *type, FILE *f)
  */
 
 static void
-encode_varray (char *name, Type *type, FILE *f, EncodeType encodetype)
+encode_varray (char *name, Type *type, FILE *f, EncodeType encodetype,
+	       Side side)
 {
      char tmp[256];
      Type lentype = {TULONG};
@@ -763,12 +788,12 @@ encode_varray (char *name, Type *type, FILE *f, EncodeType encodetype)
      strcat (tmp, ".len");
 
      encode_type (tmp, type->indextype ? type->indextype : &lentype,
-		  f, encodetype);
+		  f, encodetype, side);
      if (encodetype == DECODE_MEM || encodetype == DECODE_RX) {
 	 fprintf (f, "%s.val = (", name);
-	 print_type ("*", type->subtype, f);
+	 print_type ("*", type->subtype, TIN, VDECL, f);
 	 fprintf (f, ")malloc(sizeof(");
-	 print_type ("", type->subtype, f);
+	 print_type ("", type->subtype, TIN, VDECL, f);
 	 fprintf (f, ") * %s);\n", tmp);
      }
      if (type->subtype->type == TOPAQUE) {
@@ -821,7 +846,9 @@ encode_varray (char *name, Type *type, FILE *f, EncodeType encodetype)
 		   "++i%lu){\n", tmpcnt, tmpcnt, tmpcnt, tmp, tmpcnt);
 	  snprintf(tmp, sizeof(tmp)-1, "%s.val[i%lu]", name, tmpcnt);
 	  tmpcnt++;
-	  encode_type (tmp , type->subtype, f, encodetype);
+	  if (type->flags)
+	      type->subtype->flags |= type->flags;
+	  encode_type (tmp , type->subtype, f, encodetype, side);
 	  tmpcnt--;
 	  fprintf (f, "}\n}\n");
      }
@@ -867,7 +894,8 @@ print_varray (char *where, char *name, Type *type, FILE *f)
  */
 
 static void
-encode_pointer (char *name, Type *type, FILE *f, EncodeType encodetype)
+encode_pointer (char *name, Type *type, FILE *f, EncodeType encodetype,
+		Side side)
 {
      Type booltype = {TULONG};
      char tmp[256];
@@ -880,9 +908,9 @@ encode_pointer (char *name, Type *type, FILE *f, EncodeType encodetype)
      case ENCODE_MEM:
 	  fprintf(f, "{ unsigned bool;\n"
 		  "bool = %s != NULL;\n", name);
-	  encode_type ("bool", &booltype, f, encodetype);
+	  encode_type ("bool", &booltype, f, encodetype, side);
 	  fprintf (f, "if(%s) {\n", name);
-	  encode_type (tmp, type->subtype, f, encodetype);
+	  encode_type (tmp, type->subtype, f, encodetype, side);
 	  fprintf (f, "}\n"
 		   "}\n");
 	  break;
@@ -890,12 +918,12 @@ encode_pointer (char *name, Type *type, FILE *f, EncodeType encodetype)
 	  abort();
      case DECODE_MEM:
 	  fprintf(f, "{ unsigned bool;\n");
-	  encode_type ("bool", &booltype, f, encodetype);
+	  encode_type ("bool", &booltype, f, encodetype, side);
 	  fprintf (f, "if(bool) {\n");
 	  fprintf (f, "%s = malloc(sizeof(%s));\n"
 		   "if (%s == NULL) return ENOMEM;\n", 
 		   name, tmp, name);
-	  encode_type (tmp, type->subtype, f, encodetype);
+	  encode_type (tmp, type->subtype, f, encodetype, side);
 	  fprintf (f, "} else {\n"
 		   "%s = NULL;\n"
 		   "}\n"
@@ -911,7 +939,8 @@ encode_pointer (char *name, Type *type, FILE *f, EncodeType encodetype)
  */
 
 static void
-encode_type (char *name, Type *type, FILE *f, EncodeType encodetype)
+encode_type (char *name, Type *type, FILE *f, EncodeType encodetype,
+	     Side side)
 {
      switch (type->type) {
 	  case TCHAR :
@@ -931,23 +960,23 @@ encode_type (char *name, Type *type, FILE *f, EncodeType encodetype)
 	       encode_long (name, type, f, encodetype);
 	       break;
 	  case TSTRING :
-	       encode_string (name, type, f, encodetype);
+	       encode_string (name, type, f, encodetype, side);
 	       break;
 	  case TOPAQUE :
 	       error_message (1,
 			      "Type opaque only allowed as part of an array");
 	       break;
 	  case TUSERDEF :
-	       encode_symbol (type->symbol, name, f, encodetype);
+	       encode_symbol (type->symbol, name, f, encodetype, side);
 	       break;
 	  case TARRAY :
-	       encode_array (name, type, f, encodetype);
+	       encode_array (name, type, f, encodetype, side);
 	       break;
 	  case TVARRAY :
-	       encode_varray (name, type, f, encodetype);
+	       encode_varray (name, type, f, encodetype, side);
 	       break;
 	  case TPOINTER :
-	       encode_pointer (name, type, f, encodetype);
+	       encode_pointer (name, type, f, encodetype, side);
 	       break;
 	  default :
 	       abort();
@@ -1008,6 +1037,7 @@ struct context {
      FILE *f;
      Symbol *symbol;
      EncodeType encodetype;
+     Side side;
 };
 
 /*
@@ -1035,7 +1065,8 @@ encode_entry (List *list, Listitem *item, void *arg)
 		  context->symbol->name,
 		  tmp);
      } else {
-	 encode_type (tmp, s->type, context->f, context->encodetype);
+	 encode_type (tmp, s->type, context->f, context->encodetype,
+		      context->side);
      }
 
      return FALSE;
@@ -1046,7 +1077,8 @@ encode_entry (List *list, Listitem *item, void *arg)
  */
 
 static void
-encode_struct (Symbol *s, char *name, FILE *f, EncodeType encodetype)
+encode_struct (Symbol *s, char *name, FILE *f, EncodeType encodetype,
+	       Side side)
 {
      struct context context;
 
@@ -1054,6 +1086,7 @@ encode_struct (Symbol *s, char *name, FILE *f, EncodeType encodetype)
      context.symbol     = s;
      context.f          = f;
      context.encodetype = encodetype;
+     context.side       = side;
 
      listiter (s->u.list, encode_entry, (void *)&context);
 }
@@ -1128,15 +1161,12 @@ print_struct (char *where, Symbol *s, char *name, FILE *f)
  */
 
 static void
-encode_enum (Symbol *s, char *name, FILE *f, EncodeType encodetype)
+encode_enum (Symbol *s, char *name, FILE *f, EncodeType encodetype,
+	     Side side)
 {
      Type type = {TLONG};
-     char tmp[256];
 
-     strcpy (tmp, "(int)");
-     strcat (tmp, name);
-
-     encode_type (tmp, &type, f, encodetype);
+     encode_type (name, &type, f, encodetype, side);
 }
 
 /*
@@ -1177,9 +1207,10 @@ print_enum (char *where, Symbol *s, char *name, FILE *f)
  */
 
 static void
-encode_typedef (Symbol *s, char *name, FILE *f, EncodeType encodetype)
+encode_typedef (Symbol *s, char *name, FILE *f, EncodeType encodetype,
+		Side side)
 {
-     encode_type (name, s->u.type, f, encodetype);
+     encode_type (name, s->u.type, f, encodetype, side);
 }
 
 /*
@@ -1197,17 +1228,18 @@ print_typedef (char *where, Symbol *s, char *name, FILE *f)
  */
 
 static void
-encode_symbol (Symbol *s, char *name, FILE *f, EncodeType encodetype)
+encode_symbol (Symbol *s, char *name, FILE *f, EncodeType encodetype,
+	       Side side)
 {
      switch (s->type) {
 	  case TSTRUCT :
-	       encode_struct (s, name, f, encodetype);
+	       encode_struct (s, name, f, encodetype, side);
 	       break;
 	  case TENUM :
-	       encode_enum (s, name, f, encodetype);
+	       encode_enum (s, name, f, encodetype, side);
 	       break;
 	  case TTYPEDEF :
-	       encode_typedef (s, name, f, encodetype);
+	       encode_typedef (s, name, f, encodetype, side);
 	       break;
 	  default :
 	       abort();
@@ -1285,7 +1317,8 @@ generate_function (Symbol *s, FILE *f, Bool encodep)
      if (s->type == TSTRUCT || s->type == TENUM || s->type == TTYPEDEF) {
 	  generate_function_definition (s, f, encodep);
 	  fprintf (f, "\n{\n");
-	  encode_symbol (s, "(*o)", f, encodep ? ENCODE_MEM : DECODE_MEM);
+	  encode_symbol (s, "(*o)", f,
+			 encodep ? ENCODE_MEM : DECODE_MEM, CLIENT);
 	  fprintf (f, "return ptr;\n"
 		   "fail:\n"
 		   "errno = EFAULT;\n"
@@ -1368,7 +1401,7 @@ gen1 (List *list, Listitem *item, void *arg)
      fprintf (f, ", ");
      if (a->argtype == TIN)
 	 fprintf (f, "const ");
-     print_type (a->name, a->type, f);
+     print_type (a->name, a->type, a->argtype, FDECL, f);
      fprintf (f, "\n");
      return FALSE;
 }
@@ -1380,8 +1413,8 @@ genin (List *list, Listitem *item, void *arg)
      FILE *f = (FILE *)arg;
 
      if (a->argtype == TIN || a->argtype == TINOUT) {
-	  fprintf (f, ", ");
-	  print_type (a->name, a->type, f);
+	  fprintf (f, ", %s ", a->argtype == TIN ? "const" : "");
+	  print_type (a->name, a->type, a->argtype, FDECL, f);
 	  fprintf (f, "\n");
      }
      return FALSE;
@@ -1405,9 +1438,9 @@ gendeclare (List *list, Listitem *item, void *arg)
      FILE *f = (FILE *)arg;
 
      if (a->type->type == TPOINTER)
-	  print_type (a->name, a->type->subtype, f);
+	  print_type (a->name, a->type->subtype, TIN, VDECL, f);
      else
-	  print_type (a->name, a->type, f);
+	  print_type (a->name, a->type, TIN, VDECL, f);
      fprintf (f, ";\n");
      return FALSE;
 }
@@ -1436,7 +1469,9 @@ genfree (List *list, Listitem *item, void *arg)
 
      if (genfree_isarrayp(a->type))
 	 fprintf(f, "free(%s.val);\n", a->name);
-
+     else if (a->argtype != TIN
+	      && a->type->type == TSTRING && a->type->size == 0)
+	 fprintf (f, "free(%s);\n", a->name);
      return FALSE;
 }
 
@@ -1454,10 +1489,10 @@ genencodein (List *list, Listitem *item, void *arg)
 	       
 	       sprintf (tmp, "(*%s)", a->name);
 
-	       encode_type (tmp, a->type->subtype, f, ENCODE_RX);
+	       encode_type (tmp, a->type->subtype, f, ENCODE_RX, CLIENT);
 	       free (tmp);
 	  } else
-	       encode_type (a->name, a->type, f, ENCODE_RX);
+	       encode_type (a->name, a->type, f, ENCODE_RX, CLIENT);
 	  return FALSE;
      }
 }
@@ -1474,10 +1509,10 @@ gendecodeout (List *list, Listitem *item, void *arg)
 	       
 	     sprintf (tmp, "(*%s)", a->name);
 
-	     encode_type (tmp, a->type->subtype, f, DECODE_RX);
+	     encode_type (tmp, a->type->subtype, f, DECODE_RX, CLIENT);
 	     free (tmp);
 	 } else if(a->type->type == TSTRING) {
-	     encode_type (a->name, a->type, f, DECODE_RX);
+	     encode_type (a->name, a->type, f, DECODE_RX, CLIENT);
 	 }
      }
      return FALSE;
@@ -1498,12 +1533,12 @@ gendecodein (List *list, Listitem *item, void *arg)
 	       
 	       sprintf (tmp, "(*%s)", a->name);
 
-	       encode_type (tmp, a->type->subtype, f, DECODE_RX);
+	       encode_type (tmp, a->type->subtype, f, DECODE_RX, SERVER);
 	       free (tmp);
 #endif
-	       encode_type (a->name, a->type->subtype, f, DECODE_RX);
+	       encode_type (a->name, a->type->subtype, f, DECODE_RX, SERVER);
 	  } else
-	       encode_type (a->name, a->type, f, DECODE_RX);
+	       encode_type (a->name, a->type, f, DECODE_RX, SERVER);
 	  return FALSE;
      }
 }
@@ -1516,9 +1551,9 @@ genencodeout (List *list, Listitem *item, void *arg)
 
      if (a->argtype == TOUT || a->argtype == TINOUT) {
 	  if (a->type->type == TPOINTER)
-	       encode_type (a->name, a->type->subtype, f, ENCODE_RX);
+	       encode_type (a->name, a->type->subtype, f, ENCODE_RX, SERVER);
 	  else
-	       encode_type (a->name, a->type, f, ENCODE_RX);
+	       encode_type (a->name, a->type, f, ENCODE_RX, SERVER);
      }
      return FALSE;
 }
@@ -1552,7 +1587,9 @@ genargs (List *list, Listitem *item, void *arg)
      Argument *a = (Argument *)listdata (item);
      FILE *f = (FILE *)arg;
 
-     if (a->type->type == TPOINTER)
+     if (a->type->type == TPOINTER
+	 || (a->argtype != TIN
+	     && a->type->type == TSTRING && a->type->size == 0))
 	  putc ('&', f);
      fputs (a->name, f);
      if (listnext (list, item))
@@ -1585,7 +1622,7 @@ generate_simple_stub (Symbol *s, FILE *f, FILE *headerf)
 
      asprintf (&op, "%u", s->u.proc.id);
      
-     encode_type (op, &type, f, ENCODE_RX);
+     encode_type (op, &type, f, ENCODE_RX, CLIENT);
      free (op);
      listiter (s->u.proc.arguments, genencodein, f);
      listiter (s->u.proc.arguments, gendecodeout, f);
@@ -1618,7 +1655,7 @@ generate_split_stub (Symbol *s, FILE *f, FILE *headerf)
 
 
      asprintf (&op, "%u", s->u.proc.id);
-     encode_type (op, &type, f, ENCODE_RX);
+     encode_type (op, &type, f, ENCODE_RX, CLIENT);
      free (op);
      listiter (s->u.proc.arguments, genencodein, f);
      fprintf (f, "return 0;\n");
@@ -1731,7 +1768,7 @@ generate_standard_c_prologue (FILE *f,
 	      "#define bcopy(a,b,c) memcpy((b),(a),(c))\n"
 	      "#endif /* !HAVE_BCOPY */\n\n");
      fprintf (f, "#ifdef RCSID\n"
-	      "RCSID(\"%s generated from %s.xg with $Id: output.c,v 1.58 1999/07/24 17:54:37 assar Exp $\");\n"
+	      "RCSID(\"%s generated from %s.xg with $Id: output.c,v 1.73 2000/11/25 22:38:20 lha Exp $\");\n"
 	      "#endif\n\n", filename, basename);
 }
 
@@ -1750,7 +1787,7 @@ init_generate (const char *filename)
      asprintf (&tmp, "%s.h", filename);
      if (tmp == NULL)
 	 err (1, "malloc");
-     eefopen (tmp, "w", &headerfile);
+     ydr_fopen (tmp, "w", &headerfile);
      free (tmp);
 
      fileuppr = estrdup (filename);
@@ -1764,14 +1801,14 @@ init_generate (const char *filename)
      asprintf (&tmp, "%s.ydr.c", filename);
      if (tmp == NULL)
 	 err (1, "malloc");
-     eefopen (tmp, "w", &ydrfile);
+     ydr_fopen (tmp, "w", &ydrfile);
      generate_standard_c_prologue (ydrfile.stream, tmp, filename);
      free (tmp);
 
      asprintf (&tmp, "%s.cs.c", filename);
      if (tmp == NULL)
 	 err (1, "malloc");
-     eefopen (tmp, "w", &clientfile);
+     ydr_fopen (tmp, "w", &clientfile);
      generate_standard_c_prologue (clientfile.stream, tmp, filename);
      fprintf (clientfile.stream, "#include \"%s.cs.h\"\n\n", filename);
      free (tmp);
@@ -1779,7 +1816,7 @@ init_generate (const char *filename)
      asprintf (&tmp, "%s.ss.c", filename);
      if (tmp == NULL)
 	 err (1, "malloc");
-     eefopen (tmp, "w", &serverfile);
+     ydr_fopen (tmp, "w", &serverfile);
      generate_standard_c_prologue (serverfile.stream, tmp, filename);
      fprintf (serverfile.stream, "#include \"%s.ss.h\"\n\n", filename);
      free (tmp);
@@ -1787,7 +1824,7 @@ init_generate (const char *filename)
      asprintf (&tmp, "%s.cs.h", filename);
      if (tmp == NULL)
 	 err (1, "malloc");
-     eefopen (tmp, "w", &clienthdrfile);
+     ydr_fopen (tmp, "w", &clienthdrfile);
      free (tmp);
      fprintf (clienthdrfile.stream, "/* Generated from %s.xg */\n", filename);
      fprintf (clienthdrfile.stream, "#include <rx/rx.h>\n");
@@ -1796,7 +1833,7 @@ init_generate (const char *filename)
      asprintf (&tmp, "%s.ss.h", filename);
      if (tmp == NULL)
 	 err (1, "malloc");
-     eefopen (tmp, "w", &serverhdrfile);
+     ydr_fopen (tmp, "w", &serverhdrfile);
      free (tmp);
      fprintf (serverhdrfile.stream, "/* Generated from %s.xg */\n", filename);
      fprintf (serverhdrfile.stream, "#include <rx/rx.h>\n");
@@ -1805,8 +1842,12 @@ init_generate (const char *filename)
      asprintf (&tmp, "%s.td.c", filename);
      if (tmp == NULL)
 	 err (1, "malloc");
-     eefopen (tmp, "w", &td_file);
+     ydr_fopen (tmp, "w", &td_file);
      free (tmp);
+
+     packagelist = listnew();
+     if (packagelist == NULL)
+	 err (1, "init_generate: listnew: packagelist");
 }
 
 void
@@ -1816,13 +1857,13 @@ close_generator (const char *filename)
 	  
      strupr (fileupr);
      fprintf (headerfile.stream, "\n#endif /* %s */\n", fileupr);
-     eefclose (&headerfile);
-     eefclose (&clientfile);
-     eefclose (&serverfile);
-     eefclose (&clienthdrfile);
-     eefclose (&serverhdrfile);
-     eefclose (&ydrfile);
-     eefclose (&td_file);
+     ydr_fclose (&headerfile);
+     ydr_fclose (&clientfile);
+     ydr_fclose (&serverfile);
+     ydr_fclose (&clienthdrfile);
+     ydr_fclose (&serverhdrfile);
+     ydr_fclose (&ydrfile);
+     ydr_fclose (&td_file);
 }
 
 /*
@@ -1843,7 +1884,7 @@ generate_server_stub (Symbol *s, FILE *f, FILE *headerf, FILE *h_file)
      fprintf (f, "int _%s%s(\nstruct rx_call *call)\n",
 	      package, s->name);
      fprintf (f, "{\n"
-	      "int _result;\n");
+	      "int32_t _result;\n");
      listiter (s->u.proc.arguments, gendeclare, f);
      fprintf (f, "\n");
      listiter (s->u.proc.arguments, gendecodein, f);
@@ -1874,17 +1915,25 @@ generate_server_stub (Symbol *s, FILE *f, FILE *headerf, FILE *h_file)
 	 generate_multi (s, h_file);
 }
 
+struct gencase_context {
+    FILE *f;
+    char *package;
+};
+
 static Bool
 gencase (List *list, Listitem *item, void *arg)
 {
      Symbol *s = (Symbol *)listdata (item);
-     FILE *f = (FILE *)arg;
+     struct gencase_context *c = (struct gencase_context *)arg;
+     FILE *f = c->f;
 
-     fprintf (f, "case %u: {\n"
-	      "_result = _%s%s(call);\n"
-	      "break;\n"
-	      "}\n",
-	      s->u.proc.id, package, s->name);
+     if (c->package == s->u.proc.package) {
+	 fprintf (f, "case %u: {\n"
+		  "_result = _%s%s(call);\n"
+		  "break;\n"
+		  "}\n",
+		  s->u.proc.id, s->u.proc.package, s->name);
+     }
      return FALSE;
 }
 
@@ -1897,29 +1946,42 @@ generate_server_switch (FILE *c_file,
 			FILE *h_file)
 {
      Type optype = {TULONG};
+     Listitem *li;
+     struct gencase_context c;
+
+     c.f = c_file;
      
-     fprintf (h_file,
-	      "int %sExecuteRequest(struct rx_call *call);\n",
-	      package);
+     li = listhead (packagelist);
+     while (li) {
+	 c.package = (char *)listdata (li);
 
-     fprintf (c_file, "int %sExecuteRequest(struct rx_call *call)\n"
-	      "{\n"
-	      "unsigned opcode;\n"
-	      "int _result;\n",
-	      package);
+	 fprintf (h_file,
+		  "int32_t %sExecuteRequest(struct rx_call *call);\n",
+		  c.package);
+	 
+	 fprintf (c_file,
+		  "int32_t %sExecuteRequest(struct rx_call *call)\n"
+		  "{\n"
+		  "unsigned opcode;\n"
+		  "int32_t _result;\n",
+		  c.package);
+	 
+	 encode_type ("opcode", &optype, c_file, DECODE_RX, SERVER);
+	 fprintf (c_file, "switch(opcode) {\n");
+	 
+	 listiter (func_list, gencase, &c);
+	 
+	 fprintf (c_file, "default:\n"
+ 		  "_result = RXGEN_OPCODE;\n"
+		  "fprintf (stderr, \"Ignoring %%d\\n\", opcode);\n"
+		  "}\n"
+		  "return _result;\n"
+		  "fail:\n"
+		  "return rx_Error(call);\n"
+		  "}\n\n");
 
-     encode_type ("opcode", &optype, c_file, DECODE_RX);
-     fprintf (c_file, "switch(opcode) {\n");
-
-     listiter (func_list, gencase, c_file);
-
-     fprintf (c_file, "default:\n"
-	      "fprintf (stderr, \"Ignoring %%d\\n\", opcode);\n"
-	      "}\n"
-	      "return _result;\n"
-	      "fail:\n"
-	      "return rx_Error(call);\n"
-	      "}\n\n");
+	 li = listnext (packagelist, li);
+     }
 }
 
 static Bool
@@ -1929,9 +1991,9 @@ gentcpdump_decode (List *list, Listitem *item, void *arg)
      FILE *f = (FILE *)arg;
 
      if (a->type->type == TPOINTER)
-	 encode_type (a->name, a->type->subtype, f, ENCODE_MEM);
+	 encode_type (a->name, a->type->subtype, f, ENCODE_MEM, CLIENT);
      else
-	 encode_type (a->name, a->type, f, ENCODE_MEM);
+	 encode_type (a->name, a->type, f, ENCODE_MEM, CLIENT);
 
      fprintf (f, "++found;\n");
      return FALSE;
@@ -2023,7 +2085,7 @@ generate_tcpdump_patches(FILE *td_file, const char *filename)
 {
     Type optype = {TULONG};
 
-    fprintf(td_file, "/* generated by $Id: output.c,v 1.58 1999/07/24 17:54:37 assar Exp $ */\n\n");
+    fprintf(td_file, "/* generated by $Id: output.c,v 1.73 2000/11/25 22:38:20 lha Exp $ */\n\n");
 
     fprintf (td_file, 
 	     "#include <stdio.h>\n"
@@ -2040,7 +2102,7 @@ generate_tcpdump_patches(FILE *td_file, const char *filename)
 	     "char *ret;\n",
 	     package);
     
-    encode_type ("opcode", &optype, td_file, DECODE_MEM);
+    encode_type ("opcode", &optype, td_file, DECODE_MEM, CLIENT);
 
     fprintf (td_file, "switch(opcode) {\n");
 
@@ -2056,4 +2118,31 @@ generate_tcpdump_patches(FILE *td_file, const char *filename)
 
 }
 
+void
+ydr_fopen (const char *name, const char *mode, ydr_file *f)
+{
+     int streamfd;
 
+     asprintf (&f->curname, "%sXXXXXX", name);
+     if (f->curname == NULL)
+	 err (1, "malloc");
+
+     streamfd = mkstemp(f->curname);
+     if (streamfd < 0)
+	 err (1, "mkstemp %s failed", f->curname);
+     f->stream = fdopen (streamfd, mode);
+     if (f->stream == NULL)
+	 err (1, "open %s mode %s", f->curname, mode);
+     f->newname = estrdup(name);
+}
+
+void
+ydr_fclose (ydr_file *f)
+{
+     if (fclose (f->stream))
+	 err (1, "close %s", f->curname);
+     if (rename(f->curname, f->newname))
+	 err (1, "rename %s, %s", f->curname, f->newname);
+     free(f->curname);
+     free(f->newname);
+}
